import { Project, ProjectOptions } from "projen";
import { DocumentationFormat, Language } from "./languages";
import { TypeSafeApiModelProject } from "./model/type-safe-api-model-project";
import { GeneratedCodeOptions, GeneratedCodeProjects, ModelLanguage, ModelOptions } from "./types";
/**
 * Configuration for modelling the API
 */
export interface ModelConfiguration {
    /**
     * The language the API model is defined in.
     */
    readonly language: ModelLanguage;
    /**
     * Options for the API model.
     */
    readonly options: ModelOptions;
}
/**
 * Configuration for generated runtime projects
 */
export interface RuntimeConfiguration {
    /**
     * The languages that runtime projects will be generated in. These projects can be used to provide type safety for
     * both client and server projects.
     */
    readonly languages: Language[];
    /**
     * Options for the generated runtimes. Note that only options provided for the specified languages will apply.
     */
    readonly options?: GeneratedCodeOptions;
}
/**
 * Configuration for generated infrastructure
 */
export interface InfrastructureConfiguration {
    /**
     * The language to generate the type-safe CDK infrastructure in
     */
    readonly language: Language;
    /**
     * Options for the infrastructure package. Note that only those provided for the specified language will apply.
     */
    readonly options?: GeneratedCodeOptions;
}
/**
 * Configuration for generated documentation
 */
export interface DocumentationConfiguration {
    /**
     * Formats for generated documentation
     */
    readonly formats: DocumentationFormat[];
}
/**
 * Options for the TypeSafeApiProject
 */
export interface TypeSafeApiProjectOptions extends ProjectOptions {
    /**
     * Configuration for the API model
     */
    readonly model: ModelConfiguration;
    /**
     * Configuration for generated runtime projects (containing types, clients and server code)
     */
    readonly runtime: RuntimeConfiguration;
    /**
     * Configuration for generated infrastructure
     */
    readonly infrastructure: InfrastructureConfiguration;
    /**
     * Configuration for generated documentation
     */
    readonly documentation?: DocumentationConfiguration;
}
/**
 * Project for a type-safe API, defined using Smithy or OpenAPI.
 *
 * Generates a CDK construct to deploy your API, as well as client and server code to help build your API quickly.
 *
 * @pjid type-safe-api
 */
export declare class TypeSafeApiProject extends Project {
    /**
     * Project for the api model
     */
    readonly model: TypeSafeApiModelProject;
    /**
     * Generated runtime projects. When `runtime.languages` includes the corresponding language, the project can be
     * assumed to be defined.
     */
    readonly runtime: GeneratedCodeProjects;
    /**
     * Generated infrastructure projects. Only the property corresponding to `infrastructure.language` will be defined.
     */
    readonly infrastructure: GeneratedCodeProjects;
    constructor(options: TypeSafeApiProjectOptions);
    private getParentMonorepo;
}
