## Type Safe API

Define your APIs using [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and leverage the power of generated client and server types, infrastructure, documentation, and automatic input validation!

This package vends a projen project type which allows you to define an API using either [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and a construct which manages deploying this API in API Gateway, given an integration (eg a lambda) for every operation.

The project will generate "runtime" projects from your API definition in your desired languages, which can be utilised both client side for interacting with your API, or server side for implementing your API. The project also generates a type-safe CDK construct which ensures an integration is provided for every API operation.

Code is generated at build time, so when you change your API model, just rebuild and you'll see your changes reflected in the generated code.

### Quick Start: TypeScript

This section describes how to get started quickly, with TypeScript infrastructure and lambda handlers. See the end of the README for examples in [Python](#quick-start--python) and [Java](#quick-start--python).

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```java
// Example automatically generated from non-compiling source. May contain errors.
import software.aws.awsprototypingsdk.nxmonorepo.NxMonorepoProject;
import aws.prototyping.sdk.type.safe.api.TypeSafeApiProject;
import io.github.cdklabs.projen.awscdk.AwsCdkTypeScriptApp;

// Create the monorepo
NxMonorepoProject monorepo = NxMonorepoProject.Builder.create()....build();

// Create the API project
Object api = TypeSafeApiProject.Builder.create()
        .name("myapi")
        .parent(monorepo)
        .outdir("packages/api")
        // Smithy as the model language. You can also use ModelLanguage.OPENAPI
        .model(Map.of(
                "language", ModelLanguage.getSMITHY(),
                "options", Map.of(
                        "smithy", Map.of(
                                "serviceName", Map.of(
                                        "namespace", "com.mycompany",
                                        "serviceName", "MyApi")))))
        // Generate types, client and server code in TypeScript, Python and Java
        .runtime(Map.of(
                "languages", List.of(Language.getTYPESCRIPT(), Language.getPYTHON(), Language.getJAVA())))
        // CDK infrastructure in TypeScript
        .infrastructure(Map.of(
                "language", Language.getTYPESCRIPT()))
        // Generate HTML documentation
        .documentation(Map.of(
                "formats", List.of(DocumentationFormat.getHTML_REDOC())))
        .build();

// Create a CDK infrastructure project. Can also consider PDKPipelineTsProject as an alternative!
AwsCdkTypeScriptApp infra = AwsCdkTypeScriptApp.Builder.create()....build();

// Infrastructure can depend on the generated API infrastructure and runtime
infra.addDeps(api.getInfrastructure().getTypescript().getPackage().getPackageName());
infra.addDeps(api.getRuntime().getTypescript().getPackage().getPackageName());

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application (ie within the `infra` project we created), consume the `Api` construct, vended from the generated typescript infrastructure package.

```java
// Example automatically generated from non-compiling source. May contain errors.
import software.amazon.awscdk.Stack;
import software.amazon.awscdk.StackProps;
import software.constructs.Construct;
import myapi.typescript.infra.Api; // <- generated typescript infrastructure package
import aws.prototyping.sdk.type.safe.api.Authorizers;
import aws.prototyping.sdk.type.safe.api.Integrations;
import software.amazon.awscdk.services.lambda.nodejs.NodejsFunction;
import software.amazon.awscdk.services.apigateway.Cors;
import path.*;

public class MyStack extends Stack {
    public MyStack(Construct scope, String id) {
        this(scope, id, StackProps.builder().build());
    }

    public MyStack(Construct scope, String id, StackProps props) {
        super(scope, id, props);

        // Instantiate the generated CDK construct to deploy an API Gateway API based on your model
        // Instantiate the generated CDK construct to deploy an API Gateway API based on your model
        Api.Builder.create(this, "MyApi")
                .defaultAuthorizer(Authorizers.iam())
                .corsOptions(Map.of(
                        "allowOrigins", Cors.ALL_ORIGINS,
                        "allowMethods", Cors.ALL_METHODS))
                // Supply an integration for every operation
                .integrations(Map.of(
                        "sayHello", Map.of(
                                "integration", Integrations.lambda(NodejsFunction.Builder.create(this, "SayHelloLambda")
                                        .entry(resolve(__dirname, "say-hello.ts"))
                                        .build()))))
                .build();
    }
}
```

#### Implement a Lambda Handler

The generated runtime projects include lambda handler wrappers which provide type-safety for implementing your API operations. You can implement your lambda handlers in any of the supported languages, and even mix and match languages for different operations if you like.

In the above CDK application, we used `NodejsFunction` with the entry point as `say-hello.ts`, so we can define the lambda function in the same `infra` project.

In typescript, the implementation of `say-hello.ts` would look like:

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.sayHelloHandler.*; // <- generated typescript runtime package

// Use the handler wrapper for type-safety to ensure you correctly implement your modelled API operation
Object handler = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}`,
    },
  };
});
```

### Project

The `TypeSafeApiProject` projen project sets up the project structure for you. You have a few parameters to consider when creating the project:

* `model` - Configure the API model. Select a `language` for the model of either [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and supply `options.smithy` or `options.openapi` depending on your choice.
* `runtime` - Configure the generated runtime projects. Include one or more `languages` you wish to write your client and server-side code in. These projects contain generated types defined in your model, as well as type-safe lambda handler wrappers for implementing each operation.
* `infrastructure` - Pick the `language` you are writing your CDK infrastructure in. A construct will be generated in this language which can be used to deploy the API.
* `documentation` - Specify `formats` to generate documentation in.

It's recommended that these projects are used as part of an `nx-monorepo` project (eg. by specifying `parent: myMonorepoProject`), as it makes setting up dependencies much easier, particularly when extending your project further with a CDK app and lambda functions.

Depending on the `model.language` you choose, you must supply the corresponding `model.options`. For example:

```java
// Example automatically generated from non-compiling source. May contain errors.
TypeSafeApiProject.Builder.create()
        .model(Map.of(
                "language", ModelLanguage.getSMITHY(),
                "options", Map.of(
                        "smithy", Map.of(
                                "serviceName", Map.of(
                                        "namespace", "com.mycompany",
                                        "serviceName", "MyApi")))))...
        .build();
```

```java
// Example automatically generated from non-compiling source. May contain errors.
TypeSafeApiProject.Builder.create()
        .model(Map.of(
                "language", ModelLanguage.getOPENAPI(),
                "options", Map.of(
                        "openapi", Map.of(
                                "title", "MyApi"))))...
        .build();
```

`model.options.smithy` allows for further customisation of the Smithy project, eg:

```java
// Example automatically generated from non-compiling source. May contain errors.
TypeSafeApiProject.Builder.create()
        .model(Map.of(
                "language", ModelLanguage.getSMITHY(),
                "options", Map.of(
                        "smithy", Map.of(
                                "serviceName", Map.of(
                                        "namespace", "com.mycompany",
                                        "serviceName", "MyApi"),
                                // By default, the contents of the smithy build output directory `model/output` will be ignored by source control.
                                // Set this to false to include it, for example if you are generating clients directly from the smithy model.
                                "ignoreSmithyBuildOutput", false,
                                // The gradle wrapper used for the smithy build is copied from the PDK itself if it does not already exist in
                                // the 'smithy' folder. By default, this gradle wrapper will be ignored by source control.
                                // Set this to false if you would like to check the gradle wrapper in to source control, for example if you want
                                // to use a different version of the gradle wrapper in your project.
                                "ignoreGradleWrapper", false,
                                // Use smithyBuildOptions to control what is added to smithy-build.json.
                                "smithyBuildOptions", Map.of(
                                        "projections", Map.of(
                                                // You can customise the built-in openapi projection, used to generate the OpenAPI specification.
                                                "openapi", Map.of(
                                                        "plugins", Map.of(
                                                                "openapi", Map.of(
                                                                        // Customise the openapi projection here.
                                                                        // See: https://smithy.io/2.0/guides/converting-to-openapi.html
                                                                        "useIntegerType", true, ...))),
                                                // You can add new projections here too
                                                "ts-client", Map.of(
                                                        "plugins", Map.of(
                                                                "typescript-codegen", Map.of(
                                                                        "package", "@my-test/smithy-generated-typescript-client",
                                                                        "packageVersion", "0.0.1")))),
                                        // Note that any additional dependencies required for projections/plugins can be added here, which in turn will
                                        // add them to the `smithy/build.gradle` file
                                        "maven", Map.of(
                                                "dependencies", List.of("software.amazon.smithy:smithy-validation-model:1.27.2")))))))...
        .build();
```

#### Directory Structure

The `TypeSafeApiProject` will create the following directory structure within its `outdir`:

```
|_ model/
    |_ src/
        |_ main/
            |_ smithy - your API definition if you chose ModelLanguage.SMITHY
            |_ openapi - your API definition if you chose ModelLanguage.OPENAPI
|_ runtime/ - generated types, client, and server code in the languages you specified
    |_ typescript
    |_ python
    |_ java
|_ infrastructure/ - generated infrastructure (you'll find only one directory in here based on your chosen infrastructure language)
    |_ typescript
    |_ python
    |_ java
|_ documentation/ - generated documentation in the formats you specified
    |_ html2
    |_ html_redoc
    |_ plantuml
    |_ markdown
```

### Smithy IDL

Please refer to the [Smithy documentation](https://smithy.io/2.0/quickstart.html) for how to write models in Smithy. A basic example is provided below:

```smithy
$version: "2"
namespace example.hello

use aws.protocols#restJson1

@title("A Sample Hello World API")

/// A sample smithy api
@restJson1
service Hello {
    version: "1.0"
    operations: [SayHello]
}

@readonly
@http(method: "GET", uri: "/hello")
operation SayHello {
    input: SayHelloInput
    output: SayHelloOutput
    errors: [ApiError]
}

string Name
string Message

@input
structure SayHelloInput {
    @httpQuery("name")
    @required
    name: Name
}

@output
structure SayHelloOutput {
    @required
    message: Message
}

@error("client")
structure ApiError {
    @required
    errorMessage: Message
}
```

#### Supported Protocols

Currently only [AWS restJson1](https://smithy.io/2.0/aws/protocols/aws-restjson1-protocol.html) is supported. Please ensure your service is annotated with the `@restJson1` trait.

#### Multiple Files

You can split your definition into multiple files and folders, so long as they are all under the `model/src/main/smithy` directory in your API project.

#### Authorizers

Smithy supports [adding API Gateway authorizers in the model itself](https://smithy.io/2.0/aws/aws-auth.html). Given that at model definition time one usually does not know the ARN of the user pool or lambda function for an authorizer, it is recommended to add the authorizer(s) in your Api CDK construct.

If using Smithy generated clients, some authorizer traits (eg sigv4) will include configuring the client for that particular method of authorization, so it can be beneficial to still define authorizers in the model. We therefore support specifying authorizers in both the model and the construct, but note that the construct will take precedence where the authorizer ID is the same.

### OpenAPI Specification

Your `model/src/main/openapi/main.yaml` file defines your api using [OpenAPI Version 3.0.3](https://swagger.io/specification/). An example spec might look like:

```yaml
openapi: 3.0.3
info:
  version: 1.0.0
  title: Example API
paths:
  /hello:
    get:
      operationId: sayHello
      parameters:
        - in: query
          name: name
          schema:
            type: string
          required: true
      responses:
        '200':
          description: Successful response
          content:
            'application/json':
              schema:
                $ref: '#/components/schemas/HelloResponse'
components:
  schemas:
    HelloResponse:
      type: object
      properties:
        message:
          type: string
      required:
        - message
```

You can divide your specification into multiple files using `$ref`.

For example, you might choose to structure your spec as follows:

```
|_ model/src/main/openapi/
    |_ main.yaml
    |_ paths/
        |_ index.yaml
        |_ sayHello.yaml
    |_ schemas/
        |_ index.yaml
        |_ helloResponse.yaml
```

Where `main.yaml` looks as follows:

```yaml
openapi: 3.0.3
info:
  version: 1.0.0
  title: Example API
paths:
  $ref: './paths/index.yaml'
components:
  schemas:
    $ref: './schemas/index.yaml'
```

`paths/index.yaml`:

```yaml
/hello:
  get:
    $ref: './sayHello.yaml'
```

`paths/sayHello.yaml`:

```yaml
operationId: sayHello
parameters:
 - in: query
   name: name
   schema:
     type: string
   required: true
responses:
  '200':
    description: Successful response
    content:
      'application/json':
        schema:
          $ref: '../schemas/helloResponse.yaml'
```

`schemas/index.yaml`:

```yaml
HelloResponse:
  $ref: './helloResponse.yaml'
```

`schemas/helloResponse.yaml`:

```yaml
type: object
properties:
  message:
    type: string
required:
  - message
```

### Construct

A CDK construct is generated in the `infrastructure/<language>` directory which provides a type-safe interface for creating an API Gateway API based on your model.

You can extend or instantiate this construct in your CDK infrastructure project. You'll get a type error if you forget to define an integration for an operation defined in your api.

```java
// Example automatically generated from non-compiling source. May contain errors.
import aws.prototyping.sdk.type.safe.api.Authorizers;
import aws.prototyping.sdk.type.safe.api.Integrations;
import software.amazon.awscdk.services.lambda.nodejs.NodejsFunction;
import software.constructs.Construct;
import myapi.typescript.infra.Api;

/**
 * An example of how to wire lambda handler functions to the API
 */
public class SampleApi extends Api {
    public SampleApi(Construct scope, String id) {
        super(scope, id, Map.of(
                "defaultAuthorizer", Authorizers.iam(),
                "integrations", Map.of(
                        // Every operation defined in your API must have an integration defined!
                        "sayHello", Map.of(
                                "integration", Integrations.lambda(new NodejsFunction(scope, "say-hello"))))));
    }
}
```

#### Sharing Integrations

If you would like to use the same integration for every operation (for example you'd like to use a single lambda function to service all requests with the in-built [handler router](#handler-router)), you can use the `Operations.all` method from a generated runtime project to save repeating yourself:

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.Operations;
import aws.prototyping.sdk.type.safe.api.Authorizers;
import aws.prototyping.sdk.type.safe.api.Integrations;
import software.amazon.awscdk.services.lambda.nodejs.NodejsFunction;
import software.constructs.Construct;
import myapi.typescript.infra.Api;

public class SampleApi extends Api {
    public SampleApi(Construct scope, String id) {
        super(scope, id, Map.of(
                "defaultAuthorizer", Authorizers.iam(),
                // Use the same integration for every operation.
                "integrations", Operations.all(Map.of(
                        "integration", Integrations.lambda(new NodejsFunction(scope, "router"))))));
    }
}
```

TypeScript is demonstrated above, but this is also available in Java and Python.

#### Authorizers

The `Api` construct allows you to define one or more authorizers for securing your API. An integration will use the `defaultAuthorizer` unless an `authorizer` is specified at the integration level. The following authorizers are supported:

* `Authorizers.none` - No auth
* `Authorizers.iam` - AWS IAM (Signature Version 4)
* `Authorizers.cognito` - Cognito user pool
* `Authorizers.custom` - A custom authorizer

##### Cognito Authorizer

To use the Cognito authorizer, one or more user pools must be provided. You can optionally specify the scopes to check if using an access token. You can use the `withScopes` method to use the same authorizer but verify different scopes for individual integrations, for example:

```java
// Example automatically generated from non-compiling source. May contain errors.
public class SampleApi extends Api {
    public SampleApi(Construct scope, String id) {
        Object cognitoAuthorizer = Authorizers.cognito(Map.of(
                "authorizerId", "myCognitoAuthorizer",
                "userPools", List.of(new UserPool(scope, "UserPool"))));

        super(scope, id, Map.of(
                "defaultAuthorizer", cognitoAuthorizer,
                "integrations", Map.of(
                        // Everyone in the user pool can call this operation:
                        "sayHello", Map.of(
                                "integration", Integrations.lambda(new NodejsFunction(scope, "say-hello"))),
                        // Only users with the given scopes can call this operation
                        "myRestrictedOperation", Map.of(
                                "integration", Integrations.lambda(new NodejsFunction(scope, "my-restricted-operation")),
                                "authorizer", cognitoAuthorizer.withScopes("my-resource-server/my-scope")))));
    }
}
```

For more information about scopes or identity and access tokens, please see the [API Gateway documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-integrate-with-cognito.html).

##### Custom Authorizer

Custom authorizers use lambda functions to handle authorizing requests. These can either be simple token-based authorizers, or more complex request-based authorizers. See the [API Gateway documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-use-lambda-authorizer.html) for more details.

An example token-based authorizer (default):

```java
// Example automatically generated from non-compiling source. May contain errors.
Authorizers.custom(Map.of(
        "authorizerId", "myTokenAuthorizer",
        "function", new NodejsFunction(scope, "authorizer")));
```

An example request-based handler. By default the identitySource will be `method.request.header.Authorization`, however you can customise this as per [the API Gateway documentation](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource).

```java
// Example automatically generated from non-compiling source. May contain errors.
Authorizers.custom(Map.of(
        "authorizerId", "myRequestAuthorizer",
        "type", CustomAuthorizerType.getREQUEST(),
        "identitySource", "method.request.header.MyCustomHeader, method.request.querystring.myQueryString",
        "function", new NodejsFunction(scope, "authorizer")));
```

#### Integrations

Integrations are used by API Gateway to service requests.

##### Lambda Integration

Currently, the only built-in integration is a lambda integration. You can construct one using `Integrations.lambda(yourLambdaFunction)`.

##### Custom Integrations

You can implement your own integrations by inheriting the `Integration` class and implementing its `render` method. This method is responsible for returning a snippet of OpenAPI which will be added as the `x-amazon-apigateway-integration` for an operation. Please refer to the [API Gateway Swagger Extensions documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html) for more details.

You can also optionally override the `grant` method if you need to use CDK to grant API Gateway access to invoke your integration.

### Runtime: Clients

The generated runtime projects include clients which can be used for type-safe interaction with your API.

#### Typescript

The [typescript-fetch](https://openapi-generator.tech/docs/generators/typescript-fetch/) OpenAPI generator is used to generate typescript client. This requires an implementation of `fetch` to be passed to the client. In the browser one can pass the built in fetch, or in NodeJS you can use an implementation such as [node-fetch](https://www.npmjs.com/package/node-fetch).

Example usage of the client in a website:

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.Configuration;
import myapi.typescript.runtime.DefaultApi;

Object client = new DefaultApi(Configuration.Builder.create()
        .basePath("https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com")
        .fetchApi(window.fetch.bind(window))
        .build());

client.sayHello(Map.of("name", "Jack"));
```

#### Python

The [python](https://openapi-generator.tech/docs/generators/python) OpenAPI generator is used to generate clients for python.

Example usage:

```python
from myapi_python_runtime import ApiClient, Configuration
from myapi_python_runtime.api.default_api import DefaultApi

configuration = Configuration(
    host = "https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com"
)

with ApiClient(configuration) as api_client:
    client = DefaultApi(api_client)

    client.say_hello(
        query_params={
            'name': "name_example",
        },
    )
```

You'll find details about how to use the python client in the README.md in your generated runtime project.

#### Java

The [java](https://openapi-generator.tech/docs/generators/java/) OpenAPI generator is used to generate clients for Java.

Example usage:

```java
import com.generated.api.myapijavaruntime.runtime.api.DefaultApi;
import com.generated.api.myapijavaruntime.runtime.ApiClient;
import com.generated.api.myapijavaruntime.runtime.Configuration;
import com.generated.api.myapijavaruntime.runtime.models.HelloResponse;

ApiClient client = Configuration.getDefaultApiClient();
client.setBasePath("https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com");

DefaultApi api = new DefaultApi(client);
HelloResponse response = api.sayHello("Adrian").execute()
```

You'll find more details about how to use the Java client in the README.md in your generated runtime project.

### Lambda Handler Wrappers

Lambda handler wrappers are also importable from the generated runtime projects. These provide input/output type safety, ensuring that your API handlers return outputs that correspond to your model.

#### Typescript

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.sayHelloHandler.*;

Object handler = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so with the `handlerRouter`.

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.handlerRouter.*;
import myapi.typescript.runtime.sayHelloHandler.*;
import myapi.typescript.runtime.sayGoodbyeHandler.*;
import interceptors.corsInterceptor.*;
import handlers.say.goodbye.sayGoodbye.*;

Object sayHello = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});

Object handler = handlerRouter(Map.of(
        // Interceptors declared in this list will apply to all operations
        "interceptors", List.of(corsInterceptor),
        // Assign handlers to each operation here
        "handlers", Map.of(
                "sayHello", sayHello,
                "sayGoodbye", sayGoodbye)));
```

#### Python

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse

@say_hello_handler
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so with the `handler_router`.

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses, handler_router, HandlerRouterHandlers
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse
from other_handlers import say_goodbye
from my_interceptors import cors_interceptor

@say_hello_handler
def say_hello(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )

handler = handler_router(
  # Interceptors defined here will apply to all operations
  interceptors=[cors_interceptor],
  handlers=HandlerRouterHandlers(
    say_hello=say_hello,
    say_goodbye=say_goodbye
  )
)
```

#### Java

```java
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello200Response;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloRequestInput;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloResponse;
import com.generated.api.myapijavaruntime.runtime.model.HelloResponse;


public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so by extending the `HandlerRouter` class.

```java
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayGoodbye;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.HandlerRouter;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.Interceptors;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;

import java.util.Arrays;
import java.util.List;

// Interceptors defined here apply to all operations
@Interceptors({ TimingInterceptor.class })
public class ApiHandlerRouter extends HandlerRouter {
    // You must implement a method to return a handler for every operation
    @Override
    public SayHello sayHello() {
        return new SayHelloHandler();
    }

    @Override
    public SayGoodbye sayGoodbye() {
        return new SayGoodbyeHandler();
    }
}
```

### Interceptors

The lambda handler wrappers allow you to pass in a *chain* of handler functions to handle the request. This allows you to implement middleware / interceptors for handling requests. Each handler function may choose whether or not to continue the handler chain by invoking `chain.next`.

#### Typescript

In typescript, interceptors are passed as separate arguments to the generated handler wrapper, in the order in which they should be executed. Call `request.chain.next(request)` from an interceptor to delegate to the rest of the chain to handle a request. Note that the last handler in the chain (ie the actual request handler which transforms the input to the output) should not call `chain.next`.

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.sayHelloHandler.*;
import myapi.typescript.runtime.ChainedRequestInput;
import myapi.typescript.runtime.OperationResponse;

// Interceptor to wrap invocations in a try/catch, returning a 500 error for any unhandled exceptions.
__function tryCatchInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response | OperationResponse<500, { errorMessage: string }>> => {
  try {
    return await request.chain.next(request);
  } catch (e: any) {
    return { statusCode: 500, body: { errorMessage: e.message }};
  }
};

// tryCatchInterceptor is passed first, so it runs first and calls the second argument function (the request handler) via chain.next
Object handler = sayHelloHandler(tryCatchInterceptor, async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});
```

Another example interceptor might be to record request time metrics. The example below includes the full generic type signature for an interceptor:

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.ChainedRequestInput;

__function timingInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response> => {
  const start = Date.now();
  const response = await request.chain.next(request);
  const end = Date.now();
  console.log(`Took ${end - start} ms`);
  return response;
};
```

Interceptors may mutate the `interceptorContext` to pass state to further interceptors or the final lambda handler, for example an `identityInterceptor` might want to extract the authenticated user from the request so that it is available in handlers.

```java
// Example automatically generated from non-compiling source. May contain errors.
import myapi.typescript.runtime.LambdaRequestParameters;
import myapi.typescript.runtime.LambdaHandlerChain;

__function identityInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response> => {
  const authenticatedUser = await getAuthenticatedUser(request.event);
  return await request.chain.next({
    ...request,
    interceptorContext: {
      ...request.interceptorContext,
      authenticatedUser,
    },
  });
};
```

#### Python

In Python, a list of interceptors can be passed as a keyword argument to the generated lambda handler decorator, for example:

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse

@say_hello_handler(interceptors=[timing_interceptor, try_catch_interceptor])
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )
```

Writing an interceptor is just like writing a lambda handler. Call `chain.next(input)` from an interceptor to delegate to the rest of the chain to handle a request.

```python
import time
from myapi_python_runtime.apis.tags.default_api_operation_config import ChainedApiRequest, ApiResponse

def timing_interceptor(input: ChainedApiRequest) -> ApiResponse:
    start = int(round(time.time() * 1000))
    response = input.chain.next(input)
    end = int(round(time.time() * 1000))
    print("Took {} ms".format(end - start))
    return response
```

Interceptors may choose to return different responses, for example to return a 500 response for any unhandled exceptions:

```python
import time
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.apis.tags.default_api_operation_config import ChainedApiRequest, ApiResponse

def try_catch_interceptor(input: ChainedApiRequest) -> ApiResponse:
    try:
        return input.chain.next(input)
    except Exception as e:
        return ApiResponse(
            status_code=500,
            body=ApiError(errorMessage=str(e)),
            headers={}
        )
```

Interceptors are permitted to mutate the "interceptor context", which is a `Dict[str, Any]`. Each interceptor in the chain, and the final handler, can access this context:

```python
def identity_interceptor(input: ChainedApiRequest) -> ApiResponse:
    input.interceptor_context["AuthenticatedUser"] = get_authenticated_user(input.event)
    return input.chain.next(input)
```

Interceptors can also mutate the response returned by the handler chain. An example use case might be adding cross-origin resource sharing headers:

```python
def add_cors_headers_interceptor(input: ChainedApiRequest) -> ApiResponse:
    response = input.chain.next(input)
    return ApiResponse(
        status_code=response.status_code,
        body=response.body,
        headers={
            **response.headers,
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Headers": "*"
        }
    )
```

#### Java

In Java, interceptors can be added to a handler via the `@Interceptors` class annotation:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptors;

@Interceptors({ TimingInterceptor.class, TryCatchInterceptor.class })
public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

To write an interceptor, you can implement the `Interceptor` interface. For example, a timing interceptor:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptor;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ChainedRequestInput;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Response;

public class TimingInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        long start = System.currentTimeMillis();
        Response res = input.getChain().next(input);
        long end = System.currentTimeMillis();
        System.out.printf("Took %d ms%n", end - start);
        return res;
    }
}
```

Interceptors may choose to return different responses, for example to return a 500 response for any unhandled exceptions:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptor;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ChainedRequestInput;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Response;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ApiResponse;
import com.generated.api.myjavaapiruntime.runtime.model.ApiError;

public class TryCatchInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        try {
            return input.getChain().next(input);
        } catch (Exception e) {
            return ApiResponse.builder()
                    .statusCode(500)
                    .body(ApiError.builder()
                            .errorMessage(e.getMessage())
                            .build().toJson())
                    .build();
        }
    }
}
```

Interceptors are permitted to mutate the "interceptor context", which is a `Map<String, Object>`. Each interceptor in the chain, and the final handler, can access this context:

```java
public class IdentityInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        input.getInterceptorContext().put("AuthenticatedUser", this.getAuthenticatedUser(input.getEvent()));
        return input.getChain().next(input);
    }
}
```

Interceptors can also mutate the response returned by the handler chain. An example use case might be adding cross-origin resource sharing headers:

```java
public static class AddCorsHeadersInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        Response res = input.getChain().next(input);
        res.getHeaders().put("Access-Control-Allow-Origin", "*");
        res.getHeaders().put("Access-Control-Allow-Headers", "*");
        return res;
    }
}
```

##### Interceptors with Dependency Injection

Interceptors referenced by the `@Interceptors` annotation must be constructable with no arguments. If more complex instantiation of your interceptor is required (for example if you are using dependency injection or wish to pass configuration to your interceptor), you may instead override the `getInterceptors` method in your handler:

```java
public class SayHelloHandler extends SayHello {
    @Override
    public List<Interceptor<SayHelloInput>> getInterceptors() {
        return Arrays.asList(
                new MyConfiguredInterceptor<>(42),
                new MyOtherConfiguredInterceptor<>("configuration"));
    }

    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s!", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

### Quick Start: Python

This guide assumes you want to write your CDK infrastructure in Python and your lambda handlers in Python, however note that you your infrastructure language and lambda handler language(s) are not tied to one another, you can mix and match as you like. Just specify the language in `runtime.languages` for any language you would like to write lambda handlers in.

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```java
// Example automatically generated from non-compiling source. May contain errors.
import software.aws.awsprototypingsdk.nxmonorepo.NxMonorepoProject;
import aws.prototyping.sdk.type.safe.api.TypeSafeApiProject;
import io.github.cdklabs.projen.awscdk.AwsCdkTypeScriptApp;
import io.github.cdklabs.projen.python.PythonProject;

// Create the monorepo
NxMonorepoProject monorepo = NxMonorepoProject.Builder.create()
        .name("monorepo")
        .defaultReleaseBranch("main")
        .build();

// Create the API project
Object api = TypeSafeApiProject.Builder.create()
        .name("myapi")
        .parent(monorepo)
        .outdir("packages/api")
        // Smithy as the model language. You can also use ModelLanguage.OPENAPI
        .model(Map.of(
                "language", ModelLanguage.getSMITHY(),
                "options", Map.of(
                        "smithy", Map.of(
                                "serviceName", Map.of(
                                        "namespace", "com.mycompany",
                                        "serviceName", "MyApi")))))
        // Generate client and server types in TypeScript, Python, and Java
        .runtime(Map.of(
                "languages", List.of(Language.getTYPESCRIPT(), Language.getPYTHON(), Language.getJAVA())))
        // Generate CDK infrastructure in Python
        .infrastructure(Map.of(
                "language", Language.getPYTHON()))
        // Generate HTML documentation
        .documentation(Map.of(
                "formats", List.of(DocumentationFormat.getHTML_REDOC())))
        .build();

// Create a project for our lambda handlers written in python
PythonProject lambdas = PythonProject.Builder.create()
        .name("lambdas")
        .parent(monorepo)
        .outdir("packages/lambdas")
        .authorEmail("me@example.com")
        .authorName("me")
        .moduleName("lambdas")
        .version("1.0.0")
        // Poetry is used to simplify local python dependencies
        .poetry(true)
        .build();

// Add a local dependency on the generated python runtime
monorepo.addPythonPoetryDependency(lambdas, api.getRuntime().getPython());

// Add commands to the lambda project's package task to create a distributable which can be deployed to AWS Lambda
lambdas.packageTask.exec("mkdir -p lambda-dist && rm -rf lambda-dist/*");
lambdas.packageTask.exec(String.format("cp -r %s lambda-dist/%s", lambdas.getModuleName(), lambdas.getModuleName()));
lambdas.packageTask.exec("poetry export --without-hashes --format=requirements.txt > lambda-dist/requirements.txt");
lambdas.packageTask.exec("pip install -r lambda-dist/requirements.txt --target lambda-dist --upgrade");
lambdas.gitignore.addPatterns("lambda-dist");

// Create a CDK infrastructure project
Object infra = AwsCdkPythonApp.Builder.create()
        .name("infra")
        .parent(monorepo)
        .outdir("packages/infra")
        .authorEmail("me@example.com")
        .authorName("me")
        .cdkVersion("2.0.0")
        .moduleName("infra")
        .version("1.0.0")
        .poetry(true)
        .build();

// The infrastructure project depends on the python types, python infrastructure, and the lambda package
monorepo.addPythonPoetryDependency(infra, api.getRuntime().getPython());
monorepo.addPythonPoetryDependency(infra, api.getInfrastructure().getPython());
monorepo.addPythonPoetryDependency(infra, lambdas);

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application, consume the `Api` construct, vended from the generated Python infrastructure package.

```python
import os
from aws_cdk import Stack
from constructs import Construct
from aws_cdk.aws_lambda import LayerVersion, Code, Function, Runtime
from aws_prototyping_sdk.type_safe_api import Authorizers, TypeSafeApiIntegration, Integrations

from myapi_python_runtime.apis.tags.default_api_operation_config import OperationConfig
from myapi_python_infra.api import Api
from pathlib import Path
from os import path

class MyStack(Stack):
    def __init__(self, scope: Construct, construct_id: str, **kwargs) -> None:
        super().__init__(scope, construct_id, **kwargs)

        # Use the generated Api construct
        self.api = Api(self, 'Api',
            default_authorizer=Authorizers.iam(),
            integrations=OperationConfig(
                say_hello=TypeSafeApiIntegration(
                    # Create a python lambda function from our "lambda-dist" package
                    integration=Integrations.lambda_(Function(self, 'SayHello',
                        runtime=Runtime.PYTHON_3_9,
                        code=Code.from_asset(path.join("..", "lambdas", "lambda-dist")),
                        handler="lambdas.say_hello.handler",
                    )),
                ),
            ),
        )
```

#### Implement a Lambda Handler

In your `lambdas` project you can define your lambda handler in its source directory, eg `lambdas/lambdas/say_hello.py`:

```python
from myapi_python_runtime.model.say_hello_response_content import SayHelloResponseContent
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler,
    SayHelloRequest, SayHelloOperationResponses, ApiResponse


@say_hello_handler
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=SayHelloResponseContent(message="Hello {}".format(input.request_parameters["name"])),
        headers={}
    )
```

### Quick Start: Java

This guide assumes you want to write your CDK infrastructure in Java and your lambda handlers in Java, however note that you your infrastructure language and lambda handler language(s) are not tied to one another, you can mix and match as you like.

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```java
// Example automatically generated from non-compiling source. May contain errors.
import software.aws.awsprototypingsdk.nxmonorepo.NxMonorepoProject;
import aws.prototyping.sdk.type.safe.api.TypeSafeApiProject;
import io.github.cdklabs.projen.awscdk.AwsCdkTypeScriptApp;
import io.github.cdklabs.projen.java.JavaProject;

// Create the monorepo
NxMonorepoProject monorepo = NxMonorepoProject.Builder.create()
        .name("monorepo")
        .defaultReleaseBranch("main")
        .build();

// Create the API project
Object api = TypeSafeApiProject.Builder.create()
        .name("myapi")
        .parent(monorepo)
        .outdir("packages/api")
        // Smithy as the model language. You can also use ModelLanguage.OPENAPI
        .model(Map.of(
                "language", ModelLanguage.getSMITHY(),
                "options", Map.of(
                        "smithy", Map.of(
                                "serviceName", Map.of(
                                        "namespace", "com.mycompany",
                                        "serviceName", "MyApi")))))
        // Generate client and server types in TypeScript, Python and Java
        .runtime(Map.of(
                "languages", List.of(Language.getTYPESCRIPT(), Language.getPYTHON(), Language.getJAVA())))
        // Generate CDK infrastructure in Java
        .infrastructure(Map.of(
                "language", Language.getJAVA()))
        // Generate HTML documentation
        .documentation(Map.of(
                "formats", List.of(DocumentationFormat.getHTML_REDOC())))
        .build();

JavaProject lambdas = JavaProject.Builder.create()
        .name("lambdas")
        .parent(monorepo)
        .outdir("packages/lambdas")
        .artifactId("lambdas")
        .groupId("com.my.api")
        .version("1.0.0")
        .build();

// The lambdas package needs a dependency on the generated java runtime
monorepo.addJavaDependency(lambdas, api.getRuntime().getJava());

// Use the maven shade plugin to build a "super jar" which we can deploy to AWS Lambda
lambdas.pom.addPlugin("org.apache.maven.plugins/maven-shade-plugin@3.3.0", PluginOptions.builder()
        .configuration(Map.of(
                "createDependencyReducedPom", false))
        .executions(List.of(PluginExecution.builder()
                .id("shade-task")
                .phase("package")
                .goals(List.of("shade"))
                .build()))
        .build());

Object infra = AwsCdkJavaApp.Builder.create()
        .name("infra")
        .parent(monorepo)
        .outdir("packages/infra")
        .artifactId("infra")
        .groupId("com.my.api")
        .mainClass("com.my.api.MyApp")
        .version("1.0.0")
        .cdkVersion("2.0.0")
        .build();

// Add a dependency on the generated CDK infrastructure
monorepo.addJavaDependency(infra, api.getInfrastructure().getJava());

// Make sure the java lambda builds before our CDK infra
monorepo.addImplicitDependency(infra, lambdas);

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application, consume the `Api` construct, vended from the generated Java infrastructure package.

```java
package com.my.api;

import com.generated.api.myapijavainfra.infra.Api;
import com.generated.api.myapijavainfra.infra.ApiProps;
import com.generated.api.myapijavaruntime.runtime.api.OperationConfig;

import software.amazon.awscdk.Duration;
import software.amazon.awscdk.services.apigateway.CorsOptions;
import software.amazon.awscdk.services.lambda.Code;
import software.amazon.awscdk.services.lambda.Function;
import software.amazon.awscdk.services.lambda.FunctionProps;
import software.amazon.awscdk.services.lambda.Runtime;
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;
import software.aws.awsprototypingsdk.typesafeapi.Integrations;
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiIntegration;

import software.amazon.awscdk.App;
import software.amazon.awscdk.Stack;

import java.util.Arrays;

public class MyApp {
    public static void main(final String[] args) {
        App app = new App();
        Stack s = new Stack(app, "infra");

        // Declare the API construct to deploy the API Gateway resources
        new Api(s, "Api", ApiProps.builder()
                .defaultAuthorizer(Authorizers.iam())
                .corsOptions(CorsOptions.builder()
                        .allowOrigins(Arrays.asList("*"))
                        .allowMethods(Arrays.asList("*"))
                        .build())
                .integrations(OperationConfig.<TypeSafeApiIntegration>builder()
                        .sayHello(TypeSafeApiIntegration.builder()
                                .integration(Integrations.lambda(
                                        // Point the lambda function to our built jar from the "lambdas" package
                                        new Function(s, "say-hello", FunctionProps.builder()
                                                .code(Code.fromAsset("../lambdas/dist/java/com/my/api/lambdas/1.0.0/lambdas-1.0.0.jar"))
                                                .handler("com.my.api.SayHelloHandler")
                                                .runtime(Runtime.JAVA_11)
                                                .timeout(Duration.seconds(30))
                                                .build())))
                                .build())
                        .build())
                .build());

        app.synth();
    }
}
```

#### Implement a Lambda Handler

In your `lambdas` project you can define your lambda handler in its source directory, eg `lambdas/src/main/java/com/my/api/SayHelloHandler.java`:

```java
package com.my.api;

import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello200Response;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloRequestInput;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloResponse;
import com.generated.api.myapijavaruntime.runtime.model.SayHelloResponseContent;

/**
 * An example lambda handler which uses the generated handler wrapper class (Handlers.SayHello) to manage marshalling
 * inputs and outputs.
 */
public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(SayHelloResponseContent.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}

```

### Other Details

#### Customising Generated Types/Infrastructure Projects

By default, the generated types and infrastructure projects are configured automatically, including their project names. You can customise the generated projects using the `runtime.options.<language>` or `infrastructure.options.<language>` properties when constructing the `TypeSafeApiProject`.

#### AWS WAFv2 Web ACL

By default, a [Web ACL](https://docs.aws.amazon.com/waf/latest/developerguide/web-acl.html) is deployed and attached to your API Gateway Rest API with the "[AWSManagedRulesCommonRuleSet](https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-baseline.html)", which provides protection against exploitation of a wide range of vulnerabilities, including some of the high risk and commonly occurring vulnerabilities described in OWASP publications such as [OWASP Top 10](https://owasp.org/www-project-top-ten/).

You can customise the Web ACL configuration via the `webAclOptions` of your `Api` CDK construct, eg:

```java
// Example automatically generated from non-compiling source. May contain errors.
public class SampleApi extends Api {
    public SampleApi(Construct scope, String id) {
        super(scope, id, Map.of(
                "integrations", Map.of(...),
                "webAclOptions", Map.of(
                        // Allow access only to specific CIDR ranges
                        "cidrAllowList", Map.of(
                                "cidrType", "IPV4",
                                "cidrRanges", List.of("1.2.3.4/5")),
                        // Pick from the set here: https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-list.html
                        "managedRules", List.of(Map.of("vendor", "AWS", "name", "AWSManagedRulesSQLiRuleSet")))));
    }
}
```

You can remove the Web ACL entirely with `webAclOptions: { disable: true }` - you may wish to use this if you'd like to set up a Web ACL yourself with more control over the rules.

#### Smithy IntelliJ Plugin

The Smithy-based projects are compatible with the [Smithy IntelliJ Plugin](https://github.com/iancaffey/smithy-intellij-plugin), which provides syntax highlighting and auto-complete for your Smithy model. To make use of it, perform the following steps:

* Install the "Smithy" plugin (under `Preferences -> Plugins`)
* Right-click on the `smithy/build.gradle` file in your Smithy API project
* Select "Link Gradle Project"

#### Tagging Operations

Operations can be grouped together into logical collections via tags. This can be achieved in Smithy with the `@tags` trait:

```smithy
@tags(["pets", "users"])
operation PurchasePet {
  ...
}
```

Or in OpenAPI using the `tags` property:

```yaml
paths:
  /pets/purchase:
    post:
      operationId: purchasePet
      tags:
        - pets
        - users
      ...
```

When multiple tags are used, the "first" tag is considered to be the API that the operation belongs to, so in the generated client, the above example operation would be included in the `PetsApi` client but not the `UsersApi` client.

Multiple tags are still useful for documentation generation, for example `DocumentationFormat.HTML_REDOC` will group operations by tag in the side navigation bar.

If you would like to introduce tags without breaking existing clients, we recommend first adding a tag named `default` to all operations.

⚠️ **Important Note**: Smithy versions below `1.28.0` sort tags in alphabetical order and so the "first" tag will be the earliest in the alphabet. Therefore, if using tags with older versions of Smithy, we recommend prefixing your desired first tag with an underscore (for example `_default`). This is rectified in `1.28.0`, where tag order from the `@tags` trait is preserved.

#### Smithy Model Libraries and Dependencies

You can instantiate the TypeSafeApiModelProject on its own to create a standalone Smithy model library.

You can consume the library using the `addSmithyDeps` method, which adds a local file dependency on the built Smithy jar.

```java
// Example automatically generated from non-compiling source. May contain errors.
// Standalone model project, used as our model library
Object shapes = TypeSafeApiModelProject.Builder.create()
        .name("shapes")
        .parent(monorepo)
        .outdir("packages/shapes")
        .modelLanguage(ModelLanguage.getSMITHY())
        .modelOptions(Map.of(
                "smithy", Map.of(
                        "serviceName", Map.of(
                                "namespace", "com.my.shared.shapes",
                                "serviceName", "Ignored"))))
        .build();

Object api = TypeSafeApiProject.Builder.create()....build();

// Add the implicit monorepo dependency (if using the nx-monorepo) to ensure the shape library is built before the api model
monorepo.addImplicitDependency(api.getModel(), shapes);

// Add a local file dependency on the built shapes jar
api.model.smithy.addSmithyDeps(shapes.getSmithy());
```

#### Local API Development Server

You can use the [AWS SAM CLI](https://docs.aws.amazon.com/serverless-application-model/latest/developerguide/install-sam-cli.html) to run a local development server for your API. You can achieve this using the following steps:

1. Synthesize your CDK stack containing your `Api` construct (this might be your `AwsCdkTypeScriptApp` project for example), with the context property `type-safe-api-local` set to `true`, for example:

```bash
cd packages/infra
npx cdk synth --context type-safe-api-local=true
```

1. Use the AWS SAM CLI to start the local development server, pointing it at the cloudformation template synthesized from the above command (note that the command will fail if docker is not running)

```bash
sam local start-api -t cdk.out/<your-stack>.template.json
```

You will need to repeat the above steps every time you make a code change for them to be reflected in your local development server.

See the [AWS SAM CLI Reference](https://docs.aws.amazon.com/serverless-application-model/latest/developerguide/sam-cli-command-reference-sam-local-start-api.html) for more information on the commands to run.

Make sure you do not deploy your CDK stack with `type-safe-api-local` set to `true`, since this uses an inline API definition which bloats the CloudFormation template and can exceed the maximum template size depending on the size of your API.

##### Limitations

Note that there is currently a limitation with SAM CLI where it does not support mock integrations, which means that the development server will not respond to OPTIONS requests even if you specified `corsOptions: { ... }` in your `Api` construct. This is being tracked as a [feature request here](https://github.com/aws/aws-sam-cli/issues/4973).

Note also that your API business logic may include operations which do not work locally, or may interact with real AWS resources depending on the AWS credentials you start your local development server with.
# API Reference <a name="API Reference" id="api-reference"></a>

## Constructs <a name="Constructs" id="Constructs"></a>

### TypeSafeRestApi <a name="TypeSafeRestApi" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi"></a>

A construct for creating an api gateway rest api based on the definition in the OpenAPI spec.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeRestApi;

TypeSafeRestApi.Builder.create(Construct scope, java.lang.String id)
//  .cloudWatchRole(java.lang.Boolean)
//  .deploy(java.lang.Boolean)
//  .deployOptions(StageOptions)
//  .description(java.lang.String)
//  .disableExecuteApiEndpoint(java.lang.Boolean)
//  .domainName(DomainNameOptions)
//  .endpointExportName(java.lang.String)
//  .endpointTypes(java.util.List<EndpointType>)
//  .failOnWarnings(java.lang.Boolean)
//  .parameters(java.util.Map<java.lang.String, java.lang.String>)
//  .policy(PolicyDocument)
//  .restApiName(java.lang.String)
//  .retainDeployments(java.lang.Boolean)
    .integrations(java.util.Map<java.lang.String, TypeSafeApiIntegration>)
    .operationLookup(java.util.Map<java.lang.String, MethodAndPath>)
//  .corsOptions(CorsOptions)
//  .defaultAuthorizer(Authorizer)
    .specPath(java.lang.String)
//  .webAclOptions(TypeSafeApiWebAclOptions)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.scope">scope</a></code> | <code>software.constructs.Construct</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.id">id</a></code> | <code>java.lang.String</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.cloudWatchRole">cloudWatchRole</a></code> | <code>java.lang.Boolean</code> | Automatically configure an AWS CloudWatch role for API Gateway. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.deploy">deploy</a></code> | <code>java.lang.Boolean</code> | Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.deployOptions">deployOptions</a></code> | <code>software.amazon.awscdk.services.apigateway.StageOptions</code> | Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.description">description</a></code> | <code>java.lang.String</code> | A description of the RestApi construct. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.disableExecuteApiEndpoint">disableExecuteApiEndpoint</a></code> | <code>java.lang.Boolean</code> | Specifies whether clients can invoke the API using the default execute-api endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.domainName">domainName</a></code> | <code>software.amazon.awscdk.services.apigateway.DomainNameOptions</code> | Configure a custom domain name and map it to this API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.endpointExportName">endpointExportName</a></code> | <code>java.lang.String</code> | Export name for the CfnOutput containing the API endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.endpointTypes">endpointTypes</a></code> | <code>java.util.List<software.amazon.awscdk.services.apigateway.EndpointType></code> | A list of the endpoint types of the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.failOnWarnings">failOnWarnings</a></code> | <code>java.lang.Boolean</code> | Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.parameters">parameters</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Custom header parameters for the request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.policy">policy</a></code> | <code>software.amazon.awscdk.services.iam.PolicyDocument</code> | A policy document that contains the permissions for this RestApi. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.restApiName">restApiName</a></code> | <code>java.lang.String</code> | A name for the API Gateway RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.retainDeployments">retainDeployments</a></code> | <code>java.lang.Boolean</code> | Retains old deployment resources when the API changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.integrations">integrations</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>></code> | A mapping of API operation to its integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.operationLookup">operationLookup</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>></code> | Details about each operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.corsOptions">corsOptions</a></code> | <code>software.amazon.awscdk.services.apigateway.CorsOptions</code> | Cross Origin Resource Sharing options for the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.defaultAuthorizer">defaultAuthorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The default authorizer to use for your api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.specPath">specPath</a></code> | <code>java.lang.String</code> | Path to the JSON open api spec. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.webAclOptions">webAclOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a></code> | Options for the AWS WAF v2 WebACL associated with the api. |

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.scope"></a>

- *Type:* software.constructs.Construct

---

##### `id`<sup>Required</sup> <a name="id" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.id"></a>

- *Type:* java.lang.String

---

##### `cloudWatchRole`<sup>Optional</sup> <a name="cloudWatchRole" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.cloudWatchRole"></a>

- *Type:* java.lang.Boolean
- *Default:* false if `@aws-cdk/aws-apigateway:disableCloudWatchRole` is enabled, true otherwise

Automatically configure an AWS CloudWatch role for API Gateway.

---

##### `deploy`<sup>Optional</sup> <a name="deploy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.deploy"></a>

- *Type:* java.lang.Boolean
- *Default:* true

Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes.

Since API Gateway deployments are immutable, When this option is enabled
(by default), an AWS::ApiGateway::Deployment resource will automatically
created with a logical ID that hashes the API model (methods, resources
and options). This means that when the model changes, the logical ID of
this CloudFormation resource will change, and a new deployment will be
created.

If this is set, `latestDeployment` will refer to the `Deployment` object
and `deploymentStage` will refer to a `Stage` that points to this
deployment. To customize the stage options, use the `deployOptions`
property.

A CloudFormation Output will also be defined with the root URL endpoint
of this REST API.

---

##### `deployOptions`<sup>Optional</sup> <a name="deployOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.deployOptions"></a>

- *Type:* software.amazon.awscdk.services.apigateway.StageOptions
- *Default:* Based on defaults of `StageOptions`.

Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled.

If `deploy` is disabled,
this value cannot be set.

---

##### `description`<sup>Optional</sup> <a name="description" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.description"></a>

- *Type:* java.lang.String
- *Default:* 'Automatically created by the RestApi construct'

A description of the RestApi construct.

---

##### `disableExecuteApiEndpoint`<sup>Optional</sup> <a name="disableExecuteApiEndpoint" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.disableExecuteApiEndpoint"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Specifies whether clients can invoke the API using the default execute-api endpoint.

To require that clients use a custom domain name to invoke the
API, disable the default endpoint.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html)

---

##### `domainName`<sup>Optional</sup> <a name="domainName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.domainName"></a>

- *Type:* software.amazon.awscdk.services.apigateway.DomainNameOptions
- *Default:* no domain name is defined, use `addDomainName` or directly define a `DomainName`.

Configure a custom domain name and map it to this API.

---

##### `endpointExportName`<sup>Optional</sup> <a name="endpointExportName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.endpointExportName"></a>

- *Type:* java.lang.String
- *Default:* when no export name is given, output will be created without export

Export name for the CfnOutput containing the API endpoint.

---

##### `endpointTypes`<sup>Optional</sup> <a name="endpointTypes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.endpointTypes"></a>

- *Type:* java.util.List<software.amazon.awscdk.services.apigateway.EndpointType>
- *Default:* EndpointType.EDGE

A list of the endpoint types of the API.

Use this property when creating
an API.

---

##### `failOnWarnings`<sup>Optional</sup> <a name="failOnWarnings" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.failOnWarnings"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource.

---

##### `parameters`<sup>Optional</sup> <a name="parameters" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.parameters"></a>

- *Type:* java.util.Map<java.lang.String, java.lang.String>
- *Default:* No parameters.

Custom header parameters for the request.

> [https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html](https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html)

---

##### `policy`<sup>Optional</sup> <a name="policy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.policy"></a>

- *Type:* software.amazon.awscdk.services.iam.PolicyDocument
- *Default:* No policy.

A policy document that contains the permissions for this RestApi.

---

##### `restApiName`<sup>Optional</sup> <a name="restApiName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.restApiName"></a>

- *Type:* java.lang.String
- *Default:* ID of the RestApi construct.

A name for the API Gateway RestApi resource.

---

##### `retainDeployments`<sup>Optional</sup> <a name="retainDeployments" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.retainDeployments"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Retains old deployment resources when the API changes.

This allows
manually reverting stages to point to old deployments via the AWS
Console.

---

##### `integrations`<sup>Required</sup> <a name="integrations" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.integrations"></a>

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>>

A mapping of API operation to its integration.

---

##### `operationLookup`<sup>Required</sup> <a name="operationLookup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.operationLookup"></a>

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>>

Details about each operation.

---

##### `corsOptions`<sup>Optional</sup> <a name="corsOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.corsOptions"></a>

- *Type:* software.amazon.awscdk.services.apigateway.CorsOptions

Cross Origin Resource Sharing options for the API.

---

##### `defaultAuthorizer`<sup>Optional</sup> <a name="defaultAuthorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.defaultAuthorizer"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The default authorizer to use for your api.

When omitted, no default authorizer is used.
Authorizers specified at the integration level will override this for that operation.

---

##### `specPath`<sup>Required</sup> <a name="specPath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.specPath"></a>

- *Type:* java.lang.String

Path to the JSON open api spec.

---

##### `webAclOptions`<sup>Optional</sup> <a name="webAclOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.webAclOptions"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a>

Options for the AWS WAF v2 WebACL associated with the api.

By default, a Web ACL with the AWS default managed
rule set will be associated with the API. These options may disable or override the defaults.

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.toString">toString</a></code> | Returns a string representation of this construct. |

---

##### `toString` <a name="toString" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.toString"></a>

```java
public java.lang.String toString()
```

Returns a string representation of this construct.

#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct">isConstruct</a></code> | Checks if `x` is a construct. |

---

##### `isConstruct` <a name="isConstruct" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeRestApi;

TypeSafeRestApi.isConstruct(java.lang.Object x)
```

Checks if `x` is a construct.

Use this method instead of `instanceof` to properly detect `Construct`
instances, even when the construct library is symlinked.

Explanation: in JavaScript, multiple copies of the `constructs` library on
disk are seen as independent, completely different libraries. As a
consequence, the class `Construct` in each copy of the `constructs` library
is seen as a different class, and an instance of one class will not test as
`instanceof` the other class. `npm install` will not create installations
like this, but users may manually symlink construct libraries together or
use a monorepo tool: in those cases, multiple copies of the `constructs`
library can be accidentally installed, and `instanceof` will behave
unpredictably. It is safest to avoid using `instanceof`, and using
this type-testing method instead.

###### `x`<sup>Required</sup> <a name="x" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct.parameter.x"></a>

- *Type:* java.lang.Object

Any object.

---

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.node">node</a></code> | <code>software.constructs.Node</code> | The tree node. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.api">api</a></code> | <code>software.amazon.awscdk.services.apigateway.SpecRestApi</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.ipSet">ipSet</a></code> | <code>software.amazon.awscdk.services.wafv2.CfnIPSet</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAcl">webAcl</a></code> | <code>software.amazon.awscdk.services.wafv2.CfnWebACL</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAclAssociation">webAclAssociation</a></code> | <code>software.amazon.awscdk.services.wafv2.CfnWebACLAssociation</code> | *No description.* |

---

##### `node`<sup>Required</sup> <a name="node" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.node"></a>

```java
public Node getNode();
```

- *Type:* software.constructs.Node

The tree node.

---

##### `api`<sup>Required</sup> <a name="api" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.api"></a>

```java
public SpecRestApi getApi();
```

- *Type:* software.amazon.awscdk.services.apigateway.SpecRestApi

---

##### `ipSet`<sup>Optional</sup> <a name="ipSet" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.ipSet"></a>

```java
public CfnIPSet getIpSet();
```

- *Type:* software.amazon.awscdk.services.wafv2.CfnIPSet

---

##### `webAcl`<sup>Optional</sup> <a name="webAcl" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAcl"></a>

```java
public CfnWebACL getWebAcl();
```

- *Type:* software.amazon.awscdk.services.wafv2.CfnWebACL

---

##### `webAclAssociation`<sup>Optional</sup> <a name="webAclAssociation" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAclAssociation"></a>

```java
public CfnWebACLAssociation getWebAclAssociation();
```

- *Type:* software.amazon.awscdk.services.wafv2.CfnWebACLAssociation

---


## Structs <a name="Structs" id="Structs"></a>

### ApiGatewayIntegration <a name="ApiGatewayIntegration" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration"></a>

Represents an api gateway integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ApiGatewayIntegration;

ApiGatewayIntegration.builder()
//  .cacheKeyParameters(java.util.List<java.lang.String>)
//  .cacheNamespace(java.lang.String)
//  .connectionId(java.lang.String)
//  .connectionType(java.lang.String)
//  .contentHandling(java.lang.String)
//  .credentials(java.lang.String)
//  .httpMethod(java.lang.String)
//  .passthroughBehavior(java.lang.String)
//  .requestParameters(java.util.Map<java.lang.String, java.lang.String>)
//  .requestTemplates(java.util.Map<java.lang.String, java.lang.String>)
//  .responses(java.util.Map<java.lang.String, ApiGatewayIntegrationResponse>)
//  .timeoutInMillis(java.lang.Number)
//  .tlsConfig(ApiGatewayIntegrationTlsConfig)
//  .type(java.lang.String)
//  .uri(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheKeyParameters">cacheKeyParameters</a></code> | <code>java.util.List<java.lang.String></code> | A list of request parameters whose values are to be cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheNamespace">cacheNamespace</a></code> | <code>java.lang.String</code> | An API-specific tag group of related cached parameters. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionId">connectionId</a></code> | <code>java.lang.String</code> | The ID of a VpcLink for the private integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionType">connectionType</a></code> | <code>java.lang.String</code> | The integration connection type. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.contentHandling">contentHandling</a></code> | <code>java.lang.String</code> | Request payload encoding conversion types. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.credentials">credentials</a></code> | <code>java.lang.String</code> | For AWS IAM role-based credentials, specify the ARN of an appropriate IAM role. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.httpMethod">httpMethod</a></code> | <code>java.lang.String</code> | The HTTP method used in the integration request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.passthroughBehavior">passthroughBehavior</a></code> | <code>java.lang.String</code> | Specifies how a request payload of unmapped content type is passed through the integration request without modification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestParameters">requestParameters</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Specifies mappings from method request parameters to integration request parameters. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestTemplates">requestTemplates</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Mapping templates for a request payload of specified MIME types. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.responses">responses</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse">ApiGatewayIntegrationResponse</a>></code> | Defines the method's responses and specifies desired parameter mappings or payload mappings from integration responses to method responses. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.timeoutInMillis">timeoutInMillis</a></code> | <code>java.lang.Number</code> | Custom timeout between 50 and 29,000 milliseconds. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.tlsConfig">tlsConfig</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig">ApiGatewayIntegrationTlsConfig</a></code> | Specifies the TLS configuration for an integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.type">type</a></code> | <code>java.lang.String</code> | The type of integration with the specified backend. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.uri">uri</a></code> | <code>java.lang.String</code> | The endpoint URI of the backend. |

---

##### `cacheKeyParameters`<sup>Optional</sup> <a name="cacheKeyParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheKeyParameters"></a>

```java
public java.util.List<java.lang.String> getCacheKeyParameters();
```

- *Type:* java.util.List<java.lang.String>

A list of request parameters whose values are to be cached.

---

##### `cacheNamespace`<sup>Optional</sup> <a name="cacheNamespace" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheNamespace"></a>

```java
public java.lang.String getCacheNamespace();
```

- *Type:* java.lang.String

An API-specific tag group of related cached parameters.

---

##### `connectionId`<sup>Optional</sup> <a name="connectionId" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionId"></a>

```java
public java.lang.String getConnectionId();
```

- *Type:* java.lang.String

The ID of a VpcLink for the private integration.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_VpcLink.html](https://docs.aws.amazon.com/apigateway/latest/api/API_VpcLink.html)

---

##### `connectionType`<sup>Optional</sup> <a name="connectionType" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionType"></a>

```java
public java.lang.String getConnectionType();
```

- *Type:* java.lang.String

The integration connection type.

The valid value is "VPC_LINK" for private integration or "INTERNET", otherwise.

---

##### `contentHandling`<sup>Optional</sup> <a name="contentHandling" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.contentHandling"></a>

```java
public java.lang.String getContentHandling();
```

- *Type:* java.lang.String

Request payload encoding conversion types.

Valid values are 1) CONVERT_TO_TEXT, for converting a binary payload
into a base64-encoded string or converting a text payload into a utf-8-encoded string or passing through the text
payload natively without modification, and 2) CONVERT_TO_BINARY, for converting a text payload into a
base64-decoded blob or passing through a binary payload natively without modification.

---

##### `credentials`<sup>Optional</sup> <a name="credentials" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.credentials"></a>

```java
public java.lang.String getCredentials();
```

- *Type:* java.lang.String

For AWS IAM role-based credentials, specify the ARN of an appropriate IAM role.

If unspecified, credentials default
to resource-based permissions that must be added manually to allow the API to access the resource. For more
information, see Granting Permissions Using a Resource Policy.

Note: When using IAM credentials, make sure that AWS STS Regional endpoints are enabled for the Region where this
API is deployed for best performance.

---

##### `httpMethod`<sup>Optional</sup> <a name="httpMethod" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.httpMethod"></a>

```java
public java.lang.String getHttpMethod();
```

- *Type:* java.lang.String

The HTTP method used in the integration request.

For Lambda function invocations, the value must be POST.

---

##### `passthroughBehavior`<sup>Optional</sup> <a name="passthroughBehavior" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.passthroughBehavior"></a>

```java
public java.lang.String getPassthroughBehavior();
```

- *Type:* java.lang.String

Specifies how a request payload of unmapped content type is passed through the integration request without modification.

Supported values are when_no_templates, when_no_match, and never.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#passthroughBehavior](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#passthroughBehavior)

---

##### `requestParameters`<sup>Optional</sup> <a name="requestParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestParameters"></a>

```java
public java.util.Map<java.lang.String, java.lang.String> getRequestParameters();
```

- *Type:* java.util.Map<java.lang.String, java.lang.String>

Specifies mappings from method request parameters to integration request parameters.

Supported request parameters
are querystring, path, header, and body.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestParameters.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestParameters.html)

---

##### `requestTemplates`<sup>Optional</sup> <a name="requestTemplates" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestTemplates"></a>

```java
public java.util.Map<java.lang.String, java.lang.String> getRequestTemplates();
```

- *Type:* java.util.Map<java.lang.String, java.lang.String>

Mapping templates for a request payload of specified MIME types.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestTemplates.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestTemplates.html)

---

##### `responses`<sup>Optional</sup> <a name="responses" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.responses"></a>

```java
public java.util.Map<java.lang.String, ApiGatewayIntegrationResponse> getResponses();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse">ApiGatewayIntegrationResponse</a>>

Defines the method's responses and specifies desired parameter mappings or payload mappings from integration responses to method responses.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-responses.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-responses.html)

---

##### `timeoutInMillis`<sup>Optional</sup> <a name="timeoutInMillis" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.timeoutInMillis"></a>

```java
public java.lang.Number getTimeoutInMillis();
```

- *Type:* java.lang.Number

Custom timeout between 50 and 29,000 milliseconds.

The default value is 29,000 milliseconds or 29 seconds.

---

##### `tlsConfig`<sup>Optional</sup> <a name="tlsConfig" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.tlsConfig"></a>

```java
public ApiGatewayIntegrationTlsConfig getTlsConfig();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig">ApiGatewayIntegrationTlsConfig</a>

Specifies the TLS configuration for an integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html)

---

##### `type`<sup>Optional</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.type"></a>

```java
public java.lang.String getType();
```

- *Type:* java.lang.String

The type of integration with the specified backend.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#type](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#type)

---

##### `uri`<sup>Optional</sup> <a name="uri" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.uri"></a>

```java
public java.lang.String getUri();
```

- *Type:* java.lang.String

The endpoint URI of the backend.

For integrations of the aws type, this is an ARN value.
For the HTTP integration, this is the URL of the HTTP endpoint including the https or http scheme.

---

### ApiGatewayIntegrationResponse <a name="ApiGatewayIntegrationResponse" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse"></a>

API Gateway integration response.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ApiGatewayIntegrationResponse;

ApiGatewayIntegrationResponse.builder()
    .responseParameters(java.util.Map<java.lang.String, java.lang.String>)
    .responseTemplates(java.util.Map<java.lang.String, java.lang.String>)
    .statusCode(java.lang.String)
//  .contentHandling(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseParameters">responseParameters</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Specifies parameter mappings for the response. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseTemplates">responseTemplates</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Specifies MIME type-specific mapping templates for the response’s payload. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.statusCode">statusCode</a></code> | <code>java.lang.String</code> | HTTP status code for the method response. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.contentHandling">contentHandling</a></code> | <code>java.lang.String</code> | Response payload encoding conversion types. |

---

##### `responseParameters`<sup>Required</sup> <a name="responseParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseParameters"></a>

```java
public java.util.Map<java.lang.String, java.lang.String> getResponseParameters();
```

- *Type:* java.util.Map<java.lang.String, java.lang.String>

Specifies parameter mappings for the response.

---

##### `responseTemplates`<sup>Required</sup> <a name="responseTemplates" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseTemplates"></a>

```java
public java.util.Map<java.lang.String, java.lang.String> getResponseTemplates();
```

- *Type:* java.util.Map<java.lang.String, java.lang.String>

Specifies MIME type-specific mapping templates for the response’s payload.

---

##### `statusCode`<sup>Required</sup> <a name="statusCode" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.statusCode"></a>

```java
public java.lang.String getStatusCode();
```

- *Type:* java.lang.String

HTTP status code for the method response.

---

##### `contentHandling`<sup>Optional</sup> <a name="contentHandling" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.contentHandling"></a>

```java
public java.lang.String getContentHandling();
```

- *Type:* java.lang.String

Response payload encoding conversion types.

Valid values are 1) CONVERT_TO_TEXT, for converting a binary payload
into a base64-encoded string or converting a text payload into a utf-8-encoded string or passing through the text
payload natively without modification, and 2) CONVERT_TO_BINARY, for converting a text payload into a
base64-decoded blob or passing through a binary payload natively without modification.

---

### ApiGatewayIntegrationTlsConfig <a name="ApiGatewayIntegrationTlsConfig" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig"></a>

Specifies the TLS configuration for an integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ApiGatewayIntegrationTlsConfig;

ApiGatewayIntegrationTlsConfig.builder()
//  .insecureSkipVerification(java.lang.Boolean)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.property.insecureSkipVerification">insecureSkipVerification</a></code> | <code>java.lang.Boolean</code> | Specifies whether or not API Gateway skips verification that the certificate for an integration endpoint is issued by a supported certificate authority. |

---

##### `insecureSkipVerification`<sup>Optional</sup> <a name="insecureSkipVerification" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.property.insecureSkipVerification"></a>

```java
public java.lang.Boolean getInsecureSkipVerification();
```

- *Type:* java.lang.Boolean

Specifies whether or not API Gateway skips verification that the certificate for an integration endpoint is issued by a supported certificate authority.

This isn’t recommended, but it enables you to use certificates that are
signed by private certificate authorities, or certificates that are self-signed. If enabled, API Gateway still
performs basic certificate validation, which includes checking the certificate's expiration date, hostname, and
presence of a root certificate authority. Supported only for HTTP and HTTP_PROXY integrations.

---

### AuthorizerProps <a name="AuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps"></a>

Properties for an authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.AuthorizerProps;

AuthorizerProps.builder()
    .authorizationType(AuthorizationType)
    .authorizerId(java.lang.String)
//  .authorizationScopes(java.util.List<java.lang.String>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---

### CidrAllowList <a name="CidrAllowList" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList"></a>

Representation of a CIDR range.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.CidrAllowList;

CidrAllowList.builder()
    .cidrRanges(java.util.List<java.lang.String>)
    .cidrType(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrRanges">cidrRanges</a></code> | <code>java.util.List<java.lang.String></code> | Specify an IPv4 address by using CIDR notation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrType">cidrType</a></code> | <code>java.lang.String</code> | Type of CIDR range. |

---

##### `cidrRanges`<sup>Required</sup> <a name="cidrRanges" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrRanges"></a>

```java
public java.util.List<java.lang.String> getCidrRanges();
```

- *Type:* java.util.List<java.lang.String>

Specify an IPv4 address by using CIDR notation.

For example:
To configure AWS WAF to allow, block, or count requests that originated from the IP address 192.0.2.44, specify 192.0.2.44/32 .
To configure AWS WAF to allow, block, or count requests that originated from IP addresses from 192.0.2.0 to 192.0.2.255, specify 192.0.2.0/24 .

For more information about CIDR notation, see the Wikipedia entry Classless Inter-Domain Routing .

Specify an IPv6 address by using CIDR notation. For example:
To configure AWS WAF to allow, block, or count requests that originated from the IP address 1111:0000:0000:0000:0000:0000:0000:0111, specify 1111:0000:0000:0000:0000:0000:0000:0111/128 .
To configure AWS WAF to allow, block, or count requests that originated from IP addresses 1111:0000:0000:0000:0000:0000:0000:0000 to 1111:0000:0000:0000:ffff:ffff:ffff:ffff, specify 1111:0000:0000:0000:0000:0000:0000:0000/64 .

---

##### `cidrType`<sup>Required</sup> <a name="cidrType" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrType"></a>

```java
public java.lang.String getCidrType();
```

- *Type:* java.lang.String

Type of CIDR range.

---

### CognitoAuthorizerProps <a name="CognitoAuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps"></a>

Properties used to configure a cognito authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.CognitoAuthorizerProps;

CognitoAuthorizerProps.builder()
    .authorizerId(java.lang.String)
    .userPools(java.util.List<IUserPool>)
//  .authorizationScopes(java.util.List<java.lang.String>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.userPools">userPools</a></code> | <code>java.util.List<software.amazon.awscdk.services.cognito.IUserPool></code> | The Cognito user pools associated with this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | A list of authorization scopes configured on the method. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

Unique identifier for this authorizer.

---

##### `userPools`<sup>Required</sup> <a name="userPools" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.userPools"></a>

```java
public java.util.List<IUserPool> getUserPools();
```

- *Type:* java.util.List<software.amazon.awscdk.services.cognito.IUserPool>

The Cognito user pools associated with this authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>
- *Default:* []

A list of authorization scopes configured on the method.

When used as the default authorizer, these scopes will be
applied to all methods without an authorizer at the integration level.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes)

---

### CustomAuthorizerProps <a name="CustomAuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps"></a>

Properties used to configure a custom authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.CustomAuthorizerProps;

CustomAuthorizerProps.builder()
    .authorizerId(java.lang.String)
    .function(IFunction)
//  .authorizerResultTtlInSeconds(java.lang.Number)
//  .identitySource(java.lang.String)
//  .type(CustomAuthorizerType)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.function">function</a></code> | <code>software.amazon.awscdk.services.lambda.IFunction</code> | The lambda function used to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerResultTtlInSeconds">authorizerResultTtlInSeconds</a></code> | <code>java.lang.Number</code> | The number of seconds during which the authorizer result is cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.identitySource">identitySource</a></code> | <code>java.lang.String</code> | The source of the identity in an incoming request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.type">type</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a></code> | The type of custom authorizer. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

Unique identifier for this authorizer.

---

##### `function`<sup>Required</sup> <a name="function" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.function"></a>

```java
public IFunction getFunction();
```

- *Type:* software.amazon.awscdk.services.lambda.IFunction

The lambda function used to authorize requests.

---

##### `authorizerResultTtlInSeconds`<sup>Optional</sup> <a name="authorizerResultTtlInSeconds" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerResultTtlInSeconds"></a>

```java
public java.lang.Number getAuthorizerResultTtlInSeconds();
```

- *Type:* java.lang.Number
- *Default:* 300

The number of seconds during which the authorizer result is cached.

---

##### `identitySource`<sup>Optional</sup> <a name="identitySource" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.identitySource"></a>

```java
public java.lang.String getIdentitySource();
```

- *Type:* java.lang.String
- *Default:* "method.request.header.Authorization"

The source of the identity in an incoming request.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource)

---

##### `type`<sup>Optional</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.type"></a>

```java
public CustomAuthorizerType getType();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a>
- *Default:* CustomAuthorizerType.TOKEN

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

---

### DocumentationConfiguration <a name="DocumentationConfiguration" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration"></a>

Configuration for generated documentation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.DocumentationConfiguration;

DocumentationConfiguration.builder()
    .formats(java.util.List<DocumentationFormat>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.property.formats">formats</a></code> | <code>java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat">DocumentationFormat</a>></code> | Formats for generated documentation. |

---

##### `formats`<sup>Required</sup> <a name="formats" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.property.formats"></a>

```java
public java.util.List<DocumentationFormat> getFormats();
```

- *Type:* java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat">DocumentationFormat</a>>

Formats for generated documentation.

---

### GeneratedCodeOptions <a name="GeneratedCodeOptions" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions"></a>

Options for generated clients.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.GeneratedCodeOptions;

GeneratedCodeOptions.builder()
//  .java(JavaProjectOptions)
//  .python(PythonProjectOptions)
//  .typescript(TypeScriptProjectOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.java">java</a></code> | <code>io.github.cdklabs.projen.java.JavaProjectOptions</code> | Options for a generated java project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.python">python</a></code> | <code>io.github.cdklabs.projen.python.PythonProjectOptions</code> | Options for a generated python project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.typescript">typescript</a></code> | <code>io.github.cdklabs.projen.typescript.TypeScriptProjectOptions</code> | Options for a generated typescript project. |

---

##### `java`<sup>Optional</sup> <a name="java" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.java"></a>

```java
public JavaProjectOptions getJava();
```

- *Type:* io.github.cdklabs.projen.java.JavaProjectOptions

Options for a generated java project.

These override the default inferred options.

---

##### `python`<sup>Optional</sup> <a name="python" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.python"></a>

```java
public PythonProjectOptions getPython();
```

- *Type:* io.github.cdklabs.projen.python.PythonProjectOptions

Options for a generated python project.

These override the default inferred options.

---

##### `typescript`<sup>Optional</sup> <a name="typescript" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.typescript"></a>

```java
public TypeScriptProjectOptions getTypescript();
```

- *Type:* io.github.cdklabs.projen.typescript.TypeScriptProjectOptions

Options for a generated typescript project.

These override the default inferred options.

---

### GeneratedCodeProjects <a name="GeneratedCodeProjects" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects"></a>

Generated code projects.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.GeneratedCodeProjects;

GeneratedCodeProjects.builder()
//  .java(JavaProject)
//  .python(PythonProject)
//  .typescript(TypeScriptProject)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.java">java</a></code> | <code>io.github.cdklabs.projen.java.JavaProject</code> | Generated java project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.python">python</a></code> | <code>io.github.cdklabs.projen.python.PythonProject</code> | Generated python project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.typescript">typescript</a></code> | <code>io.github.cdklabs.projen.typescript.TypeScriptProject</code> | Generated typescript project. |

---

##### `java`<sup>Optional</sup> <a name="java" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.java"></a>

```java
public JavaProject getJava();
```

- *Type:* io.github.cdklabs.projen.java.JavaProject

Generated java project.

---

##### `python`<sup>Optional</sup> <a name="python" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.python"></a>

```java
public PythonProject getPython();
```

- *Type:* io.github.cdklabs.projen.python.PythonProject

Generated python project.

---

##### `typescript`<sup>Optional</sup> <a name="typescript" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.typescript"></a>

```java
public TypeScriptProject getTypescript();
```

- *Type:* io.github.cdklabs.projen.typescript.TypeScriptProject

Generated typescript project.

---

### InfrastructureConfiguration <a name="InfrastructureConfiguration" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration"></a>

Configuration for generated infrastructure.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.InfrastructureConfiguration;

InfrastructureConfiguration.builder()
    .language(Language)
//  .options(GeneratedCodeOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.language">language</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a></code> | The language to generate the type-safe CDK infrastructure in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a></code> | Options for the infrastructure package. |

---

##### `language`<sup>Required</sup> <a name="language" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.language"></a>

```java
public Language getLanguage();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>

The language to generate the type-safe CDK infrastructure in.

---

##### `options`<sup>Optional</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.options"></a>

```java
public GeneratedCodeOptions getOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a>

Options for the infrastructure package.

Note that only those provided for the specified language will apply.

---

### IntegrationGrantProps <a name="IntegrationGrantProps" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps"></a>

Properties for granting the API access to invoke the operation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.IntegrationGrantProps;

IntegrationGrantProps.builder()
    .method(java.lang.String)
    .path(java.lang.String)
    .api(SpecRestApi)
    .operationId(java.lang.String)
    .scope(Construct)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.method">method</a></code> | <code>java.lang.String</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.path">path</a></code> | <code>java.lang.String</code> | The path of this operation in the api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.api">api</a></code> | <code>software.amazon.awscdk.services.apigateway.SpecRestApi</code> | The api to grant permissions for. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.operationId">operationId</a></code> | <code>java.lang.String</code> | The ID of the operation for which permissions are being granted. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.scope">scope</a></code> | <code>software.constructs.Construct</code> | The scope in which permission resources can be created. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.method"></a>

```java
public java.lang.String getMethod();
```

- *Type:* java.lang.String

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.path"></a>

```java
public java.lang.String getPath();
```

- *Type:* java.lang.String

The path of this operation in the api.

---

##### `api`<sup>Required</sup> <a name="api" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.api"></a>

```java
public SpecRestApi getApi();
```

- *Type:* software.amazon.awscdk.services.apigateway.SpecRestApi

The api to grant permissions for.

---

##### `operationId`<sup>Required</sup> <a name="operationId" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.operationId"></a>

```java
public java.lang.String getOperationId();
```

- *Type:* java.lang.String

The ID of the operation for which permissions are being granted.

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.scope"></a>

```java
public Construct getScope();
```

- *Type:* software.constructs.Construct

The scope in which permission resources can be created.

---

### IntegrationRenderProps <a name="IntegrationRenderProps" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps"></a>

Properties for rendering an integration into an API Gateway OpenAPI extension.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.IntegrationRenderProps;

IntegrationRenderProps.builder()
    .method(java.lang.String)
    .path(java.lang.String)
    .operationId(java.lang.String)
    .scope(Construct)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.method">method</a></code> | <code>java.lang.String</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.path">path</a></code> | <code>java.lang.String</code> | The path of this operation in the api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.operationId">operationId</a></code> | <code>java.lang.String</code> | The ID of the operation being rendered. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.scope">scope</a></code> | <code>software.constructs.Construct</code> | The scope in which the integration is being rendered. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.method"></a>

```java
public java.lang.String getMethod();
```

- *Type:* java.lang.String

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.path"></a>

```java
public java.lang.String getPath();
```

- *Type:* java.lang.String

The path of this operation in the api.

---

##### `operationId`<sup>Required</sup> <a name="operationId" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.operationId"></a>

```java
public java.lang.String getOperationId();
```

- *Type:* java.lang.String

The ID of the operation being rendered.

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.scope"></a>

```java
public Construct getScope();
```

- *Type:* software.constructs.Construct

The scope in which the integration is being rendered.

---

### ManagedRule <a name="ManagedRule" id="@aws-prototyping-sdk/type-safe-api.ManagedRule"></a>

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ManagedRule;

ManagedRule.builder()
    .name(java.lang.String)
    .vendor(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule.property.name">name</a></code> | <code>java.lang.String</code> | The name of the managed rule group. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule.property.vendor">vendor</a></code> | <code>java.lang.String</code> | The name of the managed rule group vendor. |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String

The name of the managed rule group.

You use this, along with the vendor name, to identify the rule group.

---

##### `vendor`<sup>Required</sup> <a name="vendor" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.property.vendor"></a>

```java
public java.lang.String getVendor();
```

- *Type:* java.lang.String

The name of the managed rule group vendor.

You use this, along with the rule group name, to identify the rule group.

---

### MethodAndPath <a name="MethodAndPath" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath"></a>

Structure to contain an API operation's method and path.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.MethodAndPath;

MethodAndPath.builder()
    .method(java.lang.String)
    .path(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.method">method</a></code> | <code>java.lang.String</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.path">path</a></code> | <code>java.lang.String</code> | The path of this operation in the api. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.method"></a>

```java
public java.lang.String getMethod();
```

- *Type:* java.lang.String

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.path"></a>

```java
public java.lang.String getPath();
```

- *Type:* java.lang.String

The path of this operation in the api.

---

### ModelConfiguration <a name="ModelConfiguration" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration"></a>

Configuration for modelling the API.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ModelConfiguration;

ModelConfiguration.builder()
    .language(ModelLanguage)
    .options(ModelOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.language">language</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a></code> | The language the API model is defined in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a></code> | Options for the API model. |

---

##### `language`<sup>Required</sup> <a name="language" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.language"></a>

```java
public ModelLanguage getLanguage();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a>

The language the API model is defined in.

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.options"></a>

```java
public ModelOptions getOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a>

Options for the API model.

---

### ModelOptions <a name="ModelOptions" id="@aws-prototyping-sdk/type-safe-api.ModelOptions"></a>

Options for models.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.ModelOptions;

ModelOptions.builder()
//  .openapi(OpenApiModelOptions)
//  .smithy(SmithyModelOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions.property.openapi">openapi</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a></code> | Options for the OpenAPI model - required when model language is OPENAPI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions.property.smithy">smithy</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a></code> | Options for the Smithy model - required when model language is SMITHY. |

---

##### `openapi`<sup>Optional</sup> <a name="openapi" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.property.openapi"></a>

```java
public OpenApiModelOptions getOpenapi();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a>

Options for the OpenAPI model - required when model language is OPENAPI.

---

##### `smithy`<sup>Optional</sup> <a name="smithy" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.property.smithy"></a>

```java
public SmithyModelOptions getSmithy();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a>

Options for the Smithy model - required when model language is SMITHY.

---

### OpenApiDefinitionOptions <a name="OpenApiDefinitionOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions"></a>

Options for the OpenAPI Spec.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.OpenApiDefinitionOptions;

OpenApiDefinitionOptions.builder()
    .openApiOptions(OpenApiModelOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.property.openApiOptions">openApiOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a></code> | Options for the openapi model. |

---

##### `openApiOptions`<sup>Required</sup> <a name="openApiOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.property.openApiOptions"></a>

```java
public OpenApiModelOptions getOpenApiOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a>

Options for the openapi model.

---

### OpenApiModelOptions <a name="OpenApiModelOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions"></a>

Options for the OpenAPI model.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.OpenApiModelOptions;

OpenApiModelOptions.builder()
    .title(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.property.title">title</a></code> | <code>java.lang.String</code> | The title in the OpenAPI specification. |

---

##### `title`<sup>Required</sup> <a name="title" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.property.title"></a>

```java
public java.lang.String getTitle();
```

- *Type:* java.lang.String

The title in the OpenAPI specification.

---

### RuntimeConfiguration <a name="RuntimeConfiguration" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration"></a>

Configuration for generated runtime projects.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.RuntimeConfiguration;

RuntimeConfiguration.builder()
    .languages(java.util.List<Language>)
//  .options(GeneratedCodeOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.languages">languages</a></code> | <code>java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>></code> | The languages that runtime projects will be generated in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a></code> | Options for the generated runtimes. |

---

##### `languages`<sup>Required</sup> <a name="languages" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.languages"></a>

```java
public java.util.List<Language> getLanguages();
```

- *Type:* java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>>

The languages that runtime projects will be generated in.

These projects can be used to provide type safety for
both client and server projects.

---

##### `options`<sup>Optional</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.options"></a>

```java
public GeneratedCodeOptions getOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a>

Options for the generated runtimes.

Note that only options provided for the specified languages will apply.

---

### SmithyBuildOptions <a name="SmithyBuildOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions"></a>

Options for the smithy build files.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyBuildOptions;

SmithyBuildOptions.builder()
//  .imports(java.util.List<java.lang.String>)
//  .plugins(java.util.Map<java.lang.String, SmithyPlugin>)
//  .ignoreMissingPlugins(java.lang.Boolean)
//  .maven(SmithyMavenConfiguration)
//  .projections(java.util.Map<java.lang.String, SmithyProjection>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.imports">imports</a></code> | <code>java.util.List<java.lang.String></code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.plugins">plugins</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>></code> | Plugins keyed by plugin id. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.ignoreMissingPlugins">ignoreMissingPlugins</a></code> | <code>java.lang.Boolean</code> | If a plugin can't be found, Smithy will by default fail the build. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.maven">maven</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration">SmithyMavenConfiguration</a></code> | Maven configuration for the Smithy build project, used to specify dependencies and repositories in the build.gradle and smithy-build.json files. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.projections">projections</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection">SmithyProjection</a>></code> | Map of projections name to projection configurations https://awslabs.github.io/smithy/2.0/guides/building-models/build-config.html#projections. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.imports"></a>

```java
public java.util.List<java.lang.String> getImports();
```

- *Type:* java.util.List<java.lang.String>

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.plugins"></a>

```java
public java.util.Map<java.lang.String, SmithyPlugin> getPlugins();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>>

Plugins keyed by plugin id.

---

##### `ignoreMissingPlugins`<sup>Optional</sup> <a name="ignoreMissingPlugins" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.ignoreMissingPlugins"></a>

```java
public java.lang.Boolean getIgnoreMissingPlugins();
```

- *Type:* java.lang.Boolean
- *Default:* no ignoreMissingPlugins set in the smithy-build.json file

If a plugin can't be found, Smithy will by default fail the build.

This setting can be set to true to allow the build to progress
even if a plugin can't be found on the classpath.

---

##### `maven`<sup>Optional</sup> <a name="maven" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.maven"></a>

```java
public SmithyMavenConfiguration getMaven();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration">SmithyMavenConfiguration</a>
- *Default:* the default configuration required for Smithy to OpenAPI conversion

Maven configuration for the Smithy build project, used to specify dependencies and repositories in the build.gradle and smithy-build.json files.

---

##### `projections`<sup>Optional</sup> <a name="projections" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.projections"></a>

```java
public java.util.Map<java.lang.String, SmithyProjection> getProjections();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection">SmithyProjection</a>>
- *Default:* no projections

Map of projections name to projection configurations https://awslabs.github.io/smithy/2.0/guides/building-models/build-config.html#projections.

---

### SmithyCommon <a name="SmithyCommon" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon"></a>

Properties common to smithy plugins and the root smithy build.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyCommon;

SmithyCommon.builder()
//  .imports(java.util.List<java.lang.String>)
//  .plugins(java.util.Map<java.lang.String, SmithyPlugin>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.imports">imports</a></code> | <code>java.util.List<java.lang.String></code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.plugins">plugins</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>></code> | Plugins keyed by plugin id. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.imports"></a>

```java
public java.util.List<java.lang.String> getImports();
```

- *Type:* java.util.List<java.lang.String>

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.plugins"></a>

```java
public java.util.Map<java.lang.String, SmithyPlugin> getPlugins();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>>

Plugins keyed by plugin id.

---

### SmithyDefinitionOptions <a name="SmithyDefinitionOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions"></a>

Options for a smithy build project.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyDefinitionOptions;

SmithyDefinitionOptions.builder()
    .smithyOptions(SmithyModelOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.property.smithyOptions">smithyOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a></code> | Smithy engine options. |

---

##### `smithyOptions`<sup>Required</sup> <a name="smithyOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.property.smithyOptions"></a>

```java
public SmithyModelOptions getSmithyOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a>

Smithy engine options.

---

### SmithyMavenConfiguration <a name="SmithyMavenConfiguration" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration"></a>

Configuration for smithy maven dependencies.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyMavenConfiguration;

SmithyMavenConfiguration.builder()
//  .dependencies(java.util.List<java.lang.String>)
//  .repositoryUrls(java.util.List<java.lang.String>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.dependencies">dependencies</a></code> | <code>java.util.List<java.lang.String></code> | The dependencies used in the build.gradle and smithy-build.json files eg. software.amazon.smithy:smithy-validation-model:1.27.2 The following required dependencies are always added: - software.amazon.smithy:smithy-cli:1.27.2 - software.amazon.smithy:smithy-model:1.27.2 - software.amazon.smithy:smithy-openapi:1.27.2 - software.amazon.smithy:smithy-aws-traits:1.27.2 You can however override the version of these dependencies if required. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.repositoryUrls">repositoryUrls</a></code> | <code>java.util.List<java.lang.String></code> | The repository urls used in the build.gradle and smithy-build.json files. |

---

##### `dependencies`<sup>Optional</sup> <a name="dependencies" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.dependencies"></a>

```java
public java.util.List<java.lang.String> getDependencies();
```

- *Type:* java.util.List<java.lang.String>

The dependencies used in the build.gradle and smithy-build.json files eg. software.amazon.smithy:smithy-validation-model:1.27.2 The following required dependencies are always added: - software.amazon.smithy:smithy-cli:1.27.2 - software.amazon.smithy:smithy-model:1.27.2 - software.amazon.smithy:smithy-openapi:1.27.2 - software.amazon.smithy:smithy-aws-traits:1.27.2 You can however override the version of these dependencies if required.

---

##### `repositoryUrls`<sup>Optional</sup> <a name="repositoryUrls" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.repositoryUrls"></a>

```java
public java.util.List<java.lang.String> getRepositoryUrls();
```

- *Type:* java.util.List<java.lang.String>
- *Default:* maven central and maven local

The repository urls used in the build.gradle and smithy-build.json files.

---

### SmithyModelOptions <a name="SmithyModelOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions"></a>

Options for a Smithy model.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyModelOptions;

SmithyModelOptions.builder()
    .serviceName(SmithyServiceName)
//  .ignoreGradleWrapper(java.lang.Boolean)
//  .ignoreSmithyBuildOutput(java.lang.Boolean)
//  .smithyBuildOptions(SmithyBuildOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.serviceName">serviceName</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName">SmithyServiceName</a></code> | Smithy service name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreGradleWrapper">ignoreGradleWrapper</a></code> | <code>java.lang.Boolean</code> | Set to false if you would like to check in your gradle wrapper. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreSmithyBuildOutput">ignoreSmithyBuildOutput</a></code> | <code>java.lang.Boolean</code> | Set to false if you would like to check in your smithy build output or have more fine-grained control over what is checked in, eg if you add other projections to the smithy-build.json file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.smithyBuildOptions">smithyBuildOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions">SmithyBuildOptions</a></code> | Smithy build options. |

---

##### `serviceName`<sup>Required</sup> <a name="serviceName" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.serviceName"></a>

```java
public SmithyServiceName getServiceName();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName">SmithyServiceName</a>

Smithy service name.

---

##### `ignoreGradleWrapper`<sup>Optional</sup> <a name="ignoreGradleWrapper" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreGradleWrapper"></a>

```java
public java.lang.Boolean getIgnoreGradleWrapper();
```

- *Type:* java.lang.Boolean
- *Default:* true

Set to false if you would like to check in your gradle wrapper.

Do so if you would like to use a different version
of gradle to the one provided by default

---

##### `ignoreSmithyBuildOutput`<sup>Optional</sup> <a name="ignoreSmithyBuildOutput" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreSmithyBuildOutput"></a>

```java
public java.lang.Boolean getIgnoreSmithyBuildOutput();
```

- *Type:* java.lang.Boolean
- *Default:* true

Set to false if you would like to check in your smithy build output or have more fine-grained control over what is checked in, eg if you add other projections to the smithy-build.json file.

---

##### `smithyBuildOptions`<sup>Optional</sup> <a name="smithyBuildOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.smithyBuildOptions"></a>

```java
public SmithyBuildOptions getSmithyBuildOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions">SmithyBuildOptions</a>

Smithy build options.

---

### SmithyPlugin <a name="SmithyPlugin" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin"></a>

A smithy plugin.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyPlugin;

SmithyPlugin.builder()
//  .service(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin.property.service">service</a></code> | <code>java.lang.String</code> | The service to which the plugin should apply. |

---

##### `service`<sup>Optional</sup> <a name="service" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin.property.service"></a>

```java
public java.lang.String getService();
```

- *Type:* java.lang.String

The service to which the plugin should apply.

---

### SmithyProjection <a name="SmithyProjection" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection"></a>

A smithy build projection.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyProjection;

SmithyProjection.builder()
//  .imports(java.util.List<java.lang.String>)
//  .plugins(java.util.Map<java.lang.String, SmithyPlugin>)
//  .abstract(java.lang.Boolean)
//  .transforms(java.util.List<SmithyTransform>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.imports">imports</a></code> | <code>java.util.List<java.lang.String></code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.plugins">plugins</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>></code> | Plugins keyed by plugin id. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.abstract">abstract</a></code> | <code>java.lang.Boolean</code> | Whether or not the projection is abstract. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.transforms">transforms</a></code> | <code>java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform">SmithyTransform</a>></code> | Transforms to apply to the projection. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.imports"></a>

```java
public java.util.List<java.lang.String> getImports();
```

- *Type:* java.util.List<java.lang.String>

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.plugins"></a>

```java
public java.util.Map<java.lang.String, SmithyPlugin> getPlugins();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>>

Plugins keyed by plugin id.

---

##### `abstract`<sup>Optional</sup> <a name="abstract" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.abstract"></a>

```java
public java.lang.Boolean getAbstract();
```

- *Type:* java.lang.Boolean

Whether or not the projection is abstract.

---

##### `transforms`<sup>Optional</sup> <a name="transforms" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.transforms"></a>

```java
public java.util.List<SmithyTransform> getTransforms();
```

- *Type:* java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform">SmithyTransform</a>>

Transforms to apply to the projection.

---

### SmithyServiceName <a name="SmithyServiceName" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName"></a>

Represents a fully qualified name of a Smithy service.

> [https://awslabs.github.io/smithy/2.0/spec/service-types.html](https://awslabs.github.io/smithy/2.0/spec/service-types.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyServiceName;

SmithyServiceName.builder()
    .namespace(java.lang.String)
    .serviceName(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.namespace">namespace</a></code> | <code>java.lang.String</code> | The service namespace. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.serviceName">serviceName</a></code> | <code>java.lang.String</code> | The service name. |

---

##### `namespace`<sup>Required</sup> <a name="namespace" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.namespace"></a>

```java
public java.lang.String getNamespace();
```

- *Type:* java.lang.String

The service namespace.

Nested namespaces are separated by '.', for example com.company

> [https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id](https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id)

---

##### `serviceName`<sup>Required</sup> <a name="serviceName" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.serviceName"></a>

```java
public java.lang.String getServiceName();
```

- *Type:* java.lang.String

The service name.

Should be PascalCase, for example HelloService

> [https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id](https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id)

---

### SmithyTransform <a name="SmithyTransform" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform"></a>

A smithy build transform.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyTransform;

SmithyTransform.builder()
    .args(java.util.Map<java.lang.String, java.lang.Object>)
    .name(java.lang.String)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.args">args</a></code> | <code>java.util.Map<java.lang.String, java.lang.Object></code> | Arguments for the transform. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.name">name</a></code> | <code>java.lang.String</code> | Name of the transform. |

---

##### `args`<sup>Required</sup> <a name="args" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.args"></a>

```java
public java.util.Map<java.lang.String, java.lang.Object> getArgs();
```

- *Type:* java.util.Map<java.lang.String, java.lang.Object>

Arguments for the transform.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String

Name of the transform.

---

### TypeSafeApiIntegration <a name="TypeSafeApiIntegration" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration"></a>

Defines an integration for an individual API operation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiIntegration;

TypeSafeApiIntegration.builder()
    .integration(Integration)
//  .authorizer(Authorizer)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.integration">integration</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration">Integration</a></code> | The lambda function to service the api operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.authorizer">authorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The authorizer to use for this api operation (overrides the default). |

---

##### `integration`<sup>Required</sup> <a name="integration" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.integration"></a>

```java
public Integration getIntegration();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Integration">Integration</a>

The lambda function to service the api operation.

---

##### `authorizer`<sup>Optional</sup> <a name="authorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.authorizer"></a>

```java
public Authorizer getAuthorizer();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The authorizer to use for this api operation (overrides the default).

---

### TypeSafeApiModelProjectOptions <a name="TypeSafeApiModelProjectOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions"></a>

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiModelProjectOptions;

TypeSafeApiModelProjectOptions.builder()
    .name(java.lang.String)
//  .commitGenerated(java.lang.Boolean)
//  .gitIgnoreOptions(IgnoreFileOptions)
//  .gitOptions(GitOptions)
//  .logging(LoggerOptions)
//  .outdir(java.lang.String)
//  .parent(Project)
//  .projenCommand(java.lang.String)
//  .projenrcJson(java.lang.Boolean)
//  .projenrcJsonOptions(ProjenrcJsonOptions)
//  .renovatebot(java.lang.Boolean)
//  .renovatebotOptions(RenovatebotOptions)
    .modelLanguage(ModelLanguage)
    .modelOptions(ModelOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.name">name</a></code> | <code>java.lang.String</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>io.github.cdklabs.projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitOptions">gitOptions</a></code> | <code>io.github.cdklabs.projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.logging">logging</a></code> | <code>io.github.cdklabs.projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.outdir">outdir</a></code> | <code>java.lang.String</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJson">projenrcJson</a></code> | <code>java.lang.Boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>io.github.cdklabs.projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebot">renovatebot</a></code> | <code>java.lang.Boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebotOptions">renovatebotOptions</a></code> | <code>io.github.cdklabs.projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelLanguage">modelLanguage</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelOptions">modelOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a></code> | *No description.* |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.commitGenerated"></a>

```java
public java.lang.Boolean getCommitGenerated();
```

- *Type:* java.lang.Boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitIgnoreOptions"></a>

```java
public IgnoreFileOptions getGitIgnoreOptions();
```

- *Type:* io.github.cdklabs.projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitOptions"></a>

```java
public GitOptions getGitOptions();
```

- *Type:* io.github.cdklabs.projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.logging"></a>

```java
public LoggerOptions getLogging();
```

- *Type:* io.github.cdklabs.projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.outdir"></a>

```java
public java.lang.String getOutdir();
```

- *Type:* java.lang.String
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.parent"></a>

```java
public Project getParent();
```

- *Type:* io.github.cdklabs.projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenCommand"></a>

```java
public java.lang.String getProjenCommand();
```

- *Type:* java.lang.String
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJson"></a>

```java
public java.lang.Boolean getProjenrcJson();
```

- *Type:* java.lang.Boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJsonOptions"></a>

```java
public ProjenrcJsonOptions getProjenrcJsonOptions();
```

- *Type:* io.github.cdklabs.projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebot"></a>

```java
public java.lang.Boolean getRenovatebot();
```

- *Type:* java.lang.Boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebotOptions"></a>

```java
public RenovatebotOptions getRenovatebotOptions();
```

- *Type:* io.github.cdklabs.projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `modelLanguage`<sup>Required</sup> <a name="modelLanguage" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelLanguage"></a>

```java
public ModelLanguage getModelLanguage();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a>

---

##### `modelOptions`<sup>Required</sup> <a name="modelOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelOptions"></a>

```java
public ModelOptions getModelOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a>

---

### TypeSafeApiOptions <a name="TypeSafeApiOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions"></a>

Options required alongside an Open API specification to create API Gateway resources.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiOptions;

TypeSafeApiOptions.builder()
    .integrations(java.util.Map<java.lang.String, TypeSafeApiIntegration>)
    .operationLookup(java.util.Map<java.lang.String, MethodAndPath>)
//  .corsOptions(CorsOptions)
//  .defaultAuthorizer(Authorizer)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.integrations">integrations</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>></code> | A mapping of API operation to its integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.operationLookup">operationLookup</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>></code> | Details about each operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.corsOptions">corsOptions</a></code> | <code>software.amazon.awscdk.services.apigateway.CorsOptions</code> | Cross Origin Resource Sharing options for the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.defaultAuthorizer">defaultAuthorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The default authorizer to use for your api. |

---

##### `integrations`<sup>Required</sup> <a name="integrations" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.integrations"></a>

```java
public java.util.Map<java.lang.String, TypeSafeApiIntegration> getIntegrations();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>>

A mapping of API operation to its integration.

---

##### `operationLookup`<sup>Required</sup> <a name="operationLookup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.operationLookup"></a>

```java
public java.util.Map<java.lang.String, MethodAndPath> getOperationLookup();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>>

Details about each operation.

---

##### `corsOptions`<sup>Optional</sup> <a name="corsOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.corsOptions"></a>

```java
public CorsOptions getCorsOptions();
```

- *Type:* software.amazon.awscdk.services.apigateway.CorsOptions

Cross Origin Resource Sharing options for the API.

---

##### `defaultAuthorizer`<sup>Optional</sup> <a name="defaultAuthorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.defaultAuthorizer"></a>

```java
public Authorizer getDefaultAuthorizer();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The default authorizer to use for your api.

When omitted, no default authorizer is used.
Authorizers specified at the integration level will override this for that operation.

---

### TypeSafeApiProjectOptions <a name="TypeSafeApiProjectOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions"></a>

Options for the TypeSafeApiProject.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiProjectOptions;

TypeSafeApiProjectOptions.builder()
    .name(java.lang.String)
//  .commitGenerated(java.lang.Boolean)
//  .gitIgnoreOptions(IgnoreFileOptions)
//  .gitOptions(GitOptions)
//  .logging(LoggerOptions)
//  .outdir(java.lang.String)
//  .parent(Project)
//  .projenCommand(java.lang.String)
//  .projenrcJson(java.lang.Boolean)
//  .projenrcJsonOptions(ProjenrcJsonOptions)
//  .renovatebot(java.lang.Boolean)
//  .renovatebotOptions(RenovatebotOptions)
    .infrastructure(InfrastructureConfiguration)
    .model(ModelConfiguration)
    .runtime(RuntimeConfiguration)
//  .documentation(DocumentationConfiguration)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.name">name</a></code> | <code>java.lang.String</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>io.github.cdklabs.projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitOptions">gitOptions</a></code> | <code>io.github.cdklabs.projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.logging">logging</a></code> | <code>io.github.cdklabs.projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.outdir">outdir</a></code> | <code>java.lang.String</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJson">projenrcJson</a></code> | <code>java.lang.Boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>io.github.cdklabs.projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebot">renovatebot</a></code> | <code>java.lang.Boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebotOptions">renovatebotOptions</a></code> | <code>io.github.cdklabs.projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.infrastructure">infrastructure</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a></code> | Configuration for generated infrastructure. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.model">model</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a></code> | Configuration for the API model. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.runtime">runtime</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a></code> | Configuration for generated runtime projects (containing types, clients and server code). |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.documentation">documentation</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a></code> | Configuration for generated documentation. |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.commitGenerated"></a>

```java
public java.lang.Boolean getCommitGenerated();
```

- *Type:* java.lang.Boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitIgnoreOptions"></a>

```java
public IgnoreFileOptions getGitIgnoreOptions();
```

- *Type:* io.github.cdklabs.projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitOptions"></a>

```java
public GitOptions getGitOptions();
```

- *Type:* io.github.cdklabs.projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.logging"></a>

```java
public LoggerOptions getLogging();
```

- *Type:* io.github.cdklabs.projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.outdir"></a>

```java
public java.lang.String getOutdir();
```

- *Type:* java.lang.String
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.parent"></a>

```java
public Project getParent();
```

- *Type:* io.github.cdklabs.projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenCommand"></a>

```java
public java.lang.String getProjenCommand();
```

- *Type:* java.lang.String
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJson"></a>

```java
public java.lang.Boolean getProjenrcJson();
```

- *Type:* java.lang.Boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJsonOptions"></a>

```java
public ProjenrcJsonOptions getProjenrcJsonOptions();
```

- *Type:* io.github.cdklabs.projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebot"></a>

```java
public java.lang.Boolean getRenovatebot();
```

- *Type:* java.lang.Boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebotOptions"></a>

```java
public RenovatebotOptions getRenovatebotOptions();
```

- *Type:* io.github.cdklabs.projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `infrastructure`<sup>Required</sup> <a name="infrastructure" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.infrastructure"></a>

```java
public InfrastructureConfiguration getInfrastructure();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a>

Configuration for generated infrastructure.

---

##### `model`<sup>Required</sup> <a name="model" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.model"></a>

```java
public ModelConfiguration getModel();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a>

Configuration for the API model.

---

##### `runtime`<sup>Required</sup> <a name="runtime" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.runtime"></a>

```java
public RuntimeConfiguration getRuntime();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a>

Configuration for generated runtime projects (containing types, clients and server code).

---

##### `documentation`<sup>Optional</sup> <a name="documentation" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.documentation"></a>

```java
public DocumentationConfiguration getDocumentation();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a>

Configuration for generated documentation.

---

### TypeSafeApiWebAclOptions <a name="TypeSafeApiWebAclOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions"></a>

Configuration for the Web ACL associated with the API.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiWebAclOptions;

TypeSafeApiWebAclOptions.builder()
//  .cidrAllowList(CidrAllowList)
//  .disable(java.lang.Boolean)
//  .managedRules(java.util.List<ManagedRule>)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.cidrAllowList">cidrAllowList</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList">CidrAllowList</a></code> | List of cidr ranges to allow. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.disable">disable</a></code> | <code>java.lang.Boolean</code> | If set to true, no WebACL will be associated with the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.managedRules">managedRules</a></code> | <code>java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule">ManagedRule</a>></code> | List of managed rules to apply to the web acl. |

---

##### `cidrAllowList`<sup>Optional</sup> <a name="cidrAllowList" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.cidrAllowList"></a>

```java
public CidrAllowList getCidrAllowList();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList">CidrAllowList</a>
- *Default:* undefined

List of cidr ranges to allow.

---

##### `disable`<sup>Optional</sup> <a name="disable" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.disable"></a>

```java
public java.lang.Boolean getDisable();
```

- *Type:* java.lang.Boolean
- *Default:* false

If set to true, no WebACL will be associated with the API.

You can also use this option if you would like to create
your own WebACL and associate it yourself.

---

##### `managedRules`<sup>Optional</sup> <a name="managedRules" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.managedRules"></a>

```java
public java.util.List<ManagedRule> getManagedRules();
```

- *Type:* java.util.List<<a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule">ManagedRule</a>>
- *Default:* [{ vendor: "AWS", name: "AWSManagedRulesCommonRuleSet" }]

List of managed rules to apply to the web acl.

---

### TypeSafeRestApiProps <a name="TypeSafeRestApiProps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps"></a>

Configuration for the TypeSafeRestApi construct.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeRestApiProps;

TypeSafeRestApiProps.builder()
//  .cloudWatchRole(java.lang.Boolean)
//  .deploy(java.lang.Boolean)
//  .deployOptions(StageOptions)
//  .description(java.lang.String)
//  .disableExecuteApiEndpoint(java.lang.Boolean)
//  .domainName(DomainNameOptions)
//  .endpointExportName(java.lang.String)
//  .endpointTypes(java.util.List<EndpointType>)
//  .failOnWarnings(java.lang.Boolean)
//  .parameters(java.util.Map<java.lang.String, java.lang.String>)
//  .policy(PolicyDocument)
//  .restApiName(java.lang.String)
//  .retainDeployments(java.lang.Boolean)
    .integrations(java.util.Map<java.lang.String, TypeSafeApiIntegration>)
    .operationLookup(java.util.Map<java.lang.String, MethodAndPath>)
//  .corsOptions(CorsOptions)
//  .defaultAuthorizer(Authorizer)
    .specPath(java.lang.String)
//  .webAclOptions(TypeSafeApiWebAclOptions)
    .build();
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.cloudWatchRole">cloudWatchRole</a></code> | <code>java.lang.Boolean</code> | Automatically configure an AWS CloudWatch role for API Gateway. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deploy">deploy</a></code> | <code>java.lang.Boolean</code> | Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deployOptions">deployOptions</a></code> | <code>software.amazon.awscdk.services.apigateway.StageOptions</code> | Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.description">description</a></code> | <code>java.lang.String</code> | A description of the RestApi construct. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.disableExecuteApiEndpoint">disableExecuteApiEndpoint</a></code> | <code>java.lang.Boolean</code> | Specifies whether clients can invoke the API using the default execute-api endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.domainName">domainName</a></code> | <code>software.amazon.awscdk.services.apigateway.DomainNameOptions</code> | Configure a custom domain name and map it to this API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointExportName">endpointExportName</a></code> | <code>java.lang.String</code> | Export name for the CfnOutput containing the API endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointTypes">endpointTypes</a></code> | <code>java.util.List<software.amazon.awscdk.services.apigateway.EndpointType></code> | A list of the endpoint types of the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.failOnWarnings">failOnWarnings</a></code> | <code>java.lang.Boolean</code> | Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.parameters">parameters</a></code> | <code>java.util.Map<java.lang.String, java.lang.String></code> | Custom header parameters for the request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.policy">policy</a></code> | <code>software.amazon.awscdk.services.iam.PolicyDocument</code> | A policy document that contains the permissions for this RestApi. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.restApiName">restApiName</a></code> | <code>java.lang.String</code> | A name for the API Gateway RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.retainDeployments">retainDeployments</a></code> | <code>java.lang.Boolean</code> | Retains old deployment resources when the API changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.integrations">integrations</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>></code> | A mapping of API operation to its integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.operationLookup">operationLookup</a></code> | <code>java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>></code> | Details about each operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.corsOptions">corsOptions</a></code> | <code>software.amazon.awscdk.services.apigateway.CorsOptions</code> | Cross Origin Resource Sharing options for the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.defaultAuthorizer">defaultAuthorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The default authorizer to use for your api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.specPath">specPath</a></code> | <code>java.lang.String</code> | Path to the JSON open api spec. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.webAclOptions">webAclOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a></code> | Options for the AWS WAF v2 WebACL associated with the api. |

---

##### `cloudWatchRole`<sup>Optional</sup> <a name="cloudWatchRole" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.cloudWatchRole"></a>

```java
public java.lang.Boolean getCloudWatchRole();
```

- *Type:* java.lang.Boolean
- *Default:* false if `@aws-cdk/aws-apigateway:disableCloudWatchRole` is enabled, true otherwise

Automatically configure an AWS CloudWatch role for API Gateway.

---

##### `deploy`<sup>Optional</sup> <a name="deploy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deploy"></a>

```java
public java.lang.Boolean getDeploy();
```

- *Type:* java.lang.Boolean
- *Default:* true

Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes.

Since API Gateway deployments are immutable, When this option is enabled
(by default), an AWS::ApiGateway::Deployment resource will automatically
created with a logical ID that hashes the API model (methods, resources
and options). This means that when the model changes, the logical ID of
this CloudFormation resource will change, and a new deployment will be
created.

If this is set, `latestDeployment` will refer to the `Deployment` object
and `deploymentStage` will refer to a `Stage` that points to this
deployment. To customize the stage options, use the `deployOptions`
property.

A CloudFormation Output will also be defined with the root URL endpoint
of this REST API.

---

##### `deployOptions`<sup>Optional</sup> <a name="deployOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deployOptions"></a>

```java
public StageOptions getDeployOptions();
```

- *Type:* software.amazon.awscdk.services.apigateway.StageOptions
- *Default:* Based on defaults of `StageOptions`.

Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled.

If `deploy` is disabled,
this value cannot be set.

---

##### `description`<sup>Optional</sup> <a name="description" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.description"></a>

```java
public java.lang.String getDescription();
```

- *Type:* java.lang.String
- *Default:* 'Automatically created by the RestApi construct'

A description of the RestApi construct.

---

##### `disableExecuteApiEndpoint`<sup>Optional</sup> <a name="disableExecuteApiEndpoint" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.disableExecuteApiEndpoint"></a>

```java
public java.lang.Boolean getDisableExecuteApiEndpoint();
```

- *Type:* java.lang.Boolean
- *Default:* false

Specifies whether clients can invoke the API using the default execute-api endpoint.

To require that clients use a custom domain name to invoke the
API, disable the default endpoint.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html)

---

##### `domainName`<sup>Optional</sup> <a name="domainName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.domainName"></a>

```java
public DomainNameOptions getDomainName();
```

- *Type:* software.amazon.awscdk.services.apigateway.DomainNameOptions
- *Default:* no domain name is defined, use `addDomainName` or directly define a `DomainName`.

Configure a custom domain name and map it to this API.

---

##### `endpointExportName`<sup>Optional</sup> <a name="endpointExportName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointExportName"></a>

```java
public java.lang.String getEndpointExportName();
```

- *Type:* java.lang.String
- *Default:* when no export name is given, output will be created without export

Export name for the CfnOutput containing the API endpoint.

---

##### `endpointTypes`<sup>Optional</sup> <a name="endpointTypes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointTypes"></a>

```java
public java.util.List<EndpointType> getEndpointTypes();
```

- *Type:* java.util.List<software.amazon.awscdk.services.apigateway.EndpointType>
- *Default:* EndpointType.EDGE

A list of the endpoint types of the API.

Use this property when creating
an API.

---

##### `failOnWarnings`<sup>Optional</sup> <a name="failOnWarnings" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.failOnWarnings"></a>

```java
public java.lang.Boolean getFailOnWarnings();
```

- *Type:* java.lang.Boolean
- *Default:* false

Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource.

---

##### `parameters`<sup>Optional</sup> <a name="parameters" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.parameters"></a>

```java
public java.util.Map<java.lang.String, java.lang.String> getParameters();
```

- *Type:* java.util.Map<java.lang.String, java.lang.String>
- *Default:* No parameters.

Custom header parameters for the request.

> [https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html](https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html)

---

##### `policy`<sup>Optional</sup> <a name="policy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.policy"></a>

```java
public PolicyDocument getPolicy();
```

- *Type:* software.amazon.awscdk.services.iam.PolicyDocument
- *Default:* No policy.

A policy document that contains the permissions for this RestApi.

---

##### `restApiName`<sup>Optional</sup> <a name="restApiName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.restApiName"></a>

```java
public java.lang.String getRestApiName();
```

- *Type:* java.lang.String
- *Default:* ID of the RestApi construct.

A name for the API Gateway RestApi resource.

---

##### `retainDeployments`<sup>Optional</sup> <a name="retainDeployments" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.retainDeployments"></a>

```java
public java.lang.Boolean getRetainDeployments();
```

- *Type:* java.lang.Boolean
- *Default:* false

Retains old deployment resources when the API changes.

This allows
manually reverting stages to point to old deployments via the AWS
Console.

---

##### `integrations`<sup>Required</sup> <a name="integrations" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.integrations"></a>

```java
public java.util.Map<java.lang.String, TypeSafeApiIntegration> getIntegrations();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>>

A mapping of API operation to its integration.

---

##### `operationLookup`<sup>Required</sup> <a name="operationLookup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.operationLookup"></a>

```java
public java.util.Map<java.lang.String, MethodAndPath> getOperationLookup();
```

- *Type:* java.util.Map<java.lang.String, <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>>

Details about each operation.

---

##### `corsOptions`<sup>Optional</sup> <a name="corsOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.corsOptions"></a>

```java
public CorsOptions getCorsOptions();
```

- *Type:* software.amazon.awscdk.services.apigateway.CorsOptions

Cross Origin Resource Sharing options for the API.

---

##### `defaultAuthorizer`<sup>Optional</sup> <a name="defaultAuthorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.defaultAuthorizer"></a>

```java
public Authorizer getDefaultAuthorizer();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The default authorizer to use for your api.

When omitted, no default authorizer is used.
Authorizers specified at the integration level will override this for that operation.

---

##### `specPath`<sup>Required</sup> <a name="specPath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.specPath"></a>

```java
public java.lang.String getSpecPath();
```

- *Type:* java.lang.String

Path to the JSON open api spec.

---

##### `webAclOptions`<sup>Optional</sup> <a name="webAclOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.webAclOptions"></a>

```java
public TypeSafeApiWebAclOptions getWebAclOptions();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a>

Options for the AWS WAF v2 WebACL associated with the api.

By default, a Web ACL with the AWS default managed
rule set will be associated with the API. These options may disable or override the defaults.

---

## Classes <a name="Classes" id="Classes"></a>

### Authorizer <a name="Authorizer" id="@aws-prototyping-sdk/type-safe-api.Authorizer"></a>

An authorizer for authorizing API requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizer;

Authorizer.Builder.create()
    .authorizationType(AuthorizationType)
    .authorizerId(java.lang.String)
//  .authorizationScopes(java.util.List<java.lang.String>)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizationType"></a>

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizerId"></a>

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.authorizationScopes"></a>

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---


### Authorizers <a name="Authorizers" id="@aws-prototyping-sdk/type-safe-api.Authorizers"></a>

Class used to construct authorizers for use in the OpenApiGatewayLambdaApi construct.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Authorizers.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;

new Authorizers();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---


#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.cognito">cognito</a></code> | A Cognito User Pools authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.custom">custom</a></code> | A custom authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.iam">iam</a></code> | An IAM authorizer which uses AWS signature version 4 to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.none">none</a></code> | No authorizer. |

---

##### `cognito` <a name="cognito" id="@aws-prototyping-sdk/type-safe-api.Authorizers.cognito"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;

Authorizers.cognito(CognitoAuthorizerProps props)
```

A Cognito User Pools authorizer.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Authorizers.cognito.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps">CognitoAuthorizerProps</a>

---

##### `custom` <a name="custom" id="@aws-prototyping-sdk/type-safe-api.Authorizers.custom"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;

Authorizers.custom(CustomAuthorizerProps props)
```

A custom authorizer.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Authorizers.custom.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps">CustomAuthorizerProps</a>

---

##### `iam` <a name="iam" id="@aws-prototyping-sdk/type-safe-api.Authorizers.iam"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;

Authorizers.iam()
```

An IAM authorizer which uses AWS signature version 4 to authorize requests.

##### `none` <a name="none" id="@aws-prototyping-sdk/type-safe-api.Authorizers.none"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;

Authorizers.none()
```

No authorizer.



### CognitoAuthorizer <a name="CognitoAuthorizer" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer"></a>

An authorizer that uses Cognito identity or access tokens.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.CognitoAuthorizer;

CognitoAuthorizer.Builder.create()
    .authorizerId(java.lang.String)
    .userPools(java.util.List<IUserPool>)
//  .authorizationScopes(java.util.List<java.lang.String>)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.userPools">userPools</a></code> | <code>java.util.List<software.amazon.awscdk.services.cognito.IUserPool></code> | The Cognito user pools associated with this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | A list of authorization scopes configured on the method. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.authorizerId"></a>

- *Type:* java.lang.String

Unique identifier for this authorizer.

---

##### `userPools`<sup>Required</sup> <a name="userPools" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.userPools"></a>

- *Type:* java.util.List<software.amazon.awscdk.services.cognito.IUserPool>

The Cognito user pools associated with this authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.authorizationScopes"></a>

- *Type:* java.util.List<java.lang.String>
- *Default:* []

A list of authorization scopes configured on the method.

When used as the default authorizer, these scopes will be
applied to all methods without an authorizer at the integration level.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes)

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes">withScopes</a></code> | Returns this authorizer with scopes applied, intended for usage in individual operations where scopes may differ on a per-operation basis. |

---

##### `withScopes` <a name="withScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes"></a>

```java
public CognitoAuthorizer withScopes(java.lang.String authorizationScopes)
```

Returns this authorizer with scopes applied, intended for usage in individual operations where scopes may differ on a per-operation basis.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes)

###### `authorizationScopes`<sup>Required</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes.parameter.authorizationScopes"></a>

- *Type:* java.lang.String

the scopes to apply.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.userPools">userPools</a></code> | <code>java.util.List<software.amazon.awscdk.services.cognito.IUserPool></code> | The Cognito user pools associated with this authorizer. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---

##### `userPools`<sup>Required</sup> <a name="userPools" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.userPools"></a>

```java
public java.util.List<IUserPool> getUserPools();
```

- *Type:* java.util.List<software.amazon.awscdk.services.cognito.IUserPool>

The Cognito user pools associated with this authorizer.

---


### CustomAuthorizer <a name="CustomAuthorizer" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer"></a>

An authorizer that uses a lambda function to authorize requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.CustomAuthorizer;

CustomAuthorizer.Builder.create()
    .authorizerId(java.lang.String)
    .function(IFunction)
//  .authorizerResultTtlInSeconds(java.lang.Number)
//  .identitySource(java.lang.String)
//  .type(CustomAuthorizerType)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.function">function</a></code> | <code>software.amazon.awscdk.services.lambda.IFunction</code> | The lambda function used to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.authorizerResultTtlInSeconds">authorizerResultTtlInSeconds</a></code> | <code>java.lang.Number</code> | The number of seconds during which the authorizer result is cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.identitySource">identitySource</a></code> | <code>java.lang.String</code> | The source of the identity in an incoming request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.type">type</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a></code> | The type of custom authorizer. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.authorizerId"></a>

- *Type:* java.lang.String

Unique identifier for this authorizer.

---

##### `function`<sup>Required</sup> <a name="function" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.function"></a>

- *Type:* software.amazon.awscdk.services.lambda.IFunction

The lambda function used to authorize requests.

---

##### `authorizerResultTtlInSeconds`<sup>Optional</sup> <a name="authorizerResultTtlInSeconds" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.authorizerResultTtlInSeconds"></a>

- *Type:* java.lang.Number
- *Default:* 300

The number of seconds during which the authorizer result is cached.

---

##### `identitySource`<sup>Optional</sup> <a name="identitySource" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.identitySource"></a>

- *Type:* java.lang.String
- *Default:* "method.request.header.Authorization"

The source of the identity in an incoming request.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource)

---

##### `type`<sup>Optional</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.type"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a>
- *Default:* CustomAuthorizerType.TOKEN

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerResultTtlInSeconds">authorizerResultTtlInSeconds</a></code> | <code>java.lang.Number</code> | The number of seconds during which the authorizer result is cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.function">function</a></code> | <code>software.amazon.awscdk.services.lambda.IFunction</code> | The lambda function used to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.identitySource">identitySource</a></code> | <code>java.lang.String</code> | The source of the identity in an incoming request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.type">type</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a></code> | The type of custom authorizer. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---

##### `authorizerResultTtlInSeconds`<sup>Required</sup> <a name="authorizerResultTtlInSeconds" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerResultTtlInSeconds"></a>

```java
public java.lang.Number getAuthorizerResultTtlInSeconds();
```

- *Type:* java.lang.Number

The number of seconds during which the authorizer result is cached.

---

##### `function`<sup>Required</sup> <a name="function" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.function"></a>

```java
public IFunction getFunction();
```

- *Type:* software.amazon.awscdk.services.lambda.IFunction

The lambda function used to authorize requests.

---

##### `identitySource`<sup>Required</sup> <a name="identitySource" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.identitySource"></a>

```java
public java.lang.String getIdentitySource();
```

- *Type:* java.lang.String

The source of the identity in an incoming request.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource)

---

##### `type`<sup>Required</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.type"></a>

```java
public CustomAuthorizerType getType();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a>

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

---


### IamAuthorizer <a name="IamAuthorizer" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer"></a>

An IAM authorizer.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.IamAuthorizer;

new IamAuthorizer();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---


### Integration <a name="Integration" id="@aws-prototyping-sdk/type-safe-api.Integration"></a>

An integration for an API operation.

You can extend this to implement your own integration if you like.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Integration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Integration;

new Integration();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration.grant">grant</a></code> | Grant permissions for the API to invoke the integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration.render">render</a></code> | Render the integration into an API Gateway OpenAPI extension. |

---

##### `grant` <a name="grant" id="@aws-prototyping-sdk/type-safe-api.Integration.grant"></a>

```java
public void grant(IntegrationGrantProps _props)
```

Grant permissions for the API to invoke the integration.

###### `_props`<sup>Required</sup> <a name="_props" id="@aws-prototyping-sdk/type-safe-api.Integration.grant.parameter._props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps">IntegrationGrantProps</a>

---

##### `render` <a name="render" id="@aws-prototyping-sdk/type-safe-api.Integration.render"></a>

```java
public ApiGatewayIntegration render(IntegrationRenderProps props)
```

Render the integration into an API Gateway OpenAPI extension.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Integration.render.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps">IntegrationRenderProps</a>

---




### Integrations <a name="Integrations" id="@aws-prototyping-sdk/type-safe-api.Integrations"></a>

A collection of integrations to connect API operations with a backend to service requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Integrations.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Integrations;

new Integrations();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---


#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integrations.lambda">lambda</a></code> | An integration that invokes a lambda function to service the request. |

---

##### `lambda` <a name="lambda" id="@aws-prototyping-sdk/type-safe-api.Integrations.lambda"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.Integrations;

Integrations.lambda(IFunction lambdaFunction)
```

An integration that invokes a lambda function to service the request.

###### `lambdaFunction`<sup>Required</sup> <a name="lambdaFunction" id="@aws-prototyping-sdk/type-safe-api.Integrations.lambda.parameter.lambdaFunction"></a>

- *Type:* software.amazon.awscdk.services.lambda.IFunction

the function to invoke.

---



### LambdaIntegration <a name="LambdaIntegration" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration"></a>

A lambda integration.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.LambdaIntegration;

new LambdaIntegration(IFunction lambdaFunction);
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer.parameter.lambdaFunction">lambdaFunction</a></code> | <code>software.amazon.awscdk.services.lambda.IFunction</code> | *No description.* |

---

##### `lambdaFunction`<sup>Required</sup> <a name="lambdaFunction" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer.parameter.lambdaFunction"></a>

- *Type:* software.amazon.awscdk.services.lambda.IFunction

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant">grant</a></code> | Grant API Gateway permissions to invoke the lambda. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render">render</a></code> | Render the lambda integration as a snippet of OpenAPI. |

---

##### `grant` <a name="grant" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant"></a>

```java
public void grant(IntegrationGrantProps __0)
```

Grant API Gateway permissions to invoke the lambda.

###### `__0`<sup>Required</sup> <a name="__0" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant.parameter.__0"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps">IntegrationGrantProps</a>

---

##### `render` <a name="render" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render"></a>

```java
public ApiGatewayIntegration render(IntegrationRenderProps props)
```

Render the lambda integration as a snippet of OpenAPI.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps">IntegrationRenderProps</a>

---




### NoneAuthorizer <a name="NoneAuthorizer" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer"></a>

No authorizer.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.NoneAuthorizer;

new NoneAuthorizer();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationType">authorizationType</a></code> | <code>software.amazon.awscdk.services.apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizerId">authorizerId</a></code> | <code>java.lang.String</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>java.util.List<java.lang.String></code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationType"></a>

```java
public AuthorizationType getAuthorizationType();
```

- *Type:* software.amazon.awscdk.services.apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizerId"></a>

```java
public java.lang.String getAuthorizerId();
```

- *Type:* java.lang.String

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationScopes"></a>

```java
public java.util.List<java.lang.String> getAuthorizationScopes();
```

- *Type:* java.util.List<java.lang.String>

Scopes for the authorizer, if any.

---


### OpenApiDefinition <a name="OpenApiDefinition" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition"></a>

The OpenAPI Spec.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.OpenApiDefinition;

OpenApiDefinition.Builder.create(TypeSafeApiModelProject project)
    .openApiOptions(OpenApiModelOptions)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.project">project</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.openApiOptions">openApiOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a></code> | Options for the openapi model. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.project"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

---

##### `openApiOptions`<sup>Required</sup> <a name="openApiOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.openApiOptions"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a>

Options for the openapi model.

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.postSynthesize">postSynthesize</a></code> | Called after synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.preSynthesize">preSynthesize</a></code> | Called before synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.synthesize">synthesize</a></code> | Synthesizes files to the project output directory. |

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.postSynthesize"></a>

```java
public void postSynthesize()
```

Called after synthesis.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.preSynthesize"></a>

```java
public void preSynthesize()
```

Called before synthesis.

##### `synthesize` <a name="synthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.synthesize"></a>

```java
public void synthesize()
```

Synthesizes files to the project output directory.


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.project">project</a></code> | <code>io.github.cdklabs.projen.Project</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.openApiSpecificationPath">openApiSpecificationPath</a></code> | <code>java.lang.String</code> | Path to the root OpenAPI specification file. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.project"></a>

```java
public Project getProject();
```

- *Type:* io.github.cdklabs.projen.Project

---

##### `openApiSpecificationPath`<sup>Required</sup> <a name="openApiSpecificationPath" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.openApiSpecificationPath"></a>

```java
public java.lang.String getOpenApiSpecificationPath();
```

- *Type:* java.lang.String

Path to the root OpenAPI specification file.

---


### SmithyDefinition <a name="SmithyDefinition" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition"></a>

Creates a project which transforms a Smithy model to OpenAPI.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.SmithyDefinition;

SmithyDefinition.Builder.create(TypeSafeApiModelProject project)
    .smithyOptions(SmithyModelOptions)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.project">project</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.smithyOptions">smithyOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a></code> | Smithy engine options. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.project"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

---

##### `smithyOptions`<sup>Required</sup> <a name="smithyOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.smithyOptions"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a>

Smithy engine options.

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.postSynthesize">postSynthesize</a></code> | Called after synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.preSynthesize">preSynthesize</a></code> | Called before synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.synthesize">synthesize</a></code> | Synthesizes files to the project output directory. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps">addDeps</a></code> | Add maven-style or local file dependencies to the smithy model project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps">addSmithyDeps</a></code> | Add dependencies on other smithy models, such that their shapes can be imported in this project. |

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.postSynthesize"></a>

```java
public void postSynthesize()
```

Called after synthesis.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.preSynthesize"></a>

```java
public void preSynthesize()
```

Called before synthesis.

##### `synthesize` <a name="synthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.synthesize"></a>

```java
public void synthesize()
```

Synthesizes files to the project output directory.

##### `addDeps` <a name="addDeps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps"></a>

```java
public void addDeps(java.lang.String deps)
```

Add maven-style or local file dependencies to the smithy model project.

###### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps.parameter.deps"></a>

- *Type:* java.lang.String

dependencies to add, eg "software.amazon.smithy:smithy-validation-model:1.27.2" or "file://../some/path/build/lib/my-shapes.jar.

---

##### `addSmithyDeps` <a name="addSmithyDeps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps"></a>

```java
public void addSmithyDeps(SmithyDefinition deps)
```

Add dependencies on other smithy models, such that their shapes can be imported in this project.

###### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps.parameter.deps"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a>

smithy definitions to depend on.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.project">project</a></code> | <code>io.github.cdklabs.projen.Project</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.gradleProjectName">gradleProjectName</a></code> | <code>java.lang.String</code> | Name of the gradle project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.openApiSpecificationPath">openApiSpecificationPath</a></code> | <code>java.lang.String</code> | Path to the generated OpenAPI specification, relative to the project outdir. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.project"></a>

```java
public Project getProject();
```

- *Type:* io.github.cdklabs.projen.Project

---

##### `gradleProjectName`<sup>Required</sup> <a name="gradleProjectName" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.gradleProjectName"></a>

```java
public java.lang.String getGradleProjectName();
```

- *Type:* java.lang.String

Name of the gradle project.

---

##### `openApiSpecificationPath`<sup>Required</sup> <a name="openApiSpecificationPath" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.openApiSpecificationPath"></a>

```java
public java.lang.String getOpenApiSpecificationPath();
```

- *Type:* java.lang.String

Path to the generated OpenAPI specification, relative to the project outdir.

---


### TypeSafeApiModelProject <a name="TypeSafeApiModelProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject"></a>

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiModelProject;

TypeSafeApiModelProject.Builder.create()
    .name(java.lang.String)
//  .commitGenerated(java.lang.Boolean)
//  .gitIgnoreOptions(IgnoreFileOptions)
//  .gitOptions(GitOptions)
//  .logging(LoggerOptions)
//  .outdir(java.lang.String)
//  .parent(Project)
//  .projenCommand(java.lang.String)
//  .projenrcJson(java.lang.Boolean)
//  .projenrcJsonOptions(ProjenrcJsonOptions)
//  .renovatebot(java.lang.Boolean)
//  .renovatebotOptions(RenovatebotOptions)
    .modelLanguage(ModelLanguage)
    .modelOptions(ModelOptions)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.name">name</a></code> | <code>java.lang.String</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>io.github.cdklabs.projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.gitOptions">gitOptions</a></code> | <code>io.github.cdklabs.projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.logging">logging</a></code> | <code>io.github.cdklabs.projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.outdir">outdir</a></code> | <code>java.lang.String</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenrcJson">projenrcJson</a></code> | <code>java.lang.Boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>io.github.cdklabs.projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.renovatebot">renovatebot</a></code> | <code>java.lang.Boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.renovatebotOptions">renovatebotOptions</a></code> | <code>io.github.cdklabs.projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.modelLanguage">modelLanguage</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.modelOptions">modelOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a></code> | *No description.* |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.name"></a>

- *Type:* java.lang.String
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.commitGenerated"></a>

- *Type:* java.lang.Boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.gitIgnoreOptions"></a>

- *Type:* io.github.cdklabs.projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.gitOptions"></a>

- *Type:* io.github.cdklabs.projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.logging"></a>

- *Type:* io.github.cdklabs.projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.outdir"></a>

- *Type:* java.lang.String
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.parent"></a>

- *Type:* io.github.cdklabs.projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenCommand"></a>

- *Type:* java.lang.String
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenrcJson"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.projenrcJsonOptions"></a>

- *Type:* io.github.cdklabs.projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.renovatebot"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.renovatebotOptions"></a>

- *Type:* io.github.cdklabs.projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `modelLanguage`<sup>Required</sup> <a name="modelLanguage" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.modelLanguage"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a>

---

##### `modelOptions`<sup>Required</sup> <a name="modelOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.modelOptions"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup">addExcludeFromCleanup</a></code> | Exclude the matching files from pre-synth cleanup. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore">addGitIgnore</a></code> | Adds a .gitignore pattern. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore">addPackageIgnore</a></code> | Exclude these files from the bundled package. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask">addTask</a></code> | Adds a new task to this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip">addTip</a></code> | Prints a "tip" message during synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated">annotateGenerated</a></code> | Consider a set of files as "generated". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.postSynthesize">postSynthesize</a></code> | Called after all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.preSynthesize">preSynthesize</a></code> | Called before all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask">removeTask</a></code> | Removes a task from a project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand">runTaskCommand</a></code> | Returns the shell command to execute in order to run a task. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.synth">synth</a></code> | Synthesize all project files into `outdir`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile">tryFindFile</a></code> | Finds a file at the specified relative path within this project and all its subprojects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile">tryFindJsonFile</a></code> | Finds a json file by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile">tryFindObjectFile</a></code> | Finds an object file (like JsonFile, YamlFile, etc.) by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile">tryRemoveFile</a></code> | Finds a file at the specified relative path within this project and removes it. |

---

##### `addExcludeFromCleanup` <a name="addExcludeFromCleanup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup"></a>

```java
public void addExcludeFromCleanup(java.lang.String globs)
```

Exclude the matching files from pre-synth cleanup.

Can be used when, for example, some
source files include the projen marker and we don't want them to be erased during synth.

###### `globs`<sup>Required</sup> <a name="globs" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup.parameter.globs"></a>

- *Type:* java.lang.String

The glob patterns to match.

---

##### `addGitIgnore` <a name="addGitIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore"></a>

```java
public void addGitIgnore(java.lang.String pattern)
```

Adds a .gitignore pattern.

###### `pattern`<sup>Required</sup> <a name="pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore.parameter.pattern"></a>

- *Type:* java.lang.String

The glob pattern to ignore.

---

##### `addPackageIgnore` <a name="addPackageIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore"></a>

```java
public void addPackageIgnore(java.lang.String _pattern)
```

Exclude these files from the bundled package.

Implemented by project types based on the
packaging mechanism. For example, `NodeProject` delegates this to `.npmignore`.

###### `_pattern`<sup>Required</sup> <a name="_pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore.parameter._pattern"></a>

- *Type:* java.lang.String

The glob pattern to exclude.

---

##### `addTask` <a name="addTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask"></a>

```java
public Task addTask(java.lang.String name)
public Task addTask(java.lang.String name, TaskOptions props)
```

Adds a new task to this project.

This will fail if the project already has
a task with this name.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask.parameter.name"></a>

- *Type:* java.lang.String

The task name to add.

---

###### `props`<sup>Optional</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask.parameter.props"></a>

- *Type:* io.github.cdklabs.projen.TaskOptions

Task properties.

---

##### ~~`addTip`~~ <a name="addTip" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip"></a>

```java
public void addTip(java.lang.String message)
```

Prints a "tip" message during synthesis.

###### `message`<sup>Required</sup> <a name="message" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip.parameter.message"></a>

- *Type:* java.lang.String

The message.

---

##### `annotateGenerated` <a name="annotateGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated"></a>

```java
public void annotateGenerated(java.lang.String _glob)
```

Consider a set of files as "generated".

This method is implemented by
derived classes and used for example, to add git attributes to tell GitHub
that certain files are generated.

###### `_glob`<sup>Required</sup> <a name="_glob" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated.parameter._glob"></a>

- *Type:* java.lang.String

the glob pattern to match (could be a file path).

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.postSynthesize"></a>

```java
public void postSynthesize()
```

Called after all components are synthesized.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.preSynthesize"></a>

```java
public void preSynthesize()
```

Called before all components are synthesized.

##### `removeTask` <a name="removeTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask"></a>

```java
public Task removeTask(java.lang.String name)
```

Removes a task from a project.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask.parameter.name"></a>

- *Type:* java.lang.String

The name of the task to remove.

---

##### `runTaskCommand` <a name="runTaskCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand"></a>

```java
public java.lang.String runTaskCommand(Task task)
```

Returns the shell command to execute in order to run a task.

By default, this is `npx projen@<version> <task>`

###### `task`<sup>Required</sup> <a name="task" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand.parameter.task"></a>

- *Type:* io.github.cdklabs.projen.Task

The task for which the command is required.

---

##### `synth` <a name="synth" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.synth"></a>

```java
public void synth()
```

Synthesize all project files into `outdir`.

1. Call "this.preSynthesize()"
2. Delete all generated files
3. Synthesize all sub-projects
4. Synthesize all components of this project
5. Call "postSynthesize()" for all components of this project
6. Call "this.postSynthesize()"

##### `tryFindFile` <a name="tryFindFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile"></a>

```java
public FileBase tryFindFile(java.lang.String filePath)
```

Finds a file at the specified relative path within this project and all its subprojects.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

If this path is relative, it will be resolved
from the root of _this_ project.

---

##### ~~`tryFindJsonFile`~~ <a name="tryFindJsonFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile"></a>

```java
public JsonFile tryFindJsonFile(java.lang.String filePath)
```

Finds a json file by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

---

##### `tryFindObjectFile` <a name="tryFindObjectFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile"></a>

```java
public ObjectFile tryFindObjectFile(java.lang.String filePath)
```

Finds an object file (like JsonFile, YamlFile, etc.) by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

---

##### `tryRemoveFile` <a name="tryRemoveFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile"></a>

```java
public FileBase tryRemoveFile(java.lang.String filePath)
```

Finds a file at the specified relative path within this project and removes it.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

If this path is relative, it will be
resolved from the root of _this_ project.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.buildTask">buildTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.compileTask">compileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.components">components</a></code> | <code>java.util.List<io.github.cdklabs.projen.Component></code> | Returns all the components within this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.deps">deps</a></code> | <code>io.github.cdklabs.projen.Dependencies</code> | Project dependencies. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.ejected">ejected</a></code> | <code>java.lang.Boolean</code> | Whether or not the project is being ejected. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.files">files</a></code> | <code>java.util.List<io.github.cdklabs.projen.FileBase></code> | All files in this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitattributes">gitattributes</a></code> | <code>io.github.cdklabs.projen.GitAttributesFile</code> | The .gitattributes file for this repository. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitignore">gitignore</a></code> | <code>io.github.cdklabs.projen.IgnoreFile</code> | .gitignore. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.logger">logger</a></code> | <code>io.github.cdklabs.projen.Logger</code> | Logging utilities. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.name">name</a></code> | <code>java.lang.String</code> | Project name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.outdir">outdir</a></code> | <code>java.lang.String</code> | Absolute output directory of this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.packageTask">packageTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.postCompileTask">postCompileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.preCompileTask">preCompileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projectBuild">projectBuild</a></code> | <code>io.github.cdklabs.projen.ProjectBuild</code> | Manages the build process of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.root">root</a></code> | <code>io.github.cdklabs.projen.Project</code> | The root project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.tasks">tasks</a></code> | <code>io.github.cdklabs.projen.Tasks</code> | Project tasks. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.testTask">testTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.defaultTask">defaultTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | This is the "default" task, the one that executes "projen". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.initProject">initProject</a></code> | <code>io.github.cdklabs.projen.InitProject</code> | The options used when this project is bootstrapped via `projen new`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | A parent project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.generateTask">generateTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | Reference to the task used for generating the final bundled OpenAPI specification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parsedSpecFile">parsedSpecFile</a></code> | <code>java.lang.String</code> | Name of the final bundled OpenAPI specification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.openapi">openapi</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition">OpenApiDefinition</a></code> | Reference to the OpenAPI definition component. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.smithy">smithy</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a></code> | Reference to the Smithy definition component. |

---

##### `buildTask`<sup>Required</sup> <a name="buildTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.buildTask"></a>

```java
public Task getBuildTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `commitGenerated`<sup>Required</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.commitGenerated"></a>

```java
public java.lang.Boolean getCommitGenerated();
```

- *Type:* java.lang.Boolean

Whether to commit the managed files by default.

---

##### `compileTask`<sup>Required</sup> <a name="compileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.compileTask"></a>

```java
public Task getCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `components`<sup>Required</sup> <a name="components" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.components"></a>

```java
public java.util.List<Component> getComponents();
```

- *Type:* java.util.List<io.github.cdklabs.projen.Component>

Returns all the components within this project.

---

##### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.deps"></a>

```java
public Dependencies getDeps();
```

- *Type:* io.github.cdklabs.projen.Dependencies

Project dependencies.

---

##### `ejected`<sup>Required</sup> <a name="ejected" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.ejected"></a>

```java
public java.lang.Boolean getEjected();
```

- *Type:* java.lang.Boolean

Whether or not the project is being ejected.

---

##### `files`<sup>Required</sup> <a name="files" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.files"></a>

```java
public java.util.List<FileBase> getFiles();
```

- *Type:* java.util.List<io.github.cdklabs.projen.FileBase>

All files in this project.

---

##### `gitattributes`<sup>Required</sup> <a name="gitattributes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitattributes"></a>

```java
public GitAttributesFile getGitattributes();
```

- *Type:* io.github.cdklabs.projen.GitAttributesFile

The .gitattributes file for this repository.

---

##### `gitignore`<sup>Required</sup> <a name="gitignore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitignore"></a>

```java
public IgnoreFile getGitignore();
```

- *Type:* io.github.cdklabs.projen.IgnoreFile

.gitignore.

---

##### `logger`<sup>Required</sup> <a name="logger" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.logger"></a>

```java
public Logger getLogger();
```

- *Type:* io.github.cdklabs.projen.Logger

Logging utilities.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String

Project name.

---

##### `outdir`<sup>Required</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.outdir"></a>

```java
public java.lang.String getOutdir();
```

- *Type:* java.lang.String

Absolute output directory of this project.

---

##### `packageTask`<sup>Required</sup> <a name="packageTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.packageTask"></a>

```java
public Task getPackageTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `postCompileTask`<sup>Required</sup> <a name="postCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.postCompileTask"></a>

```java
public Task getPostCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `preCompileTask`<sup>Required</sup> <a name="preCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.preCompileTask"></a>

```java
public Task getPreCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `projectBuild`<sup>Required</sup> <a name="projectBuild" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projectBuild"></a>

```java
public ProjectBuild getProjectBuild();
```

- *Type:* io.github.cdklabs.projen.ProjectBuild

Manages the build process of the project.

---

##### `projenCommand`<sup>Required</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projenCommand"></a>

```java
public java.lang.String getProjenCommand();
```

- *Type:* java.lang.String

The command to use in order to run the projen CLI.

---

##### `root`<sup>Required</sup> <a name="root" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.root"></a>

```java
public Project getRoot();
```

- *Type:* io.github.cdklabs.projen.Project

The root project.

---

##### `tasks`<sup>Required</sup> <a name="tasks" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.tasks"></a>

```java
public Tasks getTasks();
```

- *Type:* io.github.cdklabs.projen.Tasks

Project tasks.

---

##### `testTask`<sup>Required</sup> <a name="testTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.testTask"></a>

```java
public Task getTestTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `defaultTask`<sup>Optional</sup> <a name="defaultTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.defaultTask"></a>

```java
public Task getDefaultTask();
```

- *Type:* io.github.cdklabs.projen.Task

This is the "default" task, the one that executes "projen".

Undefined if
the project is being ejected.

---

##### `initProject`<sup>Optional</sup> <a name="initProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.initProject"></a>

```java
public InitProject getInitProject();
```

- *Type:* io.github.cdklabs.projen.InitProject

The options used when this project is bootstrapped via `projen new`.

It
includes the original set of options passed to the CLI and also the JSII
FQN of the project type.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parent"></a>

```java
public Project getParent();
```

- *Type:* io.github.cdklabs.projen.Project

A parent project.

If undefined, this is the root project.

---

##### `generateTask`<sup>Required</sup> <a name="generateTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.generateTask"></a>

```java
public Task getGenerateTask();
```

- *Type:* io.github.cdklabs.projen.Task

Reference to the task used for generating the final bundled OpenAPI specification.

---

##### `parsedSpecFile`<sup>Required</sup> <a name="parsedSpecFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parsedSpecFile"></a>

```java
public java.lang.String getParsedSpecFile();
```

- *Type:* java.lang.String

Name of the final bundled OpenAPI specification.

---

##### `openapi`<sup>Optional</sup> <a name="openapi" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.openapi"></a>

```java
public OpenApiDefinition getOpenapi();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition">OpenApiDefinition</a>

Reference to the OpenAPI definition component.

Will be defined if the model language is OpenAPI

---

##### `smithy`<sup>Optional</sup> <a name="smithy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.smithy"></a>

```java
public SmithyDefinition getSmithy();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a>

Reference to the Smithy definition component.

Will be defined if the model language is Smithy

---

#### Constants <a name="Constants" id="Constants"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.DEFAULT_TASK">DEFAULT_TASK</a></code> | <code>java.lang.String</code> | The name of the default task (the task executed when `projen` is run without arguments). |

---

##### `DEFAULT_TASK`<sup>Required</sup> <a name="DEFAULT_TASK" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.DEFAULT_TASK"></a>

```java
public java.lang.String getDefaultTask();
```

- *Type:* java.lang.String

The name of the default task (the task executed when `projen` is run without arguments).

Normally
this task should synthesize the project files.

---

### TypeSafeApiProject <a name="TypeSafeApiProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject"></a>

Project for a type-safe API, defined using Smithy or OpenAPI.

Generates a CDK construct to deploy your API, as well as client and server code to help build your API quickly.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer"></a>

```java
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiProject;

TypeSafeApiProject.Builder.create()
    .name(java.lang.String)
//  .commitGenerated(java.lang.Boolean)
//  .gitIgnoreOptions(IgnoreFileOptions)
//  .gitOptions(GitOptions)
//  .logging(LoggerOptions)
//  .outdir(java.lang.String)
//  .parent(Project)
//  .projenCommand(java.lang.String)
//  .projenrcJson(java.lang.Boolean)
//  .projenrcJsonOptions(ProjenrcJsonOptions)
//  .renovatebot(java.lang.Boolean)
//  .renovatebotOptions(RenovatebotOptions)
    .infrastructure(InfrastructureConfiguration)
    .model(ModelConfiguration)
    .runtime(RuntimeConfiguration)
//  .documentation(DocumentationConfiguration)
    .build();
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.name">name</a></code> | <code>java.lang.String</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>io.github.cdklabs.projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.gitOptions">gitOptions</a></code> | <code>io.github.cdklabs.projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.logging">logging</a></code> | <code>io.github.cdklabs.projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.outdir">outdir</a></code> | <code>java.lang.String</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenrcJson">projenrcJson</a></code> | <code>java.lang.Boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>io.github.cdklabs.projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.renovatebot">renovatebot</a></code> | <code>java.lang.Boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.renovatebotOptions">renovatebotOptions</a></code> | <code>io.github.cdklabs.projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.infrastructure">infrastructure</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a></code> | Configuration for generated infrastructure. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.model">model</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a></code> | Configuration for the API model. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.runtime">runtime</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a></code> | Configuration for generated runtime projects (containing types, clients and server code). |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.documentation">documentation</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a></code> | Configuration for generated documentation. |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.name"></a>

- *Type:* java.lang.String
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.commitGenerated"></a>

- *Type:* java.lang.Boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.gitIgnoreOptions"></a>

- *Type:* io.github.cdklabs.projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.gitOptions"></a>

- *Type:* io.github.cdklabs.projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.logging"></a>

- *Type:* io.github.cdklabs.projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.outdir"></a>

- *Type:* java.lang.String
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.parent"></a>

- *Type:* io.github.cdklabs.projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenCommand"></a>

- *Type:* java.lang.String
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenrcJson"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.projenrcJsonOptions"></a>

- *Type:* io.github.cdklabs.projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.renovatebot"></a>

- *Type:* java.lang.Boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.renovatebotOptions"></a>

- *Type:* io.github.cdklabs.projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `infrastructure`<sup>Required</sup> <a name="infrastructure" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.infrastructure"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a>

Configuration for generated infrastructure.

---

##### `model`<sup>Required</sup> <a name="model" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.model"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a>

Configuration for the API model.

---

##### `runtime`<sup>Required</sup> <a name="runtime" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.runtime"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a>

Configuration for generated runtime projects (containing types, clients and server code).

---

##### `documentation`<sup>Optional</sup> <a name="documentation" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.documentation"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a>

Configuration for generated documentation.

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup">addExcludeFromCleanup</a></code> | Exclude the matching files from pre-synth cleanup. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore">addGitIgnore</a></code> | Adds a .gitignore pattern. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore">addPackageIgnore</a></code> | Exclude these files from the bundled package. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask">addTask</a></code> | Adds a new task to this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip">addTip</a></code> | Prints a "tip" message during synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated">annotateGenerated</a></code> | Consider a set of files as "generated". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.postSynthesize">postSynthesize</a></code> | Called after all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.preSynthesize">preSynthesize</a></code> | Called before all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask">removeTask</a></code> | Removes a task from a project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand">runTaskCommand</a></code> | Returns the shell command to execute in order to run a task. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.synth">synth</a></code> | Synthesize all project files into `outdir`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile">tryFindFile</a></code> | Finds a file at the specified relative path within this project and all its subprojects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile">tryFindJsonFile</a></code> | Finds a json file by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile">tryFindObjectFile</a></code> | Finds an object file (like JsonFile, YamlFile, etc.) by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile">tryRemoveFile</a></code> | Finds a file at the specified relative path within this project and removes it. |

---

##### `addExcludeFromCleanup` <a name="addExcludeFromCleanup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup"></a>

```java
public void addExcludeFromCleanup(java.lang.String globs)
```

Exclude the matching files from pre-synth cleanup.

Can be used when, for example, some
source files include the projen marker and we don't want them to be erased during synth.

###### `globs`<sup>Required</sup> <a name="globs" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup.parameter.globs"></a>

- *Type:* java.lang.String

The glob patterns to match.

---

##### `addGitIgnore` <a name="addGitIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore"></a>

```java
public void addGitIgnore(java.lang.String pattern)
```

Adds a .gitignore pattern.

###### `pattern`<sup>Required</sup> <a name="pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore.parameter.pattern"></a>

- *Type:* java.lang.String

The glob pattern to ignore.

---

##### `addPackageIgnore` <a name="addPackageIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore"></a>

```java
public void addPackageIgnore(java.lang.String _pattern)
```

Exclude these files from the bundled package.

Implemented by project types based on the
packaging mechanism. For example, `NodeProject` delegates this to `.npmignore`.

###### `_pattern`<sup>Required</sup> <a name="_pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore.parameter._pattern"></a>

- *Type:* java.lang.String

The glob pattern to exclude.

---

##### `addTask` <a name="addTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask"></a>

```java
public Task addTask(java.lang.String name)
public Task addTask(java.lang.String name, TaskOptions props)
```

Adds a new task to this project.

This will fail if the project already has
a task with this name.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask.parameter.name"></a>

- *Type:* java.lang.String

The task name to add.

---

###### `props`<sup>Optional</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask.parameter.props"></a>

- *Type:* io.github.cdklabs.projen.TaskOptions

Task properties.

---

##### ~~`addTip`~~ <a name="addTip" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip"></a>

```java
public void addTip(java.lang.String message)
```

Prints a "tip" message during synthesis.

###### `message`<sup>Required</sup> <a name="message" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip.parameter.message"></a>

- *Type:* java.lang.String

The message.

---

##### `annotateGenerated` <a name="annotateGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated"></a>

```java
public void annotateGenerated(java.lang.String _glob)
```

Consider a set of files as "generated".

This method is implemented by
derived classes and used for example, to add git attributes to tell GitHub
that certain files are generated.

###### `_glob`<sup>Required</sup> <a name="_glob" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated.parameter._glob"></a>

- *Type:* java.lang.String

the glob pattern to match (could be a file path).

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.postSynthesize"></a>

```java
public void postSynthesize()
```

Called after all components are synthesized.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.preSynthesize"></a>

```java
public void preSynthesize()
```

Called before all components are synthesized.

##### `removeTask` <a name="removeTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask"></a>

```java
public Task removeTask(java.lang.String name)
```

Removes a task from a project.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask.parameter.name"></a>

- *Type:* java.lang.String

The name of the task to remove.

---

##### `runTaskCommand` <a name="runTaskCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand"></a>

```java
public java.lang.String runTaskCommand(Task task)
```

Returns the shell command to execute in order to run a task.

By default, this is `npx projen@<version> <task>`

###### `task`<sup>Required</sup> <a name="task" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand.parameter.task"></a>

- *Type:* io.github.cdklabs.projen.Task

The task for which the command is required.

---

##### `synth` <a name="synth" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.synth"></a>

```java
public void synth()
```

Synthesize all project files into `outdir`.

1. Call "this.preSynthesize()"
2. Delete all generated files
3. Synthesize all sub-projects
4. Synthesize all components of this project
5. Call "postSynthesize()" for all components of this project
6. Call "this.postSynthesize()"

##### `tryFindFile` <a name="tryFindFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile"></a>

```java
public FileBase tryFindFile(java.lang.String filePath)
```

Finds a file at the specified relative path within this project and all its subprojects.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

If this path is relative, it will be resolved
from the root of _this_ project.

---

##### ~~`tryFindJsonFile`~~ <a name="tryFindJsonFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile"></a>

```java
public JsonFile tryFindJsonFile(java.lang.String filePath)
```

Finds a json file by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

---

##### `tryFindObjectFile` <a name="tryFindObjectFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile"></a>

```java
public ObjectFile tryFindObjectFile(java.lang.String filePath)
```

Finds an object file (like JsonFile, YamlFile, etc.) by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

---

##### `tryRemoveFile` <a name="tryRemoveFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile"></a>

```java
public FileBase tryRemoveFile(java.lang.String filePath)
```

Finds a file at the specified relative path within this project and removes it.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile.parameter.filePath"></a>

- *Type:* java.lang.String

The file path.

If this path is relative, it will be
resolved from the root of _this_ project.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.buildTask">buildTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.commitGenerated">commitGenerated</a></code> | <code>java.lang.Boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.compileTask">compileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.components">components</a></code> | <code>java.util.List<io.github.cdklabs.projen.Component></code> | Returns all the components within this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.deps">deps</a></code> | <code>io.github.cdklabs.projen.Dependencies</code> | Project dependencies. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.ejected">ejected</a></code> | <code>java.lang.Boolean</code> | Whether or not the project is being ejected. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.files">files</a></code> | <code>java.util.List<io.github.cdklabs.projen.FileBase></code> | All files in this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitattributes">gitattributes</a></code> | <code>io.github.cdklabs.projen.GitAttributesFile</code> | The .gitattributes file for this repository. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitignore">gitignore</a></code> | <code>io.github.cdklabs.projen.IgnoreFile</code> | .gitignore. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.logger">logger</a></code> | <code>io.github.cdklabs.projen.Logger</code> | Logging utilities. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.name">name</a></code> | <code>java.lang.String</code> | Project name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.outdir">outdir</a></code> | <code>java.lang.String</code> | Absolute output directory of this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.packageTask">packageTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.postCompileTask">postCompileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.preCompileTask">preCompileTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projectBuild">projectBuild</a></code> | <code>io.github.cdklabs.projen.ProjectBuild</code> | Manages the build process of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projenCommand">projenCommand</a></code> | <code>java.lang.String</code> | The command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.root">root</a></code> | <code>io.github.cdklabs.projen.Project</code> | The root project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.tasks">tasks</a></code> | <code>io.github.cdklabs.projen.Tasks</code> | Project tasks. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.testTask">testTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.defaultTask">defaultTask</a></code> | <code>io.github.cdklabs.projen.Task</code> | This is the "default" task, the one that executes "projen". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.initProject">initProject</a></code> | <code>io.github.cdklabs.projen.InitProject</code> | The options used when this project is bootstrapped via `projen new`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.parent">parent</a></code> | <code>io.github.cdklabs.projen.Project</code> | A parent project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.infrastructure">infrastructure</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a></code> | Generated infrastructure projects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.model">model</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | Project for the api model. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.runtime">runtime</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a></code> | Generated runtime projects. |

---

##### `buildTask`<sup>Required</sup> <a name="buildTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.buildTask"></a>

```java
public Task getBuildTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `commitGenerated`<sup>Required</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.commitGenerated"></a>

```java
public java.lang.Boolean getCommitGenerated();
```

- *Type:* java.lang.Boolean

Whether to commit the managed files by default.

---

##### `compileTask`<sup>Required</sup> <a name="compileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.compileTask"></a>

```java
public Task getCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `components`<sup>Required</sup> <a name="components" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.components"></a>

```java
public java.util.List<Component> getComponents();
```

- *Type:* java.util.List<io.github.cdklabs.projen.Component>

Returns all the components within this project.

---

##### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.deps"></a>

```java
public Dependencies getDeps();
```

- *Type:* io.github.cdklabs.projen.Dependencies

Project dependencies.

---

##### `ejected`<sup>Required</sup> <a name="ejected" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.ejected"></a>

```java
public java.lang.Boolean getEjected();
```

- *Type:* java.lang.Boolean

Whether or not the project is being ejected.

---

##### `files`<sup>Required</sup> <a name="files" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.files"></a>

```java
public java.util.List<FileBase> getFiles();
```

- *Type:* java.util.List<io.github.cdklabs.projen.FileBase>

All files in this project.

---

##### `gitattributes`<sup>Required</sup> <a name="gitattributes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitattributes"></a>

```java
public GitAttributesFile getGitattributes();
```

- *Type:* io.github.cdklabs.projen.GitAttributesFile

The .gitattributes file for this repository.

---

##### `gitignore`<sup>Required</sup> <a name="gitignore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitignore"></a>

```java
public IgnoreFile getGitignore();
```

- *Type:* io.github.cdklabs.projen.IgnoreFile

.gitignore.

---

##### `logger`<sup>Required</sup> <a name="logger" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.logger"></a>

```java
public Logger getLogger();
```

- *Type:* io.github.cdklabs.projen.Logger

Logging utilities.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.name"></a>

```java
public java.lang.String getName();
```

- *Type:* java.lang.String

Project name.

---

##### `outdir`<sup>Required</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.outdir"></a>

```java
public java.lang.String getOutdir();
```

- *Type:* java.lang.String

Absolute output directory of this project.

---

##### `packageTask`<sup>Required</sup> <a name="packageTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.packageTask"></a>

```java
public Task getPackageTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `postCompileTask`<sup>Required</sup> <a name="postCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.postCompileTask"></a>

```java
public Task getPostCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `preCompileTask`<sup>Required</sup> <a name="preCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.preCompileTask"></a>

```java
public Task getPreCompileTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `projectBuild`<sup>Required</sup> <a name="projectBuild" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projectBuild"></a>

```java
public ProjectBuild getProjectBuild();
```

- *Type:* io.github.cdklabs.projen.ProjectBuild

Manages the build process of the project.

---

##### `projenCommand`<sup>Required</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projenCommand"></a>

```java
public java.lang.String getProjenCommand();
```

- *Type:* java.lang.String

The command to use in order to run the projen CLI.

---

##### `root`<sup>Required</sup> <a name="root" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.root"></a>

```java
public Project getRoot();
```

- *Type:* io.github.cdklabs.projen.Project

The root project.

---

##### `tasks`<sup>Required</sup> <a name="tasks" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.tasks"></a>

```java
public Tasks getTasks();
```

- *Type:* io.github.cdklabs.projen.Tasks

Project tasks.

---

##### `testTask`<sup>Required</sup> <a name="testTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.testTask"></a>

```java
public Task getTestTask();
```

- *Type:* io.github.cdklabs.projen.Task

---

##### `defaultTask`<sup>Optional</sup> <a name="defaultTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.defaultTask"></a>

```java
public Task getDefaultTask();
```

- *Type:* io.github.cdklabs.projen.Task

This is the "default" task, the one that executes "projen".

Undefined if
the project is being ejected.

---

##### `initProject`<sup>Optional</sup> <a name="initProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.initProject"></a>

```java
public InitProject getInitProject();
```

- *Type:* io.github.cdklabs.projen.InitProject

The options used when this project is bootstrapped via `projen new`.

It
includes the original set of options passed to the CLI and also the JSII
FQN of the project type.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.parent"></a>

```java
public Project getParent();
```

- *Type:* io.github.cdklabs.projen.Project

A parent project.

If undefined, this is the root project.

---

##### `infrastructure`<sup>Required</sup> <a name="infrastructure" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.infrastructure"></a>

```java
public GeneratedCodeProjects getInfrastructure();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a>

Generated infrastructure projects.

Only the property corresponding to `infrastructure.language` will be defined.

---

##### `model`<sup>Required</sup> <a name="model" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.model"></a>

```java
public TypeSafeApiModelProject getModel();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

Project for the api model.

---

##### `runtime`<sup>Required</sup> <a name="runtime" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.runtime"></a>

```java
public GeneratedCodeProjects getRuntime();
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a>

Generated runtime projects.

When `runtime.languages` includes the corresponding language, the project can be
assumed to be defined.

---

#### Constants <a name="Constants" id="Constants"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.DEFAULT_TASK">DEFAULT_TASK</a></code> | <code>java.lang.String</code> | The name of the default task (the task executed when `projen` is run without arguments). |

---

##### `DEFAULT_TASK`<sup>Required</sup> <a name="DEFAULT_TASK" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.DEFAULT_TASK"></a>

```java
public java.lang.String getDefaultTask();
```

- *Type:* java.lang.String

The name of the default task (the task executed when `projen` is run without arguments).

Normally
this task should synthesize the project files.

---


## Enums <a name="Enums" id="Enums"></a>

### CustomAuthorizerType <a name="CustomAuthorizerType" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType"></a>

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.TOKEN">TOKEN</a></code> | A custom authorizer that uses a Lambda function. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.REQUEST">REQUEST</a></code> | An authorizer that uses a Lambda function using incoming request parameters. |

---

##### `TOKEN` <a name="TOKEN" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.TOKEN"></a>

A custom authorizer that uses a Lambda function.

---


##### `REQUEST` <a name="REQUEST" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.REQUEST"></a>

An authorizer that uses a Lambda function using incoming request parameters.

---


### DocumentationFormat <a name="DocumentationFormat" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat"></a>

Formats for documentation generation.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML_REDOC">HTML_REDOC</a></code> | HTML Documentation generated by redoc. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML2">HTML2</a></code> | OpenAPI Generator 'html2' documentation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.MARKDOWN">MARKDOWN</a></code> | OpenAPI Generator 'markdown' documentation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.PLANTUML">PLANTUML</a></code> | OpenAPI Generator 'plantuml' documentation. |

---

##### `HTML_REDOC` <a name="HTML_REDOC" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML_REDOC"></a>

HTML Documentation generated by redoc.

> [https://github.com/Redocly/redoc](https://github.com/Redocly/redoc)

---


##### `HTML2` <a name="HTML2" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML2"></a>

OpenAPI Generator 'html2' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/html2.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/html2.md)

---


##### `MARKDOWN` <a name="MARKDOWN" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.MARKDOWN"></a>

OpenAPI Generator 'markdown' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/markdown.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/markdown.md)

---


##### `PLANTUML` <a name="PLANTUML" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.PLANTUML"></a>

OpenAPI Generator 'plantuml' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/plantuml.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/plantuml.md)

---


### Language <a name="Language" id="@aws-prototyping-sdk/type-safe-api.Language"></a>

Supported languages for code generation.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.TYPESCRIPT">TYPESCRIPT</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.PYTHON">PYTHON</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.JAVA">JAVA</a></code> | *No description.* |

---

##### `TYPESCRIPT` <a name="TYPESCRIPT" id="@aws-prototyping-sdk/type-safe-api.Language.TYPESCRIPT"></a>

---


##### `PYTHON` <a name="PYTHON" id="@aws-prototyping-sdk/type-safe-api.Language.PYTHON"></a>

---


##### `JAVA` <a name="JAVA" id="@aws-prototyping-sdk/type-safe-api.Language.JAVA"></a>

---


### ModelLanguage <a name="ModelLanguage" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage"></a>

The model definition language.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage.SMITHY">SMITHY</a></code> | Smithy. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage.OPENAPI">OPENAPI</a></code> | OpenAPI. |

---

##### `SMITHY` <a name="SMITHY" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage.SMITHY"></a>

Smithy.

> [https://smithy.io/2.0/](https://smithy.io/2.0/)

---


##### `OPENAPI` <a name="OPENAPI" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage.OPENAPI"></a>

OpenAPI.

> [https://www.openapis.org/](https://www.openapis.org/)

---

