## Type Safe API

Define your APIs using [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and leverage the power of generated client and server types, infrastructure, documentation, and automatic input validation!

This package vends a projen project type which allows you to define an API using either [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and a construct which manages deploying this API in API Gateway, given an integration (eg a lambda) for every operation.

The project will generate "runtime" projects from your API definition in your desired languages, which can be utilised both client side for interacting with your API, or server side for implementing your API. The project also generates a type-safe CDK construct which ensures an integration is provided for every API operation.

Code is generated at build time, so when you change your API model, just rebuild and you'll see your changes reflected in the generated code.

### Quick Start: TypeScript

This section describes how to get started quickly, with TypeScript infrastructure and lambda handlers. See the end of the README for examples in [Python](#quick-start--python) and [Java](#quick-start--python).

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```ts
import { NxMonorepoProject } from "@aws-prototyping-sdk/nx-monorepo";
import { TypeSafeApiProject } from "@aws-prototyping-sdk/type-safe-api";
import { AwsCdkTypeScriptApp } from "projen/lib/awscdk";

// Create the monorepo
const monorepo = new NxMonorepoProject({ ... });

// Create the API project
const api = new TypeSafeApiProject({
  name: "myapi",
  parent: monorepo,
  outdir: 'packages/api',
  // Smithy as the model language. You can also use ModelLanguage.OPENAPI
  model: {
    language: ModelLanguage.SMITHY,
    options: {
      smithy: {
        serviceName: {
          namespace: 'com.mycompany',
          serviceName: 'MyApi',
        },
      },
    },
  },
  // Generate types, client and server code in TypeScript, Python and Java
  runtime: {
    languages: [Language.TYPESCRIPT, Language.PYTHON, Language.JAVA],
  },
  // CDK infrastructure in TypeScript
  infrastructure: {
    language: Language.TYPESCRIPT,
  },
  // Generate HTML documentation
  documentation: {
    formats: [DocumentationFormat.HTML_REDOC],
  }
});

// Create a CDK infrastructure project. Can also consider PDKPipelineTsProject as an alternative!
const infra = new AwsCdkTypeScriptApp({ ... });

// Infrastructure can depend on the generated API infrastructure and runtime
infra.addDeps(api.infrastructure.typescript!.package.packageName);
infra.addDeps(api.runtime.typescript!.package.packageName);

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application (ie within the `infra` project we created), consume the `Api` construct, vended from the generated typescript infrastructure package.

```ts
import { Stack, StackProps } from 'aws-cdk-lib';
import { Construct } from 'constructs';
import { Api } from "myapi-typescript-infra"; // <- generated typescript infrastructure package
import { Authorizers, Integrations } from "@aws-prototyping-sdk/type-safe-api";
import { NodejsFunction } from "aws-cdk-lib/aws-lambda-nodejs";
import { Cors } from "aws-cdk-lib/aws-apigateway";
import * as path from 'path';

export class MyStack extends Stack {
  constructor(scope: Construct, id: string, props: StackProps = {}) {
    super(scope, id, props);

    // Instantiate the generated CDK construct to deploy an API Gateway API based on your model
    new Api(this, 'MyApi', {
      defaultAuthorizer: Authorizers.iam(),
      corsOptions: {
        allowOrigins: Cors.ALL_ORIGINS,
        allowMethods: Cors.ALL_METHODS,
      },
      // Supply an integration for every operation
      integrations: {
        sayHello: {
          integration: Integrations.lambda(new NodejsFunction(this, 'SayHelloLambda', {
            entry: path.resolve(__dirname, 'say-hello.ts'),
          })),
        },
      },
    });
  }
}
```

#### Implement a Lambda Handler

The generated runtime projects include lambda handler wrappers which provide type-safety for implementing your API operations. You can implement your lambda handlers in any of the supported languages, and even mix and match languages for different operations if you like.

In the above CDK application, we used `NodejsFunction` with the entry point as `say-hello.ts`, so we can define the lambda function in the same `infra` project.

In typescript, the implementation of `say-hello.ts` would look like:

```ts
import { sayHelloHandler } from "myapi-typescript-runtime"; // <- generated typescript runtime package

// Use the handler wrapper for type-safety to ensure you correctly implement your modelled API operation
export const handler = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}`,
    },
  };
});
```

### Project

The `TypeSafeApiProject` projen project sets up the project structure for you. You have a few parameters to consider when creating the project:

* `model` - Configure the API model. Select a `language` for the model of either [Smithy](https://smithy.io/2.0/) or [OpenAPI v3](https://swagger.io/specification/), and supply `options.smithy` or `options.openapi` depending on your choice.
* `runtime` - Configure the generated runtime projects. Include one or more `languages` you wish to write your client and server-side code in. These projects contain generated types defined in your model, as well as type-safe lambda handler wrappers for implementing each operation.
* `infrastructure` - Pick the `language` you are writing your CDK infrastructure in. A construct will be generated in this language which can be used to deploy the API.
* `documentation` - Specify `formats` to generate documentation in.

It's recommended that these projects are used as part of an `nx-monorepo` project (eg. by specifying `parent: myMonorepoProject`), as it makes setting up dependencies much easier, particularly when extending your project further with a CDK app and lambda functions.

Depending on the `model.language` you choose, you must supply the corresponding `model.options`. For example:

```ts
new TypeSafeApiProject({
  model: {
    language: ModelLanguage.SMITHY,
    options: {
      smithy: {
        serviceName: {
          namespace: 'com.mycompany',
          serviceName: 'MyApi',
        },
      },
    },
  },
  ...
});
```

```ts
new TypeSafeApiProject({
  model: {
    language: ModelLanguage.OPENAPI,
    options: {
      openapi: {
        title: 'MyApi',
      },
    },
  },
  ...
});
```

`model.options.smithy` allows for further customisation of the Smithy project, eg:

```ts
new TypeSafeApiProject({
  model: {
    language: ModelLanguage.SMITHY,
    options: {
      smithy: {
        serviceName: {
          namespace: 'com.mycompany',
          serviceName: 'MyApi',
        },
        // By default, the contents of the smithy build output directory `model/output` will be ignored by source control.
        // Set this to false to include it, for example if you are generating clients directly from the smithy model.
        ignoreSmithyBuildOutput: false,
        // The gradle wrapper used for the smithy build is copied from the PDK itself if it does not already exist in
        // the 'smithy' folder. By default, this gradle wrapper will be ignored by source control.
        // Set this to false if you would like to check the gradle wrapper in to source control, for example if you want
        // to use a different version of the gradle wrapper in your project.
        ignoreGradleWrapper: false,
        // Use smithyBuildOptions to control what is added to smithy-build.json.
        smithyBuildOptions: {
          projections: {
            // You can customise the built-in openapi projection, used to generate the OpenAPI specification.
            openapi: {
              plugins: {
                openapi: {
                  // Customise the openapi projection here.
                  // See: https://smithy.io/2.0/guides/converting-to-openapi.html
                  useIntegerType: true,
                  ...
                }
              }
            },
            // You can add new projections here too
            "ts-client": {
              "plugins": {
                "typescript-codegen": {
                  "package" : "@my-test/smithy-generated-typescript-client",
                  "packageVersion": "0.0.1"
                }
              }
            }
          },
          // Note that any additional dependencies required for projections/plugins can be added here, which in turn will
          // add them to the `smithy/build.gradle` file
          maven: {
            dependencies: [
              "software.amazon.smithy:smithy-validation-model:1.27.2",
            ]
          }
        }
      },
    }
  },
  ...
});
```

#### Directory Structure

The `TypeSafeApiProject` will create the following directory structure within its `outdir`:

```
|_ model/
    |_ src/
        |_ main/
            |_ smithy - your API definition if you chose ModelLanguage.SMITHY
            |_ openapi - your API definition if you chose ModelLanguage.OPENAPI
|_ runtime/ - generated types, client, and server code in the languages you specified
    |_ typescript
    |_ python
    |_ java
|_ infrastructure/ - generated infrastructure (you'll find only one directory in here based on your chosen infrastructure language)
    |_ typescript
    |_ python
    |_ java
|_ documentation/ - generated documentation in the formats you specified
    |_ html2
    |_ html_redoc
    |_ plantuml
    |_ markdown
```


### Smithy IDL

Please refer to the [Smithy documentation](https://smithy.io/2.0/quickstart.html) for how to write models in Smithy. A basic example is provided below:

```smithy
$version: "2"
namespace example.hello

use aws.protocols#restJson1

@title("A Sample Hello World API")

/// A sample smithy api
@restJson1
service Hello {
    version: "1.0"
    operations: [SayHello]
}

@readonly
@http(method: "GET", uri: "/hello")
operation SayHello {
    input: SayHelloInput
    output: SayHelloOutput
    errors: [ApiError]
}

string Name
string Message

@input
structure SayHelloInput {
    @httpQuery("name")
    @required
    name: Name
}

@output
structure SayHelloOutput {
    @required
    message: Message
}

@error("client")
structure ApiError {
    @required
    errorMessage: Message
}
```

#### Supported Protocols

Currently only [AWS restJson1](https://smithy.io/2.0/aws/protocols/aws-restjson1-protocol.html) is supported. Please ensure your service is annotated with the `@restJson1` trait.

#### Multiple Files

You can split your definition into multiple files and folders, so long as they are all under the `model/src/main/smithy` directory in your API project.

#### Authorizers

Smithy supports [adding API Gateway authorizers in the model itself](https://smithy.io/2.0/aws/aws-auth.html). Given that at model definition time one usually does not know the ARN of the user pool or lambda function for an authorizer, it is recommended to add the authorizer(s) in your Api CDK construct.

If using Smithy generated clients, some authorizer traits (eg sigv4) will include configuring the client for that particular method of authorization, so it can be beneficial to still define authorizers in the model. We therefore support specifying authorizers in both the model and the construct, but note that the construct will take precedence where the authorizer ID is the same.

### OpenAPI Specification

Your `model/src/main/openapi/main.yaml` file defines your api using [OpenAPI Version 3.0.3](https://swagger.io/specification/). An example spec might look like:

```yaml
openapi: 3.0.3
info:
  version: 1.0.0
  title: Example API
paths:
  /hello:
    get:
      operationId: sayHello
      parameters:
        - in: query
          name: name
          schema:
            type: string
          required: true
      responses:
        '200':
          description: Successful response
          content:
            'application/json':
              schema:
                $ref: '#/components/schemas/HelloResponse'
components:
  schemas:
    HelloResponse:
      type: object
      properties:
        message:
          type: string
      required:
        - message
```

You can divide your specification into multiple files using `$ref`.

For example, you might choose to structure your spec as follows:

```
|_ model/src/main/openapi/
    |_ main.yaml
    |_ paths/
        |_ index.yaml
        |_ sayHello.yaml
    |_ schemas/
        |_ index.yaml
        |_ helloResponse.yaml
```

Where `main.yaml` looks as follows:

```yaml
openapi: 3.0.3
info:
  version: 1.0.0
  title: Example API
paths:
  $ref: './paths/index.yaml'
components:
  schemas:
    $ref: './schemas/index.yaml'
```

`paths/index.yaml`:

```yaml
/hello:
  get:
    $ref: './sayHello.yaml'
```

`paths/sayHello.yaml`:

```yaml
operationId: sayHello
parameters:
 - in: query
   name: name
   schema:
     type: string
   required: true
responses:
  '200':
    description: Successful response
    content:
      'application/json':
        schema:
          $ref: '../schemas/helloResponse.yaml'
```

`schemas/index.yaml`:

```yaml
HelloResponse:
  $ref: './helloResponse.yaml'
```

`schemas/helloResponse.yaml`:

```yaml
type: object
properties:
  message:
    type: string
required:
  - message
```

### Construct

A CDK construct is generated in the `infrastructure/<language>` directory which provides a type-safe interface for creating an API Gateway API based on your model.

You can extend or instantiate this construct in your CDK infrastructure project. You'll get a type error if you forget to define an integration for an operation defined in your api.

```ts
import { Authorizers, Integrations } from '@aws-prototyping-sdk/type-safe-api';
import { NodejsFunction } from 'aws-cdk-lib/aws-lambda-nodejs';
import { Construct } from 'constructs';
import { Api } from 'myapi-typescript-infra';

/**
 * An example of how to wire lambda handler functions to the API
 */
export class SampleApi extends Api {
  constructor(scope: Construct, id: string) {
    super(scope, id, {
      defaultAuthorizer: Authorizers.iam(),
      integrations: {
        // Every operation defined in your API must have an integration defined!
        sayHello: {
          integration: Integrations.lambda(new NodejsFunction(scope, 'say-hello')),
        },
      },
    });
  }
}
```

#### Sharing Integrations

If you would like to use the same integration for every operation (for example you'd like to use a single lambda function to service all requests with the in-built [handler router](#handler-router)), you can use the `Operations.all` method from a generated runtime project to save repeating yourself:

```ts
import { Operations } from 'myapi-typescript-runtime';
import { Authorizers, Integrations } from '@aws-prototyping-sdk/type-safe-api';
import { NodejsFunction } from 'aws-cdk-lib/aws-lambda-nodejs';
import { Construct } from 'constructs';
import { Api } from 'myapi-typescript-infra';

export class SampleApi extends Api {
  constructor(scope: Construct, id: string) {
    super(scope, id, {
      defaultAuthorizer: Authorizers.iam(),
      // Use the same integration for every operation.
      integrations: Operations.all({
        integration: Integrations.lambda(new NodejsFunction(scope, 'router')),
      }),
    });
  }
}
```

TypeScript is demonstrated above, but this is also available in Java and Python.

#### Authorizers

The `Api` construct allows you to define one or more authorizers for securing your API. An integration will use the `defaultAuthorizer` unless an `authorizer` is specified at the integration level. The following authorizers are supported:

* `Authorizers.none` - No auth
* `Authorizers.iam` - AWS IAM (Signature Version 4)
* `Authorizers.cognito` - Cognito user pool
* `Authorizers.custom` - A custom authorizer

##### Cognito Authorizer

To use the Cognito authorizer, one or more user pools must be provided. You can optionally specify the scopes to check if using an access token. You can use the `withScopes` method to use the same authorizer but verify different scopes for individual integrations, for example:

```ts
export class SampleApi extends Api {
  constructor(scope: Construct, id: string) {
    const cognitoAuthorizer = Authorizers.cognito({
      authorizerId: 'myCognitoAuthorizer',
      userPools: [new UserPool(scope, 'UserPool')],
    });

    super(scope, id, {
      defaultAuthorizer: cognitoAuthorizer,
      integrations: {
        // Everyone in the user pool can call this operation:
        sayHello: {
          integration: Integrations.lambda(new NodejsFunction(scope, 'say-hello')),
        },
        // Only users with the given scopes can call this operation
        myRestrictedOperation: {
          integration: Integrations.lambda(new NodejsFunction(scope, 'my-restricted-operation')),
          authorizer: cognitoAuthorizer.withScopes('my-resource-server/my-scope'),
        },
      },
    });
  }
}
```

For more information about scopes or identity and access tokens, please see the [API Gateway documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-integrate-with-cognito.html).

##### Custom Authorizer

Custom authorizers use lambda functions to handle authorizing requests. These can either be simple token-based authorizers, or more complex request-based authorizers. See the [API Gateway documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-use-lambda-authorizer.html) for more details.

An example token-based authorizer (default):

```ts
Authorizers.custom({
  authorizerId: 'myTokenAuthorizer',
  function: new NodejsFunction(scope, 'authorizer'),
});
```

An example request-based handler. By default the identitySource will be `method.request.header.Authorization`, however you can customise this as per [the API Gateway documentation](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource).

```ts
Authorizers.custom({
  authorizerId: 'myRequestAuthorizer',
  type: CustomAuthorizerType.REQUEST,
  identitySource: 'method.request.header.MyCustomHeader, method.request.querystring.myQueryString',
  function: new NodejsFunction(scope, 'authorizer'),
});
```

#### Integrations

Integrations are used by API Gateway to service requests.

##### Lambda Integration

Currently, the only built-in integration is a lambda integration. You can construct one using `Integrations.lambda(yourLambdaFunction)`.

##### Custom Integrations

You can implement your own integrations by inheriting the `Integration` class and implementing its `render` method. This method is responsible for returning a snippet of OpenAPI which will be added as the `x-amazon-apigateway-integration` for an operation. Please refer to the [API Gateway Swagger Extensions documentation](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html) for more details.

You can also optionally override the `grant` method if you need to use CDK to grant API Gateway access to invoke your integration.

### Runtime: Clients

The generated runtime projects include clients which can be used for type-safe interaction with your API.

#### Typescript

The [typescript-fetch](https://openapi-generator.tech/docs/generators/typescript-fetch/) OpenAPI generator is used to generate typescript client. This requires an implementation of `fetch` to be passed to the client. In the browser one can pass the built in fetch, or in NodeJS you can use an implementation such as [node-fetch](https://www.npmjs.com/package/node-fetch).

Example usage of the client in a website:

```ts
import { Configuration, DefaultApi } from "myapi-typescript-runtime";

const client = new DefaultApi(new Configuration({
  basePath: "https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com",
  fetchApi: window.fetch.bind(window),
}));

await client.sayHello({ name: "Jack" });
```

#### Python

The [python](https://openapi-generator.tech/docs/generators/python) OpenAPI generator is used to generate clients for python.

Example usage:

```python
from myapi_python_runtime import ApiClient, Configuration
from myapi_python_runtime.api.default_api import DefaultApi

configuration = Configuration(
    host = "https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com"
)

with ApiClient(configuration) as api_client:
    client = DefaultApi(api_client)

    client.say_hello(
        query_params={
            'name': "name_example",
        },
    )
```

You'll find details about how to use the python client in the README.md in your generated runtime project.

#### Java

The [java](https://openapi-generator.tech/docs/generators/java/) OpenAPI generator is used to generate clients for Java.

Example usage:

```java
import com.generated.api.myapijavaruntime.runtime.api.DefaultApi;
import com.generated.api.myapijavaruntime.runtime.ApiClient;
import com.generated.api.myapijavaruntime.runtime.Configuration;
import com.generated.api.myapijavaruntime.runtime.models.HelloResponse;

ApiClient client = Configuration.getDefaultApiClient();
client.setBasePath("https://xxxxxxxxxx.execute-api.ap-southeast-2.amazonaws.com");

DefaultApi api = new DefaultApi(client);
HelloResponse response = api.sayHello("Adrian").execute()
```

You'll find more details about how to use the Java client in the README.md in your generated runtime project.

### Lambda Handler Wrappers

Lambda handler wrappers are also importable from the generated runtime projects. These provide input/output type safety, ensuring that your API handlers return outputs that correspond to your model.

#### Typescript

```ts
import { sayHelloHandler } from "myapi-typescript-runtime";

export const handler = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so with the `handlerRouter`.

```ts
import { handlerRouter, sayHelloHandler, sayGoodbyeHandler } from "myapi-typescript-runtime";
import { corsInterceptor } from "./interceptors";
import { sayGoodbye } from "./handlers/say-goodbye";

const sayHello = sayHelloHandler(async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});

export const handler = handlerRouter({
  // Interceptors declared in this list will apply to all operations
  interceptors: [corsInterceptor],
  // Assign handlers to each operation here
  handlers: {
    sayHello,
    sayGoodbye,
  },
});
```

#### Python

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse

@say_hello_handler
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so with the `handler_router`.

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses, handler_router, HandlerRouterHandlers
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse
from other_handlers import say_goodbye
from my_interceptors import cors_interceptor

@say_hello_handler
def say_hello(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )

handler = handler_router(
  # Interceptors defined here will apply to all operations
  interceptors=[cors_interceptor],
  handlers=HandlerRouterHandlers(
    say_hello=say_hello,
    say_goodbye=say_goodbye
  )
)
```

#### Java

```java
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello200Response;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloRequestInput;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloResponse;
import com.generated.api.myapijavaruntime.runtime.model.HelloResponse;


public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

##### Handler Router

The lambda handler wrappers can be used in isolation as handler methods for separate lambdas. If you would like to use a single lambda function to serve all requests, you can do so by extending the `HandlerRouter` class.

```java
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayGoodbye;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.HandlerRouter;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.Interceptors;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;

import java.util.Arrays;
import java.util.List;

// Interceptors defined here apply to all operations
@Interceptors({ TimingInterceptor.class })
public class ApiHandlerRouter extends HandlerRouter {
    // You must implement a method to return a handler for every operation
    @Override
    public SayHello sayHello() {
        return new SayHelloHandler();
    }

    @Override
    public SayGoodbye sayGoodbye() {
        return new SayGoodbyeHandler();
    }
}
```

### Interceptors

The lambda handler wrappers allow you to pass in a _chain_ of handler functions to handle the request. This allows you to implement middleware / interceptors for handling requests. Each handler function may choose whether or not to continue the handler chain by invoking `chain.next`.

#### Typescript

In typescript, interceptors are passed as separate arguments to the generated handler wrapper, in the order in which they should be executed. Call `request.chain.next(request)` from an interceptor to delegate to the rest of the chain to handle a request. Note that the last handler in the chain (ie the actual request handler which transforms the input to the output) should not call `chain.next`.

```ts
import {
  sayHelloHandler,
  ChainedRequestInput,
  OperationResponse,
} from "myapi-typescript-runtime";

// Interceptor to wrap invocations in a try/catch, returning a 500 error for any unhandled exceptions.
const tryCatchInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response | OperationResponse<500, { errorMessage: string }>> => {
  try {
    return await request.chain.next(request);
  } catch (e: any) {
    return { statusCode: 500, body: { errorMessage: e.message }};
  }
};

// tryCatchInterceptor is passed first, so it runs first and calls the second argument function (the request handler) via chain.next
export const handler = sayHelloHandler(tryCatchInterceptor, async ({ input }) => {
  return {
    statusCode: 200,
    body: {
      message: `Hello ${input.requestParameters.name}!`,
    },
  };
});
```

Another example interceptor might be to record request time metrics. The example below includes the full generic type signature for an interceptor:

```ts
import {
  ChainedRequestInput,
} from 'myapi-typescript-runtime';

const timingInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response> => {
  const start = Date.now();
  const response = await request.chain.next(request);
  const end = Date.now();
  console.log(`Took ${end - start} ms`);
  return response;
};
```

Interceptors may mutate the `interceptorContext` to pass state to further interceptors or the final lambda handler, for example an `identityInterceptor` might want to extract the authenticated user from the request so that it is available in handlers.

```ts
import {
  LambdaRequestParameters,
  LambdaHandlerChain,
} from 'myapi-typescript-runtime';

const identityInterceptor = async <RequestParameters, RequestArrayParameters, RequestBody, Response>(
  request: ChainedRequestInput<RequestParameters, RequestArrayParameters, RequestBody, Response>
): Promise<Response> => {
  const authenticatedUser = await getAuthenticatedUser(request.event);
  return await request.chain.next({
    ...request,
    interceptorContext: {
      ...request.interceptorContext,
      authenticatedUser,
    },
  });
};
```

#### Python

In Python, a list of interceptors can be passed as a keyword argument to the generated lambda handler decorator, for example:

```python
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler, SayHelloRequest, ApiResponse, SayHelloOperationResponses
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.model.hello_response import HelloResponse

@say_hello_handler(interceptors=[timing_interceptor, try_catch_interceptor])
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=HelloResponse(message="Hello {}!".format(input.request_parameters["name"])),
        headers={}
    )
```

Writing an interceptor is just like writing a lambda handler. Call `chain.next(input)` from an interceptor to delegate to the rest of the chain to handle a request.

```python
import time
from myapi_python_runtime.apis.tags.default_api_operation_config import ChainedApiRequest, ApiResponse

def timing_interceptor(input: ChainedApiRequest) -> ApiResponse:
    start = int(round(time.time() * 1000))
    response = input.chain.next(input)
    end = int(round(time.time() * 1000))
    print("Took {} ms".format(end - start))
    return response
```

Interceptors may choose to return different responses, for example to return a 500 response for any unhandled exceptions:

```python
import time
from myapi_python_runtime.model.api_error import ApiError
from myapi_python_runtime.apis.tags.default_api_operation_config import ChainedApiRequest, ApiResponse

def try_catch_interceptor(input: ChainedApiRequest) -> ApiResponse:
    try:
        return input.chain.next(input)
    except Exception as e:
        return ApiResponse(
            status_code=500,
            body=ApiError(errorMessage=str(e)),
            headers={}
        )
```

Interceptors are permitted to mutate the "interceptor context", which is a `Dict[str, Any]`. Each interceptor in the chain, and the final handler, can access this context:

```python
def identity_interceptor(input: ChainedApiRequest) -> ApiResponse:
    input.interceptor_context["AuthenticatedUser"] = get_authenticated_user(input.event)
    return input.chain.next(input)
```

Interceptors can also mutate the response returned by the handler chain. An example use case might be adding cross-origin resource sharing headers:

```python
def add_cors_headers_interceptor(input: ChainedApiRequest) -> ApiResponse:
    response = input.chain.next(input)
    return ApiResponse(
        status_code=response.status_code,
        body=response.body,
        headers={
            **response.headers,
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Headers": "*"
        }
    )
```

#### Java

In Java, interceptors can be added to a handler via the `@Interceptors` class annotation:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptors;

@Interceptors({ TimingInterceptor.class, TryCatchInterceptor.class })
public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

To write an interceptor, you can implement the `Interceptor` interface. For example, a timing interceptor:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptor;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ChainedRequestInput;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Response;

public class TimingInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        long start = System.currentTimeMillis();
        Response res = input.getChain().next(input);
        long end = System.currentTimeMillis();
        System.out.printf("Took %d ms%n", end - start);
        return res;
    }
}
```

Interceptors may choose to return different responses, for example to return a 500 response for any unhandled exceptions:

```java
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Interceptor;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ChainedRequestInput;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.Response;
import com.generated.api.myjavaapiruntime.runtime.api.Handlers.ApiResponse;
import com.generated.api.myjavaapiruntime.runtime.model.ApiError;

public class TryCatchInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        try {
            return input.getChain().next(input);
        } catch (Exception e) {
            return ApiResponse.builder()
                    .statusCode(500)
                    .body(ApiError.builder()
                            .errorMessage(e.getMessage())
                            .build().toJson())
                    .build();
        }
    }
}
```

Interceptors are permitted to mutate the "interceptor context", which is a `Map<String, Object>`. Each interceptor in the chain, and the final handler, can access this context:

```java
public class IdentityInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        input.getInterceptorContext().put("AuthenticatedUser", this.getAuthenticatedUser(input.getEvent()));
        return input.getChain().next(input);
    }
}
```

Interceptors can also mutate the response returned by the handler chain. An example use case might be adding cross-origin resource sharing headers:

```java
public static class AddCorsHeadersInterceptor<Input> implements Interceptor<Input> {
    @Override
    public Response handle(ChainedRequestInput<Input> input) {
        Response res = input.getChain().next(input);
        res.getHeaders().put("Access-Control-Allow-Origin", "*");
        res.getHeaders().put("Access-Control-Allow-Headers", "*");
        return res;
    }
}
```

##### Interceptors with Dependency Injection

Interceptors referenced by the `@Interceptors` annotation must be constructable with no arguments. If more complex instantiation of your interceptor is required (for example if you are using dependency injection or wish to pass configuration to your interceptor), you may instead override the `getInterceptors` method in your handler:

```java
public class SayHelloHandler extends SayHello {
    @Override
    public List<Interceptor<SayHelloInput>> getInterceptors() {
        return Arrays.asList(
                new MyConfiguredInterceptor<>(42),
                new MyOtherConfiguredInterceptor<>("configuration"));
    }

    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(HelloResponse.builder()
                .message(String.format("Hello %s!", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}
```

### Quick Start: Python

This guide assumes you want to write your CDK infrastructure in Python and your lambda handlers in Python, however note that you your infrastructure language and lambda handler language(s) are not tied to one another, you can mix and match as you like. Just specify the language in `runtime.languages` for any language you would like to write lambda handlers in.

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```ts
import { NxMonorepoProject } from "@aws-prototyping-sdk/nx-monorepo";
import { TypeSafeApiProject } from "@aws-prototyping-sdk/type-safe-api";
import { AwsCdkTypeScriptApp } from "projen/lib/awscdk";
import { PythonProject } from "projen/lib/python";

// Create the monorepo
const monorepo = new NxMonorepoProject({
  name: 'monorepo',
  defaultReleaseBranch: "main",
});

// Create the API project
const api = new TypeSafeApiProject({
  name: "myapi",
  parent: monorepo,
  outdir: 'packages/api',
  // Smithy as the model language. You can also use ModelLanguage.OPENAPI
  model: {
    language: ModelLanguage.SMITHY,
    options: {
      smithy: {
        serviceName: {
          namespace: 'com.mycompany',
          serviceName: 'MyApi',
        },
      },
    },
  },
  // Generate client and server types in TypeScript, Python, and Java
  runtime: {
    languages: [Language.TYPESCRIPT, Language.PYTHON, Language.JAVA],
  },
  // Generate CDK infrastructure in Python
  infrastructure: {
    language: Language.PYTHON,
  },
  // Generate HTML documentation
  documentation: {
    formats: [DocumentationFormat.HTML_REDOC]
  },
});

// Create a project for our lambda handlers written in python
const lambdas = new PythonProject({
  name: "lambdas",
  parent: monorepo,
  outdir: 'packages/lambdas',
  authorEmail: "me@example.com",
  authorName: "me",
  moduleName: "lambdas",
  version: "1.0.0",
  // Poetry is used to simplify local python dependencies
  poetry: true,
});

// Add a local dependency on the generated python runtime
monorepo.addPythonPoetryDependency(lambdas, api.runtime.python!);

// Add commands to the lambda project's package task to create a distributable which can be deployed to AWS Lambda
lambdas.packageTask.exec(`mkdir -p lambda-dist && rm -rf lambda-dist/*`);
lambdas.packageTask.exec(`cp -r ${lambdas.moduleName} lambda-dist/${lambdas.moduleName}`);
lambdas.packageTask.exec(`poetry export --without-hashes --format=requirements.txt > lambda-dist/requirements.txt`);
lambdas.packageTask.exec(`pip install -r lambda-dist/requirements.txt --target lambda-dist --upgrade`);
lambdas.gitignore.addPatterns('lambda-dist');

// Create a CDK infrastructure project
const infra = new AwsCdkPythonApp({
  name: "infra",
  parent: monorepo,
  outdir: 'packages/infra',
  authorEmail: "me@example.com",
  authorName: "me",
  cdkVersion: "2.0.0",
  moduleName: "infra",
  version: "1.0.0",
  poetry: true,
});

// The infrastructure project depends on the python types, python infrastructure, and the lambda package
monorepo.addPythonPoetryDependency(infra, api.runtime.python!);
monorepo.addPythonPoetryDependency(infra, api.infrastructure.python!);
monorepo.addPythonPoetryDependency(infra, lambdas);

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application, consume the `Api` construct, vended from the generated Python infrastructure package.

```python
import os
from aws_cdk import Stack
from constructs import Construct
from aws_cdk.aws_lambda import LayerVersion, Code, Function, Runtime
from aws_prototyping_sdk.type_safe_api import Authorizers, TypeSafeApiIntegration, Integrations

from myapi_python_runtime.apis.tags.default_api_operation_config import OperationConfig
from myapi_python_infra.api import Api
from pathlib import Path
from os import path

class MyStack(Stack):
    def __init__(self, scope: Construct, construct_id: str, **kwargs) -> None:
        super().__init__(scope, construct_id, **kwargs)

        # Use the generated Api construct
        self.api = Api(self, 'Api',
            default_authorizer=Authorizers.iam(),
            integrations=OperationConfig(
                say_hello=TypeSafeApiIntegration(
                    # Create a python lambda function from our "lambda-dist" package
                    integration=Integrations.lambda_(Function(self, 'SayHello',
                        runtime=Runtime.PYTHON_3_9,
                        code=Code.from_asset(path.join("..", "lambdas", "lambda-dist")),
                        handler="lambdas.say_hello.handler",
                    )),
                ),
            ),
        )
```

#### Implement a Lambda Handler

In your `lambdas` project you can define your lambda handler in its source directory, eg `lambdas/lambdas/say_hello.py`:

```python
from myapi_python_runtime.model.say_hello_response_content import SayHelloResponseContent
from myapi_python_runtime.apis.tags.default_api_operation_config import say_hello_handler,
    SayHelloRequest, SayHelloOperationResponses, ApiResponse


@say_hello_handler
def handler(input: SayHelloRequest, **kwargs) -> SayHelloOperationResponses:
    return ApiResponse(
        status_code=200,
        body=SayHelloResponseContent(message="Hello {}".format(input.request_parameters["name"])),
        headers={}
    )
```

### Quick Start: Java

This guide assumes you want to write your CDK infrastructure in Java and your lambda handlers in Java, however note that you your infrastructure language and lambda handler language(s) are not tied to one another, you can mix and match as you like.

#### Create Your API Project

Use the project in your `.projenrc.ts`. It can either be part of an [`nx-monorepo`](../nx-monorepo/README.md) (recommended) or used in a standalone fashion.

```ts
import { NxMonorepoProject } from "@aws-prototyping-sdk/nx-monorepo";
import { TypeSafeApiProject } from "@aws-prototyping-sdk/type-safe-api";
import { AwsCdkTypeScriptApp } from "projen/lib/awscdk";
import { JavaProject } from "projen/lib/java";

// Create the monorepo
const monorepo = new NxMonorepoProject({
  name: 'monorepo',
  defaultReleaseBranch: "main",
});

// Create the API project
const api = new TypeSafeApiProject({
  name: "myapi",
  parent: monorepo,
  outdir: 'packages/api',
  // Smithy as the model language. You can also use ModelLanguage.OPENAPI
  model: {
    language: ModelLanguage.SMITHY,
    options: {
      smithy: {
        serviceName: {
          namespace: 'com.mycompany',
          serviceName: 'MyApi',
        },
      },
    },
  },
  // Generate client and server types in TypeScript, Python and Java
  runtime: {
    languages: [Language.TYPESCRIPT, Language.PYTHON, Language.JAVA],
  },
  // Generate CDK infrastructure in Java
  infrastructure: {
    language: Language.JAVA,
  },
  // Generate HTML documentation
  documentation: {
    formats: [DocumentationFormat.HTML_REDOC],
  },
});

const lambdas = new JavaProject({
  name: "lambdas",
  parent: monorepo,
  outdir: 'packages/lambdas',
  artifactId: "lambdas",
  groupId: "com.my.api",
  version: "1.0.0",
});

// The lambdas package needs a dependency on the generated java runtime
monorepo.addJavaDependency(lambdas, api.runtime.java!);

// Use the maven shade plugin to build a "super jar" which we can deploy to AWS Lambda
lambdas.pom.addPlugin("org.apache.maven.plugins/maven-shade-plugin@3.3.0", {
  configuration: {
    createDependencyReducedPom: false,
  },
  executions: [
    {
      id: "shade-task",
      phase: "package",
      goals: ["shade"],
    },
  ],
});

const infra = new AwsCdkJavaApp({
  name: "infra",
  parent: monorepo,
  outdir: 'packages/infra',
  artifactId: "infra",
  groupId: "com.my.api",
  mainClass: "com.my.api.MyApp",
  version: "1.0.0",
  cdkVersion: "2.0.0",
});

// Add a dependency on the generated CDK infrastructure
monorepo.addJavaDependency(infra, api.infrastructure.java!);

// Make sure the java lambda builds before our CDK infra
monorepo.addImplicitDependency(infra, lambdas);

monorepo.synth();
```

#### Use the CDK Construct

In your CDK application, consume the `Api` construct, vended from the generated Java infrastructure package.

```java
package com.my.api;

import com.generated.api.myapijavainfra.infra.Api;
import com.generated.api.myapijavainfra.infra.ApiProps;
import com.generated.api.myapijavaruntime.runtime.api.OperationConfig;

import software.amazon.awscdk.Duration;
import software.amazon.awscdk.services.apigateway.CorsOptions;
import software.amazon.awscdk.services.lambda.Code;
import software.amazon.awscdk.services.lambda.Function;
import software.amazon.awscdk.services.lambda.FunctionProps;
import software.amazon.awscdk.services.lambda.Runtime;
import software.aws.awsprototypingsdk.typesafeapi.Authorizers;
import software.aws.awsprototypingsdk.typesafeapi.Integrations;
import software.aws.awsprototypingsdk.typesafeapi.TypeSafeApiIntegration;

import software.amazon.awscdk.App;
import software.amazon.awscdk.Stack;

import java.util.Arrays;

public class MyApp {
    public static void main(final String[] args) {
        App app = new App();
        Stack s = new Stack(app, "infra");

        // Declare the API construct to deploy the API Gateway resources
        new Api(s, "Api", ApiProps.builder()
                .defaultAuthorizer(Authorizers.iam())
                .corsOptions(CorsOptions.builder()
                        .allowOrigins(Arrays.asList("*"))
                        .allowMethods(Arrays.asList("*"))
                        .build())
                .integrations(OperationConfig.<TypeSafeApiIntegration>builder()
                        .sayHello(TypeSafeApiIntegration.builder()
                                .integration(Integrations.lambda(
                                        // Point the lambda function to our built jar from the "lambdas" package
                                        new Function(s, "say-hello", FunctionProps.builder()
                                                .code(Code.fromAsset("../lambdas/dist/java/com/my/api/lambdas/1.0.0/lambdas-1.0.0.jar"))
                                                .handler("com.my.api.SayHelloHandler")
                                                .runtime(Runtime.JAVA_11)
                                                .timeout(Duration.seconds(30))
                                                .build())))
                                .build())
                        .build())
                .build());

        app.synth();
    }
}
```

#### Implement a Lambda Handler

In your `lambdas` project you can define your lambda handler in its source directory, eg `lambdas/src/main/java/com/my/api/SayHelloHandler.java`:

```java
package com.my.api;

import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHello200Response;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloRequestInput;
import com.generated.api.myapijavaruntime.runtime.api.Handlers.SayHelloResponse;
import com.generated.api.myapijavaruntime.runtime.model.SayHelloResponseContent;

/**
 * An example lambda handler which uses the generated handler wrapper class (Handlers.SayHello) to manage marshalling
 * inputs and outputs.
 */
public class SayHelloHandler extends SayHello {
    @Override
    public SayHelloResponse handle(SayHelloRequestInput sayHelloRequestInput) {
        return SayHello200Response.of(SayHelloResponseContent.builder()
                .message(String.format("Hello %s", sayHelloRequestInput.getInput().getRequestParameters().getName()))
                .build());
    }
}

```


### Other Details

#### Customising Generated Types/Infrastructure Projects

By default, the generated types and infrastructure projects are configured automatically, including their project names. You can customise the generated projects using the `runtime.options.<language>` or `infrastructure.options.<language>` properties when constructing the `TypeSafeApiProject`.

#### AWS WAFv2 Web ACL

By default, a [Web ACL](https://docs.aws.amazon.com/waf/latest/developerguide/web-acl.html) is deployed and attached to your API Gateway Rest API with the "[AWSManagedRulesCommonRuleSet](https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-baseline.html)", which provides protection against exploitation of a wide range of vulnerabilities, including some of the high risk and commonly occurring vulnerabilities described in OWASP publications such as [OWASP Top 10](https://owasp.org/www-project-top-ten/).

You can customise the Web ACL configuration via the `webAclOptions` of your `Api` CDK construct, eg:

```ts
export class SampleApi extends Api {
  constructor(scope: Construct, id: string) {
    super(scope, id, {
      integrations: { ... },
      webAclOptions: {
        // Allow access only to specific CIDR ranges
        cidrAllowList: {
          cidrType: 'IPV4',
          cidrRanges: ['1.2.3.4/5'],
        },
        // Pick from the set here: https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-list.html
        managedRules: [
          { vendor: 'AWS', name: 'AWSManagedRulesSQLiRuleSet' },
        ],
      },
    });
  }
}
```

You can remove the Web ACL entirely with `webAclOptions: { disable: true }` - you may wish to use this if you'd like to set up a Web ACL yourself with more control over the rules.

#### Smithy IntelliJ Plugin

The Smithy-based projects are compatible with the [Smithy IntelliJ Plugin](https://github.com/iancaffey/smithy-intellij-plugin), which provides syntax highlighting and auto-complete for your Smithy model. To make use of it, perform the following steps:

* Install the "Smithy" plugin (under `Preferences -> Plugins`)
* Right-click on the `smithy/build.gradle` file in your Smithy API project
* Select "Link Gradle Project"

#### Tagging Operations

Operations can be grouped together into logical collections via tags. This can be achieved in Smithy with the `@tags` trait:

```smithy
@tags(["pets", "users"])
operation PurchasePet {
  ...
}
```

Or in OpenAPI using the `tags` property:

```yaml
paths:
  /pets/purchase:
    post:
      operationId: purchasePet
      tags:
        - pets
        - users
      ...
```

When multiple tags are used, the "first" tag is considered to be the API that the operation belongs to, so in the generated client, the above example operation would be included in the `PetsApi` client but not the `UsersApi` client.

Multiple tags are still useful for documentation generation, for example `DocumentationFormat.HTML_REDOC` will group operations by tag in the side navigation bar.

If you would like to introduce tags without breaking existing clients, we recommend first adding a tag named `default` to all operations.

⚠️ __Important Note__: Smithy versions below `1.28.0` sort tags in alphabetical order and so the "first" tag will be the earliest in the alphabet. Therefore, if using tags with older versions of Smithy, we recommend prefixing your desired first tag with an underscore (for example `_default`). This is rectified in `1.28.0`, where tag order from the `@tags` trait is preserved.

#### Smithy Model Libraries and Dependencies

You can instantiate the TypeSafeApiModelProject on its own to create a standalone Smithy model library.

You can consume the library using the `addSmithyDeps` method, which adds a local file dependency on the built Smithy jar.

```ts
// Standalone model project, used as our model library
const shapes = new TypeSafeApiModelProject({
  name: "shapes",
  parent: monorepo,
  outdir: "packages/shapes",
  modelLanguage: ModelLanguage.SMITHY,
  modelOptions: {
    smithy: {
      serviceName: {
        namespace: "com.my.shared.shapes",
        serviceName: "Ignored",
      },
    },
  },
});

const api = new TypeSafeApiProject({ ... });

// Add the implicit monorepo dependency (if using the nx-monorepo) to ensure the shape library is built before the api model
monorepo.addImplicitDependency(api.model, shapes);

// Add a local file dependency on the built shapes jar
api.model.smithy!.addSmithyDeps(shapes.smithy!);
```

#### Local API Development Server

You can use the [AWS SAM CLI](https://docs.aws.amazon.com/serverless-application-model/latest/developerguide/install-sam-cli.html) to run a local development server for your API. You can achieve this using the following steps:

1. Synthesize your CDK stack containing your `Api` construct (this might be your `AwsCdkTypeScriptApp` project for example), with the context property `type-safe-api-local` set to `true`, for example:

```bash
cd packages/infra
npx cdk synth --context type-safe-api-local=true
```

2. Use the AWS SAM CLI to start the local development server, pointing it at the cloudformation template synthesized from the above command (note that the command will fail if docker is not running)

```bash
sam local start-api -t cdk.out/<your-stack>.template.json
```

You will need to repeat the above steps every time you make a code change for them to be reflected in your local development server.

See the [AWS SAM CLI Reference](https://docs.aws.amazon.com/serverless-application-model/latest/developerguide/sam-cli-command-reference-sam-local-start-api.html) for more information on the commands to run.

Make sure you do not deploy your CDK stack with `type-safe-api-local` set to `true`, since this uses an inline API definition which bloats the CloudFormation template and can exceed the maximum template size depending on the size of your API.

##### Limitations

Note that there is currently a limitation with SAM CLI where it does not support mock integrations, which means that the development server will not respond to OPTIONS requests even if you specified `corsOptions: { ... }` in your `Api` construct. This is being tracked as a [feature request here](https://github.com/aws/aws-sam-cli/issues/4973).

Note also that your API business logic may include operations which do not work locally, or may interact with real AWS resources depending on the AWS credentials you start your local development server with.

# API Reference <a name="API Reference" id="api-reference"></a>

## Constructs <a name="Constructs" id="Constructs"></a>

### TypeSafeRestApi <a name="TypeSafeRestApi" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi"></a>

A construct for creating an api gateway rest api based on the definition in the OpenAPI spec.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer"></a>

```typescript
import { TypeSafeRestApi } from '@aws-prototyping-sdk/type-safe-api'

new TypeSafeRestApi(scope: Construct, id: string, props: TypeSafeRestApiProps)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.scope">scope</a></code> | <code>constructs.Construct</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.id">id</a></code> | <code>string</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.props">props</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps">TypeSafeRestApiProps</a></code> | *No description.* |

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.scope"></a>

- *Type:* constructs.Construct

---

##### `id`<sup>Required</sup> <a name="id" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.id"></a>

- *Type:* string

---

##### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.Initializer.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps">TypeSafeRestApiProps</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.toString">toString</a></code> | Returns a string representation of this construct. |

---

##### `toString` <a name="toString" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.toString"></a>

```typescript
public toString(): string
```

Returns a string representation of this construct.

#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct">isConstruct</a></code> | Checks if `x` is a construct. |

---

##### `isConstruct` <a name="isConstruct" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct"></a>

```typescript
import { TypeSafeRestApi } from '@aws-prototyping-sdk/type-safe-api'

TypeSafeRestApi.isConstruct(x: any)
```

Checks if `x` is a construct.

Use this method instead of `instanceof` to properly detect `Construct`
instances, even when the construct library is symlinked.

Explanation: in JavaScript, multiple copies of the `constructs` library on
disk are seen as independent, completely different libraries. As a
consequence, the class `Construct` in each copy of the `constructs` library
is seen as a different class, and an instance of one class will not test as
`instanceof` the other class. `npm install` will not create installations
like this, but users may manually symlink construct libraries together or
use a monorepo tool: in those cases, multiple copies of the `constructs`
library can be accidentally installed, and `instanceof` will behave
unpredictably. It is safest to avoid using `instanceof`, and using
this type-testing method instead.

###### `x`<sup>Required</sup> <a name="x" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.isConstruct.parameter.x"></a>

- *Type:* any

Any object.

---

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.node">node</a></code> | <code>constructs.Node</code> | The tree node. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.api">api</a></code> | <code>aws-cdk-lib.aws_apigateway.SpecRestApi</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.ipSet">ipSet</a></code> | <code>aws-cdk-lib.aws_wafv2.CfnIPSet</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAcl">webAcl</a></code> | <code>aws-cdk-lib.aws_wafv2.CfnWebACL</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAclAssociation">webAclAssociation</a></code> | <code>aws-cdk-lib.aws_wafv2.CfnWebACLAssociation</code> | *No description.* |

---

##### `node`<sup>Required</sup> <a name="node" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.node"></a>

```typescript
public readonly node: Node;
```

- *Type:* constructs.Node

The tree node.

---

##### `api`<sup>Required</sup> <a name="api" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.api"></a>

```typescript
public readonly api: SpecRestApi;
```

- *Type:* aws-cdk-lib.aws_apigateway.SpecRestApi

---

##### `ipSet`<sup>Optional</sup> <a name="ipSet" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.ipSet"></a>

```typescript
public readonly ipSet: CfnIPSet;
```

- *Type:* aws-cdk-lib.aws_wafv2.CfnIPSet

---

##### `webAcl`<sup>Optional</sup> <a name="webAcl" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAcl"></a>

```typescript
public readonly webAcl: CfnWebACL;
```

- *Type:* aws-cdk-lib.aws_wafv2.CfnWebACL

---

##### `webAclAssociation`<sup>Optional</sup> <a name="webAclAssociation" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApi.property.webAclAssociation"></a>

```typescript
public readonly webAclAssociation: CfnWebACLAssociation;
```

- *Type:* aws-cdk-lib.aws_wafv2.CfnWebACLAssociation

---


## Structs <a name="Structs" id="Structs"></a>

### ApiGatewayIntegration <a name="ApiGatewayIntegration" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration"></a>

Represents an api gateway integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.Initializer"></a>

```typescript
import { ApiGatewayIntegration } from '@aws-prototyping-sdk/type-safe-api'

const apiGatewayIntegration: ApiGatewayIntegration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheKeyParameters">cacheKeyParameters</a></code> | <code>string[]</code> | A list of request parameters whose values are to be cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheNamespace">cacheNamespace</a></code> | <code>string</code> | An API-specific tag group of related cached parameters. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionId">connectionId</a></code> | <code>string</code> | The ID of a VpcLink for the private integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionType">connectionType</a></code> | <code>string</code> | The integration connection type. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.contentHandling">contentHandling</a></code> | <code>string</code> | Request payload encoding conversion types. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.credentials">credentials</a></code> | <code>string</code> | For AWS IAM role-based credentials, specify the ARN of an appropriate IAM role. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.httpMethod">httpMethod</a></code> | <code>string</code> | The HTTP method used in the integration request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.passthroughBehavior">passthroughBehavior</a></code> | <code>string</code> | Specifies how a request payload of unmapped content type is passed through the integration request without modification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestParameters">requestParameters</a></code> | <code>{[ key: string ]: string}</code> | Specifies mappings from method request parameters to integration request parameters. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestTemplates">requestTemplates</a></code> | <code>{[ key: string ]: string}</code> | Mapping templates for a request payload of specified MIME types. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.responses">responses</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse">ApiGatewayIntegrationResponse</a>}</code> | Defines the method's responses and specifies desired parameter mappings or payload mappings from integration responses to method responses. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.timeoutInMillis">timeoutInMillis</a></code> | <code>number</code> | Custom timeout between 50 and 29,000 milliseconds. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.tlsConfig">tlsConfig</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig">ApiGatewayIntegrationTlsConfig</a></code> | Specifies the TLS configuration for an integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.type">type</a></code> | <code>string</code> | The type of integration with the specified backend. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.uri">uri</a></code> | <code>string</code> | The endpoint URI of the backend. |

---

##### `cacheKeyParameters`<sup>Optional</sup> <a name="cacheKeyParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheKeyParameters"></a>

```typescript
public readonly cacheKeyParameters: string[];
```

- *Type:* string[]

A list of request parameters whose values are to be cached.

---

##### `cacheNamespace`<sup>Optional</sup> <a name="cacheNamespace" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.cacheNamespace"></a>

```typescript
public readonly cacheNamespace: string;
```

- *Type:* string

An API-specific tag group of related cached parameters.

---

##### `connectionId`<sup>Optional</sup> <a name="connectionId" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionId"></a>

```typescript
public readonly connectionId: string;
```

- *Type:* string

The ID of a VpcLink for the private integration.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_VpcLink.html](https://docs.aws.amazon.com/apigateway/latest/api/API_VpcLink.html)

---

##### `connectionType`<sup>Optional</sup> <a name="connectionType" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.connectionType"></a>

```typescript
public readonly connectionType: string;
```

- *Type:* string

The integration connection type.

The valid value is "VPC_LINK" for private integration or "INTERNET", otherwise.

---

##### `contentHandling`<sup>Optional</sup> <a name="contentHandling" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.contentHandling"></a>

```typescript
public readonly contentHandling: string;
```

- *Type:* string

Request payload encoding conversion types.

Valid values are 1) CONVERT_TO_TEXT, for converting a binary payload
into a base64-encoded string or converting a text payload into a utf-8-encoded string or passing through the text
payload natively without modification, and 2) CONVERT_TO_BINARY, for converting a text payload into a
base64-decoded blob or passing through a binary payload natively without modification.

---

##### `credentials`<sup>Optional</sup> <a name="credentials" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.credentials"></a>

```typescript
public readonly credentials: string;
```

- *Type:* string

For AWS IAM role-based credentials, specify the ARN of an appropriate IAM role.

If unspecified, credentials default
to resource-based permissions that must be added manually to allow the API to access the resource. For more
information, see Granting Permissions Using a Resource Policy.

Note: When using IAM credentials, make sure that AWS STS Regional endpoints are enabled for the Region where this
API is deployed for best performance.

---

##### `httpMethod`<sup>Optional</sup> <a name="httpMethod" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.httpMethod"></a>

```typescript
public readonly httpMethod: string;
```

- *Type:* string

The HTTP method used in the integration request.

For Lambda function invocations, the value must be POST.

---

##### `passthroughBehavior`<sup>Optional</sup> <a name="passthroughBehavior" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.passthroughBehavior"></a>

```typescript
public readonly passthroughBehavior: string;
```

- *Type:* string

Specifies how a request payload of unmapped content type is passed through the integration request without modification.

Supported values are when_no_templates, when_no_match, and never.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#passthroughBehavior](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#passthroughBehavior)

---

##### `requestParameters`<sup>Optional</sup> <a name="requestParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestParameters"></a>

```typescript
public readonly requestParameters: {[ key: string ]: string};
```

- *Type:* {[ key: string ]: string}

Specifies mappings from method request parameters to integration request parameters.

Supported request parameters
are querystring, path, header, and body.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestParameters.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestParameters.html)

---

##### `requestTemplates`<sup>Optional</sup> <a name="requestTemplates" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.requestTemplates"></a>

```typescript
public readonly requestTemplates: {[ key: string ]: string};
```

- *Type:* {[ key: string ]: string}

Mapping templates for a request payload of specified MIME types.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestTemplates.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-requestTemplates.html)

---

##### `responses`<sup>Optional</sup> <a name="responses" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.responses"></a>

```typescript
public readonly responses: {[ key: string ]: ApiGatewayIntegrationResponse};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse">ApiGatewayIntegrationResponse</a>}

Defines the method's responses and specifies desired parameter mappings or payload mappings from integration responses to method responses.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-responses.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-integration-responses.html)

---

##### `timeoutInMillis`<sup>Optional</sup> <a name="timeoutInMillis" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.timeoutInMillis"></a>

```typescript
public readonly timeoutInMillis: number;
```

- *Type:* number

Custom timeout between 50 and 29,000 milliseconds.

The default value is 29,000 milliseconds or 29 seconds.

---

##### `tlsConfig`<sup>Optional</sup> <a name="tlsConfig" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.tlsConfig"></a>

```typescript
public readonly tlsConfig: ApiGatewayIntegrationTlsConfig;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig">ApiGatewayIntegrationTlsConfig</a>

Specifies the TLS configuration for an integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html)

---

##### `type`<sup>Optional</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.type"></a>

```typescript
public readonly type: string;
```

- *Type:* string

The type of integration with the specified backend.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#type](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html#type)

---

##### `uri`<sup>Optional</sup> <a name="uri" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegration.property.uri"></a>

```typescript
public readonly uri: string;
```

- *Type:* string

The endpoint URI of the backend.

For integrations of the aws type, this is an ARN value.
For the HTTP integration, this is the URL of the HTTP endpoint including the https or http scheme.

---

### ApiGatewayIntegrationResponse <a name="ApiGatewayIntegrationResponse" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse"></a>

API Gateway integration response.

> [https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html](https://docs.aws.amazon.com/apigateway/latest/api/API_Integration.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.Initializer"></a>

```typescript
import { ApiGatewayIntegrationResponse } from '@aws-prototyping-sdk/type-safe-api'

const apiGatewayIntegrationResponse: ApiGatewayIntegrationResponse = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseParameters">responseParameters</a></code> | <code>{[ key: string ]: string}</code> | Specifies parameter mappings for the response. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseTemplates">responseTemplates</a></code> | <code>{[ key: string ]: string}</code> | Specifies MIME type-specific mapping templates for the response’s payload. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.statusCode">statusCode</a></code> | <code>string</code> | HTTP status code for the method response. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.contentHandling">contentHandling</a></code> | <code>string</code> | Response payload encoding conversion types. |

---

##### `responseParameters`<sup>Required</sup> <a name="responseParameters" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseParameters"></a>

```typescript
public readonly responseParameters: {[ key: string ]: string};
```

- *Type:* {[ key: string ]: string}

Specifies parameter mappings for the response.

---

##### `responseTemplates`<sup>Required</sup> <a name="responseTemplates" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.responseTemplates"></a>

```typescript
public readonly responseTemplates: {[ key: string ]: string};
```

- *Type:* {[ key: string ]: string}

Specifies MIME type-specific mapping templates for the response’s payload.

---

##### `statusCode`<sup>Required</sup> <a name="statusCode" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.statusCode"></a>

```typescript
public readonly statusCode: string;
```

- *Type:* string

HTTP status code for the method response.

---

##### `contentHandling`<sup>Optional</sup> <a name="contentHandling" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationResponse.property.contentHandling"></a>

```typescript
public readonly contentHandling: string;
```

- *Type:* string

Response payload encoding conversion types.

Valid values are 1) CONVERT_TO_TEXT, for converting a binary payload
into a base64-encoded string or converting a text payload into a utf-8-encoded string or passing through the text
payload natively without modification, and 2) CONVERT_TO_BINARY, for converting a text payload into a
base64-decoded blob or passing through a binary payload natively without modification.

---

### ApiGatewayIntegrationTlsConfig <a name="ApiGatewayIntegrationTlsConfig" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig"></a>

Specifies the TLS configuration for an integration.

> [https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html](https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-extensions-integration-tls-config.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.Initializer"></a>

```typescript
import { ApiGatewayIntegrationTlsConfig } from '@aws-prototyping-sdk/type-safe-api'

const apiGatewayIntegrationTlsConfig: ApiGatewayIntegrationTlsConfig = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.property.insecureSkipVerification">insecureSkipVerification</a></code> | <code>boolean</code> | Specifies whether or not API Gateway skips verification that the certificate for an integration endpoint is issued by a supported certificate authority. |

---

##### `insecureSkipVerification`<sup>Optional</sup> <a name="insecureSkipVerification" id="@aws-prototyping-sdk/type-safe-api.ApiGatewayIntegrationTlsConfig.property.insecureSkipVerification"></a>

```typescript
public readonly insecureSkipVerification: boolean;
```

- *Type:* boolean

Specifies whether or not API Gateway skips verification that the certificate for an integration endpoint is issued by a supported certificate authority.

This isn’t recommended, but it enables you to use certificates that are
signed by private certificate authorities, or certificates that are self-signed. If enabled, API Gateway still
performs basic certificate validation, which includes checking the certificate's expiration date, hostname, and
presence of a root certificate authority. Supported only for HTTP and HTTP_PROXY integrations.

---

### AuthorizerProps <a name="AuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps"></a>

Properties for an authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.Initializer"></a>

```typescript
import { AuthorizerProps } from '@aws-prototyping-sdk/type-safe-api'

const authorizerProps: AuthorizerProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.AuthorizerProps.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---

### CidrAllowList <a name="CidrAllowList" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList"></a>

Representation of a CIDR range.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.Initializer"></a>

```typescript
import { CidrAllowList } from '@aws-prototyping-sdk/type-safe-api'

const cidrAllowList: CidrAllowList = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrRanges">cidrRanges</a></code> | <code>string[]</code> | Specify an IPv4 address by using CIDR notation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrType">cidrType</a></code> | <code>string</code> | Type of CIDR range. |

---

##### `cidrRanges`<sup>Required</sup> <a name="cidrRanges" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrRanges"></a>

```typescript
public readonly cidrRanges: string[];
```

- *Type:* string[]

Specify an IPv4 address by using CIDR notation.

For example:
To configure AWS WAF to allow, block, or count requests that originated from the IP address 192.0.2.44, specify 192.0.2.44/32 .
To configure AWS WAF to allow, block, or count requests that originated from IP addresses from 192.0.2.0 to 192.0.2.255, specify 192.0.2.0/24 .

For more information about CIDR notation, see the Wikipedia entry Classless Inter-Domain Routing .

Specify an IPv6 address by using CIDR notation. For example:
To configure AWS WAF to allow, block, or count requests that originated from the IP address 1111:0000:0000:0000:0000:0000:0000:0111, specify 1111:0000:0000:0000:0000:0000:0000:0111/128 .
To configure AWS WAF to allow, block, or count requests that originated from IP addresses 1111:0000:0000:0000:0000:0000:0000:0000 to 1111:0000:0000:0000:ffff:ffff:ffff:ffff, specify 1111:0000:0000:0000:0000:0000:0000:0000/64 .

---

##### `cidrType`<sup>Required</sup> <a name="cidrType" id="@aws-prototyping-sdk/type-safe-api.CidrAllowList.property.cidrType"></a>

```typescript
public readonly cidrType: string;
```

- *Type:* string

Type of CIDR range.

---

### CognitoAuthorizerProps <a name="CognitoAuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps"></a>

Properties used to configure a cognito authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.Initializer"></a>

```typescript
import { CognitoAuthorizerProps } from '@aws-prototyping-sdk/type-safe-api'

const cognitoAuthorizerProps: CognitoAuthorizerProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>string</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.userPools">userPools</a></code> | <code>aws-cdk-lib.aws_cognito.IUserPool[]</code> | The Cognito user pools associated with this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | A list of authorization scopes configured on the method. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

Unique identifier for this authorizer.

---

##### `userPools`<sup>Required</sup> <a name="userPools" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.userPools"></a>

```typescript
public readonly userPools: IUserPool[];
```

- *Type:* aws-cdk-lib.aws_cognito.IUserPool[]

The Cognito user pools associated with this authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]
- *Default:* []

A list of authorization scopes configured on the method.

When used as the default authorizer, these scopes will be
applied to all methods without an authorizer at the integration level.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes)

---

### CustomAuthorizerProps <a name="CustomAuthorizerProps" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps"></a>

Properties used to configure a custom authorizer.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.Initializer"></a>

```typescript
import { CustomAuthorizerProps } from '@aws-prototyping-sdk/type-safe-api'

const customAuthorizerProps: CustomAuthorizerProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerId">authorizerId</a></code> | <code>string</code> | Unique identifier for this authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.function">function</a></code> | <code>aws-cdk-lib.aws_lambda.IFunction</code> | The lambda function used to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerResultTtlInSeconds">authorizerResultTtlInSeconds</a></code> | <code>number</code> | The number of seconds during which the authorizer result is cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.identitySource">identitySource</a></code> | <code>string</code> | The source of the identity in an incoming request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.type">type</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a></code> | The type of custom authorizer. |

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

Unique identifier for this authorizer.

---

##### `function`<sup>Required</sup> <a name="function" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.function"></a>

```typescript
public readonly function: IFunction;
```

- *Type:* aws-cdk-lib.aws_lambda.IFunction

The lambda function used to authorize requests.

---

##### `authorizerResultTtlInSeconds`<sup>Optional</sup> <a name="authorizerResultTtlInSeconds" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.authorizerResultTtlInSeconds"></a>

```typescript
public readonly authorizerResultTtlInSeconds: number;
```

- *Type:* number
- *Default:* 300

The number of seconds during which the authorizer result is cached.

---

##### `identitySource`<sup>Optional</sup> <a name="identitySource" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.identitySource"></a>

```typescript
public readonly identitySource: string;
```

- *Type:* string
- *Default:* "method.request.header.Authorization"

The source of the identity in an incoming request.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource)

---

##### `type`<sup>Optional</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps.property.type"></a>

```typescript
public readonly type: CustomAuthorizerType;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a>
- *Default:* CustomAuthorizerType.TOKEN

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

---

### DocumentationConfiguration <a name="DocumentationConfiguration" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration"></a>

Configuration for generated documentation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.Initializer"></a>

```typescript
import { DocumentationConfiguration } from '@aws-prototyping-sdk/type-safe-api'

const documentationConfiguration: DocumentationConfiguration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.property.formats">formats</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat">DocumentationFormat</a>[]</code> | Formats for generated documentation. |

---

##### `formats`<sup>Required</sup> <a name="formats" id="@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration.property.formats"></a>

```typescript
public readonly formats: DocumentationFormat[];
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat">DocumentationFormat</a>[]

Formats for generated documentation.

---

### GeneratedCodeOptions <a name="GeneratedCodeOptions" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions"></a>

Options for generated clients.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.Initializer"></a>

```typescript
import { GeneratedCodeOptions } from '@aws-prototyping-sdk/type-safe-api'

const generatedCodeOptions: GeneratedCodeOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.java">java</a></code> | <code>projen.java.JavaProjectOptions</code> | Options for a generated java project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.python">python</a></code> | <code>projen.python.PythonProjectOptions</code> | Options for a generated python project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.typescript">typescript</a></code> | <code>projen.typescript.TypeScriptProjectOptions</code> | Options for a generated typescript project. |

---

##### `java`<sup>Optional</sup> <a name="java" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.java"></a>

```typescript
public readonly java: JavaProjectOptions;
```

- *Type:* projen.java.JavaProjectOptions

Options for a generated java project.

These override the default inferred options.

---

##### `python`<sup>Optional</sup> <a name="python" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.python"></a>

```typescript
public readonly python: PythonProjectOptions;
```

- *Type:* projen.python.PythonProjectOptions

Options for a generated python project.

These override the default inferred options.

---

##### `typescript`<sup>Optional</sup> <a name="typescript" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions.property.typescript"></a>

```typescript
public readonly typescript: TypeScriptProjectOptions;
```

- *Type:* projen.typescript.TypeScriptProjectOptions

Options for a generated typescript project.

These override the default inferred options.

---

### GeneratedCodeProjects <a name="GeneratedCodeProjects" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects"></a>

Generated code projects.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.Initializer"></a>

```typescript
import { GeneratedCodeProjects } from '@aws-prototyping-sdk/type-safe-api'

const generatedCodeProjects: GeneratedCodeProjects = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.java">java</a></code> | <code>projen.java.JavaProject</code> | Generated java project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.python">python</a></code> | <code>projen.python.PythonProject</code> | Generated python project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.typescript">typescript</a></code> | <code>projen.typescript.TypeScriptProject</code> | Generated typescript project. |

---

##### `java`<sup>Optional</sup> <a name="java" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.java"></a>

```typescript
public readonly java: JavaProject;
```

- *Type:* projen.java.JavaProject

Generated java project.

---

##### `python`<sup>Optional</sup> <a name="python" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.python"></a>

```typescript
public readonly python: PythonProject;
```

- *Type:* projen.python.PythonProject

Generated python project.

---

##### `typescript`<sup>Optional</sup> <a name="typescript" id="@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects.property.typescript"></a>

```typescript
public readonly typescript: TypeScriptProject;
```

- *Type:* projen.typescript.TypeScriptProject

Generated typescript project.

---

### InfrastructureConfiguration <a name="InfrastructureConfiguration" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration"></a>

Configuration for generated infrastructure.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.Initializer"></a>

```typescript
import { InfrastructureConfiguration } from '@aws-prototyping-sdk/type-safe-api'

const infrastructureConfiguration: InfrastructureConfiguration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.language">language</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a></code> | The language to generate the type-safe CDK infrastructure in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a></code> | Options for the infrastructure package. |

---

##### `language`<sup>Required</sup> <a name="language" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.language"></a>

```typescript
public readonly language: Language;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>

The language to generate the type-safe CDK infrastructure in.

---

##### `options`<sup>Optional</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration.property.options"></a>

```typescript
public readonly options: GeneratedCodeOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a>

Options for the infrastructure package.

Note that only those provided for the specified language will apply.

---

### IntegrationGrantProps <a name="IntegrationGrantProps" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps"></a>

Properties for granting the API access to invoke the operation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.Initializer"></a>

```typescript
import { IntegrationGrantProps } from '@aws-prototyping-sdk/type-safe-api'

const integrationGrantProps: IntegrationGrantProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.method">method</a></code> | <code>string</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.path">path</a></code> | <code>string</code> | The path of this operation in the api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.api">api</a></code> | <code>aws-cdk-lib.aws_apigateway.SpecRestApi</code> | The api to grant permissions for. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.operationId">operationId</a></code> | <code>string</code> | The ID of the operation for which permissions are being granted. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.scope">scope</a></code> | <code>constructs.Construct</code> | The scope in which permission resources can be created. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.method"></a>

```typescript
public readonly method: string;
```

- *Type:* string

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.path"></a>

```typescript
public readonly path: string;
```

- *Type:* string

The path of this operation in the api.

---

##### `api`<sup>Required</sup> <a name="api" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.api"></a>

```typescript
public readonly api: SpecRestApi;
```

- *Type:* aws-cdk-lib.aws_apigateway.SpecRestApi

The api to grant permissions for.

---

##### `operationId`<sup>Required</sup> <a name="operationId" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.operationId"></a>

```typescript
public readonly operationId: string;
```

- *Type:* string

The ID of the operation for which permissions are being granted.

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps.property.scope"></a>

```typescript
public readonly scope: Construct;
```

- *Type:* constructs.Construct

The scope in which permission resources can be created.

---

### IntegrationRenderProps <a name="IntegrationRenderProps" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps"></a>

Properties for rendering an integration into an API Gateway OpenAPI extension.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.Initializer"></a>

```typescript
import { IntegrationRenderProps } from '@aws-prototyping-sdk/type-safe-api'

const integrationRenderProps: IntegrationRenderProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.method">method</a></code> | <code>string</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.path">path</a></code> | <code>string</code> | The path of this operation in the api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.operationId">operationId</a></code> | <code>string</code> | The ID of the operation being rendered. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.scope">scope</a></code> | <code>constructs.Construct</code> | The scope in which the integration is being rendered. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.method"></a>

```typescript
public readonly method: string;
```

- *Type:* string

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.path"></a>

```typescript
public readonly path: string;
```

- *Type:* string

The path of this operation in the api.

---

##### `operationId`<sup>Required</sup> <a name="operationId" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.operationId"></a>

```typescript
public readonly operationId: string;
```

- *Type:* string

The ID of the operation being rendered.

---

##### `scope`<sup>Required</sup> <a name="scope" id="@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps.property.scope"></a>

```typescript
public readonly scope: Construct;
```

- *Type:* constructs.Construct

The scope in which the integration is being rendered.

---

### ManagedRule <a name="ManagedRule" id="@aws-prototyping-sdk/type-safe-api.ManagedRule"></a>

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.Initializer"></a>

```typescript
import { ManagedRule } from '@aws-prototyping-sdk/type-safe-api'

const managedRule: ManagedRule = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule.property.name">name</a></code> | <code>string</code> | The name of the managed rule group. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule.property.vendor">vendor</a></code> | <code>string</code> | The name of the managed rule group vendor. |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string

The name of the managed rule group.

You use this, along with the vendor name, to identify the rule group.

---

##### `vendor`<sup>Required</sup> <a name="vendor" id="@aws-prototyping-sdk/type-safe-api.ManagedRule.property.vendor"></a>

```typescript
public readonly vendor: string;
```

- *Type:* string

The name of the managed rule group vendor.

You use this, along with the rule group name, to identify the rule group.

---

### MethodAndPath <a name="MethodAndPath" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath"></a>

Structure to contain an API operation's method and path.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.Initializer"></a>

```typescript
import { MethodAndPath } from '@aws-prototyping-sdk/type-safe-api'

const methodAndPath: MethodAndPath = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.method">method</a></code> | <code>string</code> | The http method of this operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.path">path</a></code> | <code>string</code> | The path of this operation in the api. |

---

##### `method`<sup>Required</sup> <a name="method" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.method"></a>

```typescript
public readonly method: string;
```

- *Type:* string

The http method of this operation.

---

##### `path`<sup>Required</sup> <a name="path" id="@aws-prototyping-sdk/type-safe-api.MethodAndPath.property.path"></a>

```typescript
public readonly path: string;
```

- *Type:* string

The path of this operation in the api.

---

### ModelConfiguration <a name="ModelConfiguration" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration"></a>

Configuration for modelling the API.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.Initializer"></a>

```typescript
import { ModelConfiguration } from '@aws-prototyping-sdk/type-safe-api'

const modelConfiguration: ModelConfiguration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.language">language</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a></code> | The language the API model is defined in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a></code> | Options for the API model. |

---

##### `language`<sup>Required</sup> <a name="language" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.language"></a>

```typescript
public readonly language: ModelLanguage;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a>

The language the API model is defined in.

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.ModelConfiguration.property.options"></a>

```typescript
public readonly options: ModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a>

Options for the API model.

---

### ModelOptions <a name="ModelOptions" id="@aws-prototyping-sdk/type-safe-api.ModelOptions"></a>

Options for models.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.Initializer"></a>

```typescript
import { ModelOptions } from '@aws-prototyping-sdk/type-safe-api'

const modelOptions: ModelOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions.property.openapi">openapi</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a></code> | Options for the OpenAPI model - required when model language is OPENAPI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions.property.smithy">smithy</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a></code> | Options for the Smithy model - required when model language is SMITHY. |

---

##### `openapi`<sup>Optional</sup> <a name="openapi" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.property.openapi"></a>

```typescript
public readonly openapi: OpenApiModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a>

Options for the OpenAPI model - required when model language is OPENAPI.

---

##### `smithy`<sup>Optional</sup> <a name="smithy" id="@aws-prototyping-sdk/type-safe-api.ModelOptions.property.smithy"></a>

```typescript
public readonly smithy: SmithyModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a>

Options for the Smithy model - required when model language is SMITHY.

---

### OpenApiDefinitionOptions <a name="OpenApiDefinitionOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions"></a>

Options for the OpenAPI Spec.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.Initializer"></a>

```typescript
import { OpenApiDefinitionOptions } from '@aws-prototyping-sdk/type-safe-api'

const openApiDefinitionOptions: OpenApiDefinitionOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.property.openApiOptions">openApiOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a></code> | Options for the openapi model. |

---

##### `openApiOptions`<sup>Required</sup> <a name="openApiOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions.property.openApiOptions"></a>

```typescript
public readonly openApiOptions: OpenApiModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions">OpenApiModelOptions</a>

Options for the openapi model.

---

### OpenApiModelOptions <a name="OpenApiModelOptions" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions"></a>

Options for the OpenAPI model.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.Initializer"></a>

```typescript
import { OpenApiModelOptions } from '@aws-prototyping-sdk/type-safe-api'

const openApiModelOptions: OpenApiModelOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.property.title">title</a></code> | <code>string</code> | The title in the OpenAPI specification. |

---

##### `title`<sup>Required</sup> <a name="title" id="@aws-prototyping-sdk/type-safe-api.OpenApiModelOptions.property.title"></a>

```typescript
public readonly title: string;
```

- *Type:* string

The title in the OpenAPI specification.

---

### RuntimeConfiguration <a name="RuntimeConfiguration" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration"></a>

Configuration for generated runtime projects.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.Initializer"></a>

```typescript
import { RuntimeConfiguration } from '@aws-prototyping-sdk/type-safe-api'

const runtimeConfiguration: RuntimeConfiguration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.languages">languages</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>[]</code> | The languages that runtime projects will be generated in. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a></code> | Options for the generated runtimes. |

---

##### `languages`<sup>Required</sup> <a name="languages" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.languages"></a>

```typescript
public readonly languages: Language[];
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Language">Language</a>[]

The languages that runtime projects will be generated in.

These projects can be used to provide type safety for
both client and server projects.

---

##### `options`<sup>Optional</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration.property.options"></a>

```typescript
public readonly options: GeneratedCodeOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeOptions">GeneratedCodeOptions</a>

Options for the generated runtimes.

Note that only options provided for the specified languages will apply.

---

### SmithyBuildOptions <a name="SmithyBuildOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions"></a>

Options for the smithy build files.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.Initializer"></a>

```typescript
import { SmithyBuildOptions } from '@aws-prototyping-sdk/type-safe-api'

const smithyBuildOptions: SmithyBuildOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.imports">imports</a></code> | <code>string[]</code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.plugins">plugins</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}</code> | Plugins keyed by plugin id. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.ignoreMissingPlugins">ignoreMissingPlugins</a></code> | <code>boolean</code> | If a plugin can't be found, Smithy will by default fail the build. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.maven">maven</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration">SmithyMavenConfiguration</a></code> | Maven configuration for the Smithy build project, used to specify dependencies and repositories in the build.gradle and smithy-build.json files. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.projections">projections</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection">SmithyProjection</a>}</code> | Map of projections name to projection configurations https://awslabs.github.io/smithy/2.0/guides/building-models/build-config.html#projections. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.imports"></a>

```typescript
public readonly imports: string[];
```

- *Type:* string[]

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.plugins"></a>

```typescript
public readonly plugins: {[ key: string ]: SmithyPlugin};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}

Plugins keyed by plugin id.

---

##### `ignoreMissingPlugins`<sup>Optional</sup> <a name="ignoreMissingPlugins" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.ignoreMissingPlugins"></a>

```typescript
public readonly ignoreMissingPlugins: boolean;
```

- *Type:* boolean
- *Default:* no ignoreMissingPlugins set in the smithy-build.json file

If a plugin can't be found, Smithy will by default fail the build.

This setting can be set to true to allow the build to progress
even if a plugin can't be found on the classpath.

---

##### `maven`<sup>Optional</sup> <a name="maven" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.maven"></a>

```typescript
public readonly maven: SmithyMavenConfiguration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration">SmithyMavenConfiguration</a>
- *Default:* the default configuration required for Smithy to OpenAPI conversion

Maven configuration for the Smithy build project, used to specify dependencies and repositories in the build.gradle and smithy-build.json files.

---

##### `projections`<sup>Optional</sup> <a name="projections" id="@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions.property.projections"></a>

```typescript
public readonly projections: {[ key: string ]: SmithyProjection};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection">SmithyProjection</a>}
- *Default:* no projections

Map of projections name to projection configurations https://awslabs.github.io/smithy/2.0/guides/building-models/build-config.html#projections.

---

### SmithyCommon <a name="SmithyCommon" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon"></a>

Properties common to smithy plugins and the root smithy build.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.Initializer"></a>

```typescript
import { SmithyCommon } from '@aws-prototyping-sdk/type-safe-api'

const smithyCommon: SmithyCommon = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.imports">imports</a></code> | <code>string[]</code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.plugins">plugins</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}</code> | Plugins keyed by plugin id. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.imports"></a>

```typescript
public readonly imports: string[];
```

- *Type:* string[]

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyCommon.property.plugins"></a>

```typescript
public readonly plugins: {[ key: string ]: SmithyPlugin};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}

Plugins keyed by plugin id.

---

### SmithyDefinitionOptions <a name="SmithyDefinitionOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions"></a>

Options for a smithy build project.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.Initializer"></a>

```typescript
import { SmithyDefinitionOptions } from '@aws-prototyping-sdk/type-safe-api'

const smithyDefinitionOptions: SmithyDefinitionOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.property.smithyOptions">smithyOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a></code> | Smithy engine options. |

---

##### `smithyOptions`<sup>Required</sup> <a name="smithyOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions.property.smithyOptions"></a>

```typescript
public readonly smithyOptions: SmithyModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions">SmithyModelOptions</a>

Smithy engine options.

---

### SmithyMavenConfiguration <a name="SmithyMavenConfiguration" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration"></a>

Configuration for smithy maven dependencies.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.Initializer"></a>

```typescript
import { SmithyMavenConfiguration } from '@aws-prototyping-sdk/type-safe-api'

const smithyMavenConfiguration: SmithyMavenConfiguration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.dependencies">dependencies</a></code> | <code>string[]</code> | The dependencies used in the build.gradle and smithy-build.json files eg. software.amazon.smithy:smithy-validation-model:1.27.2 The following required dependencies are always added: - software.amazon.smithy:smithy-cli:1.27.2 - software.amazon.smithy:smithy-model:1.27.2 - software.amazon.smithy:smithy-openapi:1.27.2 - software.amazon.smithy:smithy-aws-traits:1.27.2 You can however override the version of these dependencies if required. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.repositoryUrls">repositoryUrls</a></code> | <code>string[]</code> | The repository urls used in the build.gradle and smithy-build.json files. |

---

##### `dependencies`<sup>Optional</sup> <a name="dependencies" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.dependencies"></a>

```typescript
public readonly dependencies: string[];
```

- *Type:* string[]

The dependencies used in the build.gradle and smithy-build.json files eg. software.amazon.smithy:smithy-validation-model:1.27.2 The following required dependencies are always added: - software.amazon.smithy:smithy-cli:1.27.2 - software.amazon.smithy:smithy-model:1.27.2 - software.amazon.smithy:smithy-openapi:1.27.2 - software.amazon.smithy:smithy-aws-traits:1.27.2 You can however override the version of these dependencies if required.

---

##### `repositoryUrls`<sup>Optional</sup> <a name="repositoryUrls" id="@aws-prototyping-sdk/type-safe-api.SmithyMavenConfiguration.property.repositoryUrls"></a>

```typescript
public readonly repositoryUrls: string[];
```

- *Type:* string[]
- *Default:* maven central and maven local

The repository urls used in the build.gradle and smithy-build.json files.

---

### SmithyModelOptions <a name="SmithyModelOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions"></a>

Options for a Smithy model.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.Initializer"></a>

```typescript
import { SmithyModelOptions } from '@aws-prototyping-sdk/type-safe-api'

const smithyModelOptions: SmithyModelOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.serviceName">serviceName</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName">SmithyServiceName</a></code> | Smithy service name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreGradleWrapper">ignoreGradleWrapper</a></code> | <code>boolean</code> | Set to false if you would like to check in your gradle wrapper. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreSmithyBuildOutput">ignoreSmithyBuildOutput</a></code> | <code>boolean</code> | Set to false if you would like to check in your smithy build output or have more fine-grained control over what is checked in, eg if you add other projections to the smithy-build.json file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.smithyBuildOptions">smithyBuildOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions">SmithyBuildOptions</a></code> | Smithy build options. |

---

##### `serviceName`<sup>Required</sup> <a name="serviceName" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.serviceName"></a>

```typescript
public readonly serviceName: SmithyServiceName;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName">SmithyServiceName</a>

Smithy service name.

---

##### `ignoreGradleWrapper`<sup>Optional</sup> <a name="ignoreGradleWrapper" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreGradleWrapper"></a>

```typescript
public readonly ignoreGradleWrapper: boolean;
```

- *Type:* boolean
- *Default:* true

Set to false if you would like to check in your gradle wrapper.

Do so if you would like to use a different version
of gradle to the one provided by default

---

##### `ignoreSmithyBuildOutput`<sup>Optional</sup> <a name="ignoreSmithyBuildOutput" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.ignoreSmithyBuildOutput"></a>

```typescript
public readonly ignoreSmithyBuildOutput: boolean;
```

- *Type:* boolean
- *Default:* true

Set to false if you would like to check in your smithy build output or have more fine-grained control over what is checked in, eg if you add other projections to the smithy-build.json file.

---

##### `smithyBuildOptions`<sup>Optional</sup> <a name="smithyBuildOptions" id="@aws-prototyping-sdk/type-safe-api.SmithyModelOptions.property.smithyBuildOptions"></a>

```typescript
public readonly smithyBuildOptions: SmithyBuildOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyBuildOptions">SmithyBuildOptions</a>

Smithy build options.

---

### SmithyPlugin <a name="SmithyPlugin" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin"></a>

A smithy plugin.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin.Initializer"></a>

```typescript
import { SmithyPlugin } from '@aws-prototyping-sdk/type-safe-api'

const smithyPlugin: SmithyPlugin = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin.property.service">service</a></code> | <code>string</code> | The service to which the plugin should apply. |

---

##### `service`<sup>Optional</sup> <a name="service" id="@aws-prototyping-sdk/type-safe-api.SmithyPlugin.property.service"></a>

```typescript
public readonly service: string;
```

- *Type:* string

The service to which the plugin should apply.

---

### SmithyProjection <a name="SmithyProjection" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection"></a>

A smithy build projection.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.Initializer"></a>

```typescript
import { SmithyProjection } from '@aws-prototyping-sdk/type-safe-api'

const smithyProjection: SmithyProjection = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.imports">imports</a></code> | <code>string[]</code> | List of imports. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.plugins">plugins</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}</code> | Plugins keyed by plugin id. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.abstract">abstract</a></code> | <code>boolean</code> | Whether or not the projection is abstract. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.transforms">transforms</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform">SmithyTransform</a>[]</code> | Transforms to apply to the projection. |

---

##### `imports`<sup>Optional</sup> <a name="imports" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.imports"></a>

```typescript
public readonly imports: string[];
```

- *Type:* string[]

List of imports.

---

##### `plugins`<sup>Optional</sup> <a name="plugins" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.plugins"></a>

```typescript
public readonly plugins: {[ key: string ]: SmithyPlugin};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.SmithyPlugin">SmithyPlugin</a>}

Plugins keyed by plugin id.

---

##### `abstract`<sup>Optional</sup> <a name="abstract" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.abstract"></a>

```typescript
public readonly abstract: boolean;
```

- *Type:* boolean

Whether or not the projection is abstract.

---

##### `transforms`<sup>Optional</sup> <a name="transforms" id="@aws-prototyping-sdk/type-safe-api.SmithyProjection.property.transforms"></a>

```typescript
public readonly transforms: SmithyTransform[];
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform">SmithyTransform</a>[]

Transforms to apply to the projection.

---

### SmithyServiceName <a name="SmithyServiceName" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName"></a>

Represents a fully qualified name of a Smithy service.

> [https://awslabs.github.io/smithy/2.0/spec/service-types.html](https://awslabs.github.io/smithy/2.0/spec/service-types.html)

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.Initializer"></a>

```typescript
import { SmithyServiceName } from '@aws-prototyping-sdk/type-safe-api'

const smithyServiceName: SmithyServiceName = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.namespace">namespace</a></code> | <code>string</code> | The service namespace. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.serviceName">serviceName</a></code> | <code>string</code> | The service name. |

---

##### `namespace`<sup>Required</sup> <a name="namespace" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.namespace"></a>

```typescript
public readonly namespace: string;
```

- *Type:* string

The service namespace.

Nested namespaces are separated by '.', for example com.company

> [https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id](https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id)

---

##### `serviceName`<sup>Required</sup> <a name="serviceName" id="@aws-prototyping-sdk/type-safe-api.SmithyServiceName.property.serviceName"></a>

```typescript
public readonly serviceName: string;
```

- *Type:* string

The service name.

Should be PascalCase, for example HelloService

> [https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id](https://awslabs.github.io/smithy/2.0/spec/model.html#shape-id)

---

### SmithyTransform <a name="SmithyTransform" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform"></a>

A smithy build transform.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.Initializer"></a>

```typescript
import { SmithyTransform } from '@aws-prototyping-sdk/type-safe-api'

const smithyTransform: SmithyTransform = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.args">args</a></code> | <code>{[ key: string ]: any}</code> | Arguments for the transform. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.name">name</a></code> | <code>string</code> | Name of the transform. |

---

##### `args`<sup>Required</sup> <a name="args" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.args"></a>

```typescript
public readonly args: {[ key: string ]: any};
```

- *Type:* {[ key: string ]: any}

Arguments for the transform.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.SmithyTransform.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string

Name of the transform.

---

### TypeSafeApiIntegration <a name="TypeSafeApiIntegration" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration"></a>

Defines an integration for an individual API operation.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.Initializer"></a>

```typescript
import { TypeSafeApiIntegration } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeApiIntegration: TypeSafeApiIntegration = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.integration">integration</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration">Integration</a></code> | The lambda function to service the api operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.authorizer">authorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The authorizer to use for this api operation (overrides the default). |

---

##### `integration`<sup>Required</sup> <a name="integration" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.integration"></a>

```typescript
public readonly integration: Integration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Integration">Integration</a>

The lambda function to service the api operation.

---

##### `authorizer`<sup>Optional</sup> <a name="authorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration.property.authorizer"></a>

```typescript
public readonly authorizer: Authorizer;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The authorizer to use for this api operation (overrides the default).

---

### TypeSafeApiModelProjectOptions <a name="TypeSafeApiModelProjectOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions"></a>

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.Initializer"></a>

```typescript
import { TypeSafeApiModelProjectOptions } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeApiModelProjectOptions: TypeSafeApiModelProjectOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.name">name</a></code> | <code>string</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.commitGenerated">commitGenerated</a></code> | <code>boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitOptions">gitOptions</a></code> | <code>projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.logging">logging</a></code> | <code>projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.outdir">outdir</a></code> | <code>string</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.parent">parent</a></code> | <code>projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenCommand">projenCommand</a></code> | <code>string</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJson">projenrcJson</a></code> | <code>boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebot">renovatebot</a></code> | <code>boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebotOptions">renovatebotOptions</a></code> | <code>projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelLanguage">modelLanguage</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelOptions">modelOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a></code> | *No description.* |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.commitGenerated"></a>

```typescript
public readonly commitGenerated: boolean;
```

- *Type:* boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitIgnoreOptions"></a>

```typescript
public readonly gitIgnoreOptions: IgnoreFileOptions;
```

- *Type:* projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.gitOptions"></a>

```typescript
public readonly gitOptions: GitOptions;
```

- *Type:* projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.logging"></a>

```typescript
public readonly logging: LoggerOptions;
```

- *Type:* projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.outdir"></a>

```typescript
public readonly outdir: string;
```

- *Type:* string
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.parent"></a>

```typescript
public readonly parent: Project;
```

- *Type:* projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenCommand"></a>

```typescript
public readonly projenCommand: string;
```

- *Type:* string
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJson"></a>

```typescript
public readonly projenrcJson: boolean;
```

- *Type:* boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.projenrcJsonOptions"></a>

```typescript
public readonly projenrcJsonOptions: ProjenrcJsonOptions;
```

- *Type:* projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebot"></a>

```typescript
public readonly renovatebot: boolean;
```

- *Type:* boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.renovatebotOptions"></a>

```typescript
public readonly renovatebotOptions: RenovatebotOptions;
```

- *Type:* projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `modelLanguage`<sup>Required</sup> <a name="modelLanguage" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelLanguage"></a>

```typescript
public readonly modelLanguage: ModelLanguage;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage">ModelLanguage</a>

---

##### `modelOptions`<sup>Required</sup> <a name="modelOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions.property.modelOptions"></a>

```typescript
public readonly modelOptions: ModelOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelOptions">ModelOptions</a>

---

### TypeSafeApiOptions <a name="TypeSafeApiOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions"></a>

Options required alongside an Open API specification to create API Gateway resources.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.Initializer"></a>

```typescript
import { TypeSafeApiOptions } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeApiOptions: TypeSafeApiOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.integrations">integrations</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>}</code> | A mapping of API operation to its integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.operationLookup">operationLookup</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>}</code> | Details about each operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.corsOptions">corsOptions</a></code> | <code>aws-cdk-lib.aws_apigateway.CorsOptions</code> | Cross Origin Resource Sharing options for the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.defaultAuthorizer">defaultAuthorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The default authorizer to use for your api. |

---

##### `integrations`<sup>Required</sup> <a name="integrations" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.integrations"></a>

```typescript
public readonly integrations: {[ key: string ]: TypeSafeApiIntegration};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>}

A mapping of API operation to its integration.

---

##### `operationLookup`<sup>Required</sup> <a name="operationLookup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.operationLookup"></a>

```typescript
public readonly operationLookup: {[ key: string ]: MethodAndPath};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>}

Details about each operation.

---

##### `corsOptions`<sup>Optional</sup> <a name="corsOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.corsOptions"></a>

```typescript
public readonly corsOptions: CorsOptions;
```

- *Type:* aws-cdk-lib.aws_apigateway.CorsOptions

Cross Origin Resource Sharing options for the API.

---

##### `defaultAuthorizer`<sup>Optional</sup> <a name="defaultAuthorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiOptions.property.defaultAuthorizer"></a>

```typescript
public readonly defaultAuthorizer: Authorizer;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The default authorizer to use for your api.

When omitted, no default authorizer is used.
Authorizers specified at the integration level will override this for that operation.

---

### TypeSafeApiProjectOptions <a name="TypeSafeApiProjectOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions"></a>

Options for the TypeSafeApiProject.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.Initializer"></a>

```typescript
import { TypeSafeApiProjectOptions } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeApiProjectOptions: TypeSafeApiProjectOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.name">name</a></code> | <code>string</code> | This is the name of your project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.commitGenerated">commitGenerated</a></code> | <code>boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitIgnoreOptions">gitIgnoreOptions</a></code> | <code>projen.IgnoreFileOptions</code> | Configuration options for .gitignore file. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitOptions">gitOptions</a></code> | <code>projen.GitOptions</code> | Configuration options for git. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.logging">logging</a></code> | <code>projen.LoggerOptions</code> | Configure logging options such as verbosity. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.outdir">outdir</a></code> | <code>string</code> | The root directory of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.parent">parent</a></code> | <code>projen.Project</code> | The parent project, if this project is part of a bigger project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenCommand">projenCommand</a></code> | <code>string</code> | The shell command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJson">projenrcJson</a></code> | <code>boolean</code> | Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJsonOptions">projenrcJsonOptions</a></code> | <code>projen.ProjenrcJsonOptions</code> | Options for .projenrc.json. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebot">renovatebot</a></code> | <code>boolean</code> | Use renovatebot to handle dependency upgrades. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebotOptions">renovatebotOptions</a></code> | <code>projen.RenovatebotOptions</code> | Options for renovatebot. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.infrastructure">infrastructure</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a></code> | Configuration for generated infrastructure. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.model">model</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a></code> | Configuration for the API model. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.runtime">runtime</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a></code> | Configuration for generated runtime projects (containing types, clients and server code). |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.documentation">documentation</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a></code> | Configuration for generated documentation. |

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string
- *Default:* $BASEDIR

This is the name of your project.

---

##### `commitGenerated`<sup>Optional</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.commitGenerated"></a>

```typescript
public readonly commitGenerated: boolean;
```

- *Type:* boolean
- *Default:* true

Whether to commit the managed files by default.

---

##### `gitIgnoreOptions`<sup>Optional</sup> <a name="gitIgnoreOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitIgnoreOptions"></a>

```typescript
public readonly gitIgnoreOptions: IgnoreFileOptions;
```

- *Type:* projen.IgnoreFileOptions

Configuration options for .gitignore file.

---

##### `gitOptions`<sup>Optional</sup> <a name="gitOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.gitOptions"></a>

```typescript
public readonly gitOptions: GitOptions;
```

- *Type:* projen.GitOptions

Configuration options for git.

---

##### `logging`<sup>Optional</sup> <a name="logging" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.logging"></a>

```typescript
public readonly logging: LoggerOptions;
```

- *Type:* projen.LoggerOptions
- *Default:* {}

Configure logging options such as verbosity.

---

##### `outdir`<sup>Optional</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.outdir"></a>

```typescript
public readonly outdir: string;
```

- *Type:* string
- *Default:* "."

The root directory of the project.

Relative to this directory, all files are synthesized.

If this project has a parent, this directory is relative to the parent
directory and it cannot be the same as the parent or any of it's other
sub-projects.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.parent"></a>

```typescript
public readonly parent: Project;
```

- *Type:* projen.Project

The parent project, if this project is part of a bigger project.

---

##### `projenCommand`<sup>Optional</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenCommand"></a>

```typescript
public readonly projenCommand: string;
```

- *Type:* string
- *Default:* "npx projen"

The shell command to use in order to run the projen CLI.

Can be used to customize in special environments.

---

##### `projenrcJson`<sup>Optional</sup> <a name="projenrcJson" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJson"></a>

```typescript
public readonly projenrcJson: boolean;
```

- *Type:* boolean
- *Default:* false

Generate (once) .projenrc.json (in JSON). Set to `false` in order to disable .projenrc.json generation.

---

##### `projenrcJsonOptions`<sup>Optional</sup> <a name="projenrcJsonOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.projenrcJsonOptions"></a>

```typescript
public readonly projenrcJsonOptions: ProjenrcJsonOptions;
```

- *Type:* projen.ProjenrcJsonOptions
- *Default:* default options

Options for .projenrc.json.

---

##### `renovatebot`<sup>Optional</sup> <a name="renovatebot" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebot"></a>

```typescript
public readonly renovatebot: boolean;
```

- *Type:* boolean
- *Default:* false

Use renovatebot to handle dependency upgrades.

---

##### `renovatebotOptions`<sup>Optional</sup> <a name="renovatebotOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.renovatebotOptions"></a>

```typescript
public readonly renovatebotOptions: RenovatebotOptions;
```

- *Type:* projen.RenovatebotOptions
- *Default:* default options

Options for renovatebot.

---

##### `infrastructure`<sup>Required</sup> <a name="infrastructure" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.infrastructure"></a>

```typescript
public readonly infrastructure: InfrastructureConfiguration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.InfrastructureConfiguration">InfrastructureConfiguration</a>

Configuration for generated infrastructure.

---

##### `model`<sup>Required</sup> <a name="model" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.model"></a>

```typescript
public readonly model: ModelConfiguration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ModelConfiguration">ModelConfiguration</a>

Configuration for the API model.

---

##### `runtime`<sup>Required</sup> <a name="runtime" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.runtime"></a>

```typescript
public readonly runtime: RuntimeConfiguration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.RuntimeConfiguration">RuntimeConfiguration</a>

Configuration for generated runtime projects (containing types, clients and server code).

---

##### `documentation`<sup>Optional</sup> <a name="documentation" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions.property.documentation"></a>

```typescript
public readonly documentation: DocumentationConfiguration;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.DocumentationConfiguration">DocumentationConfiguration</a>

Configuration for generated documentation.

---

### TypeSafeApiWebAclOptions <a name="TypeSafeApiWebAclOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions"></a>

Configuration for the Web ACL associated with the API.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.Initializer"></a>

```typescript
import { TypeSafeApiWebAclOptions } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeApiWebAclOptions: TypeSafeApiWebAclOptions = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.cidrAllowList">cidrAllowList</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList">CidrAllowList</a></code> | List of cidr ranges to allow. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.disable">disable</a></code> | <code>boolean</code> | If set to true, no WebACL will be associated with the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.managedRules">managedRules</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule">ManagedRule</a>[]</code> | List of managed rules to apply to the web acl. |

---

##### `cidrAllowList`<sup>Optional</sup> <a name="cidrAllowList" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.cidrAllowList"></a>

```typescript
public readonly cidrAllowList: CidrAllowList;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CidrAllowList">CidrAllowList</a>
- *Default:* undefined

List of cidr ranges to allow.

---

##### `disable`<sup>Optional</sup> <a name="disable" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.disable"></a>

```typescript
public readonly disable: boolean;
```

- *Type:* boolean
- *Default:* false

If set to true, no WebACL will be associated with the API.

You can also use this option if you would like to create
your own WebACL and associate it yourself.

---

##### `managedRules`<sup>Optional</sup> <a name="managedRules" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions.property.managedRules"></a>

```typescript
public readonly managedRules: ManagedRule[];
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.ManagedRule">ManagedRule</a>[]
- *Default:* [{ vendor: "AWS", name: "AWSManagedRulesCommonRuleSet" }]

List of managed rules to apply to the web acl.

---

### TypeSafeRestApiProps <a name="TypeSafeRestApiProps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps"></a>

Configuration for the TypeSafeRestApi construct.

#### Initializer <a name="Initializer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.Initializer"></a>

```typescript
import { TypeSafeRestApiProps } from '@aws-prototyping-sdk/type-safe-api'

const typeSafeRestApiProps: TypeSafeRestApiProps = { ... }
```

#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.cloudWatchRole">cloudWatchRole</a></code> | <code>boolean</code> | Automatically configure an AWS CloudWatch role for API Gateway. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deploy">deploy</a></code> | <code>boolean</code> | Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deployOptions">deployOptions</a></code> | <code>aws-cdk-lib.aws_apigateway.StageOptions</code> | Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.description">description</a></code> | <code>string</code> | A description of the RestApi construct. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.disableExecuteApiEndpoint">disableExecuteApiEndpoint</a></code> | <code>boolean</code> | Specifies whether clients can invoke the API using the default execute-api endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.domainName">domainName</a></code> | <code>aws-cdk-lib.aws_apigateway.DomainNameOptions</code> | Configure a custom domain name and map it to this API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointExportName">endpointExportName</a></code> | <code>string</code> | Export name for the CfnOutput containing the API endpoint. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointTypes">endpointTypes</a></code> | <code>aws-cdk-lib.aws_apigateway.EndpointType[]</code> | A list of the endpoint types of the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.failOnWarnings">failOnWarnings</a></code> | <code>boolean</code> | Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.parameters">parameters</a></code> | <code>{[ key: string ]: string}</code> | Custom header parameters for the request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.policy">policy</a></code> | <code>aws-cdk-lib.aws_iam.PolicyDocument</code> | A policy document that contains the permissions for this RestApi. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.restApiName">restApiName</a></code> | <code>string</code> | A name for the API Gateway RestApi resource. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.retainDeployments">retainDeployments</a></code> | <code>boolean</code> | Retains old deployment resources when the API changes. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.integrations">integrations</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>}</code> | A mapping of API operation to its integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.operationLookup">operationLookup</a></code> | <code>{[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>}</code> | Details about each operation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.corsOptions">corsOptions</a></code> | <code>aws-cdk-lib.aws_apigateway.CorsOptions</code> | Cross Origin Resource Sharing options for the API. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.defaultAuthorizer">defaultAuthorizer</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a></code> | The default authorizer to use for your api. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.specPath">specPath</a></code> | <code>string</code> | Path to the JSON open api spec. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.webAclOptions">webAclOptions</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a></code> | Options for the AWS WAF v2 WebACL associated with the api. |

---

##### `cloudWatchRole`<sup>Optional</sup> <a name="cloudWatchRole" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.cloudWatchRole"></a>

```typescript
public readonly cloudWatchRole: boolean;
```

- *Type:* boolean
- *Default:* false if `@aws-cdk/aws-apigateway:disableCloudWatchRole` is enabled, true otherwise

Automatically configure an AWS CloudWatch role for API Gateway.

---

##### `deploy`<sup>Optional</sup> <a name="deploy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deploy"></a>

```typescript
public readonly deploy: boolean;
```

- *Type:* boolean
- *Default:* true

Indicates if a Deployment should be automatically created for this API, and recreated when the API model (resources, methods) changes.

Since API Gateway deployments are immutable, When this option is enabled
(by default), an AWS::ApiGateway::Deployment resource will automatically
created with a logical ID that hashes the API model (methods, resources
and options). This means that when the model changes, the logical ID of
this CloudFormation resource will change, and a new deployment will be
created.

If this is set, `latestDeployment` will refer to the `Deployment` object
and `deploymentStage` will refer to a `Stage` that points to this
deployment. To customize the stage options, use the `deployOptions`
property.

A CloudFormation Output will also be defined with the root URL endpoint
of this REST API.

---

##### `deployOptions`<sup>Optional</sup> <a name="deployOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.deployOptions"></a>

```typescript
public readonly deployOptions: StageOptions;
```

- *Type:* aws-cdk-lib.aws_apigateway.StageOptions
- *Default:* Based on defaults of `StageOptions`.

Options for the API Gateway stage that will always point to the latest deployment when `deploy` is enabled.

If `deploy` is disabled,
this value cannot be set.

---

##### `description`<sup>Optional</sup> <a name="description" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.description"></a>

```typescript
public readonly description: string;
```

- *Type:* string
- *Default:* 'Automatically created by the RestApi construct'

A description of the RestApi construct.

---

##### `disableExecuteApiEndpoint`<sup>Optional</sup> <a name="disableExecuteApiEndpoint" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.disableExecuteApiEndpoint"></a>

```typescript
public readonly disableExecuteApiEndpoint: boolean;
```

- *Type:* boolean
- *Default:* false

Specifies whether clients can invoke the API using the default execute-api endpoint.

To require that clients use a custom domain name to invoke the
API, disable the default endpoint.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html)

---

##### `domainName`<sup>Optional</sup> <a name="domainName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.domainName"></a>

```typescript
public readonly domainName: DomainNameOptions;
```

- *Type:* aws-cdk-lib.aws_apigateway.DomainNameOptions
- *Default:* no domain name is defined, use `addDomainName` or directly define a `DomainName`.

Configure a custom domain name and map it to this API.

---

##### `endpointExportName`<sup>Optional</sup> <a name="endpointExportName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointExportName"></a>

```typescript
public readonly endpointExportName: string;
```

- *Type:* string
- *Default:* when no export name is given, output will be created without export

Export name for the CfnOutput containing the API endpoint.

---

##### `endpointTypes`<sup>Optional</sup> <a name="endpointTypes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.endpointTypes"></a>

```typescript
public readonly endpointTypes: EndpointType[];
```

- *Type:* aws-cdk-lib.aws_apigateway.EndpointType[]
- *Default:* EndpointType.EDGE

A list of the endpoint types of the API.

Use this property when creating
an API.

---

##### `failOnWarnings`<sup>Optional</sup> <a name="failOnWarnings" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.failOnWarnings"></a>

```typescript
public readonly failOnWarnings: boolean;
```

- *Type:* boolean
- *Default:* false

Indicates whether to roll back the resource if a warning occurs while API Gateway is creating the RestApi resource.

---

##### `parameters`<sup>Optional</sup> <a name="parameters" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.parameters"></a>

```typescript
public readonly parameters: {[ key: string ]: string};
```

- *Type:* {[ key: string ]: string}
- *Default:* No parameters.

Custom header parameters for the request.

> [https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html](https://docs.aws.amazon.com/cli/latest/reference/apigateway/import-rest-api.html)

---

##### `policy`<sup>Optional</sup> <a name="policy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.policy"></a>

```typescript
public readonly policy: PolicyDocument;
```

- *Type:* aws-cdk-lib.aws_iam.PolicyDocument
- *Default:* No policy.

A policy document that contains the permissions for this RestApi.

---

##### `restApiName`<sup>Optional</sup> <a name="restApiName" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.restApiName"></a>

```typescript
public readonly restApiName: string;
```

- *Type:* string
- *Default:* ID of the RestApi construct.

A name for the API Gateway RestApi resource.

---

##### `retainDeployments`<sup>Optional</sup> <a name="retainDeployments" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.retainDeployments"></a>

```typescript
public readonly retainDeployments: boolean;
```

- *Type:* boolean
- *Default:* false

Retains old deployment resources when the API changes.

This allows
manually reverting stages to point to old deployments via the AWS
Console.

---

##### `integrations`<sup>Required</sup> <a name="integrations" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.integrations"></a>

```typescript
public readonly integrations: {[ key: string ]: TypeSafeApiIntegration};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiIntegration">TypeSafeApiIntegration</a>}

A mapping of API operation to its integration.

---

##### `operationLookup`<sup>Required</sup> <a name="operationLookup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.operationLookup"></a>

```typescript
public readonly operationLookup: {[ key: string ]: MethodAndPath};
```

- *Type:* {[ key: string ]: <a href="#@aws-prototyping-sdk/type-safe-api.MethodAndPath">MethodAndPath</a>}

Details about each operation.

---

##### `corsOptions`<sup>Optional</sup> <a name="corsOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.corsOptions"></a>

```typescript
public readonly corsOptions: CorsOptions;
```

- *Type:* aws-cdk-lib.aws_apigateway.CorsOptions

Cross Origin Resource Sharing options for the API.

---

##### `defaultAuthorizer`<sup>Optional</sup> <a name="defaultAuthorizer" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.defaultAuthorizer"></a>

```typescript
public readonly defaultAuthorizer: Authorizer;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.Authorizer">Authorizer</a>

The default authorizer to use for your api.

When omitted, no default authorizer is used.
Authorizers specified at the integration level will override this for that operation.

---

##### `specPath`<sup>Required</sup> <a name="specPath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.specPath"></a>

```typescript
public readonly specPath: string;
```

- *Type:* string

Path to the JSON open api spec.

---

##### `webAclOptions`<sup>Optional</sup> <a name="webAclOptions" id="@aws-prototyping-sdk/type-safe-api.TypeSafeRestApiProps.property.webAclOptions"></a>

```typescript
public readonly webAclOptions: TypeSafeApiWebAclOptions;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiWebAclOptions">TypeSafeApiWebAclOptions</a>

Options for the AWS WAF v2 WebACL associated with the api.

By default, a Web ACL with the AWS default managed
rule set will be associated with the API. These options may disable or override the defaults.

---

## Classes <a name="Classes" id="Classes"></a>

### Authorizer <a name="Authorizer" id="@aws-prototyping-sdk/type-safe-api.Authorizer"></a>

An authorizer for authorizing API requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer"></a>

```typescript
import { Authorizer } from '@aws-prototyping-sdk/type-safe-api'

new Authorizer(props: AuthorizerProps)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.props">props</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps">AuthorizerProps</a></code> | *No description.* |

---

##### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Authorizer.Initializer.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.AuthorizerProps">AuthorizerProps</a>

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.Authorizer.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---


### Authorizers <a name="Authorizers" id="@aws-prototyping-sdk/type-safe-api.Authorizers"></a>

Class used to construct authorizers for use in the OpenApiGatewayLambdaApi construct.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Authorizers.Initializer"></a>

```typescript
import { Authorizers } from '@aws-prototyping-sdk/type-safe-api'

new Authorizers()
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---


#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.cognito">cognito</a></code> | A Cognito User Pools authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.custom">custom</a></code> | A custom authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.iam">iam</a></code> | An IAM authorizer which uses AWS signature version 4 to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Authorizers.none">none</a></code> | No authorizer. |

---

##### `cognito` <a name="cognito" id="@aws-prototyping-sdk/type-safe-api.Authorizers.cognito"></a>

```typescript
import { Authorizers } from '@aws-prototyping-sdk/type-safe-api'

Authorizers.cognito(props: CognitoAuthorizerProps)
```

A Cognito User Pools authorizer.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Authorizers.cognito.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps">CognitoAuthorizerProps</a>

---

##### `custom` <a name="custom" id="@aws-prototyping-sdk/type-safe-api.Authorizers.custom"></a>

```typescript
import { Authorizers } from '@aws-prototyping-sdk/type-safe-api'

Authorizers.custom(props: CustomAuthorizerProps)
```

A custom authorizer.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Authorizers.custom.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps">CustomAuthorizerProps</a>

---

##### `iam` <a name="iam" id="@aws-prototyping-sdk/type-safe-api.Authorizers.iam"></a>

```typescript
import { Authorizers } from '@aws-prototyping-sdk/type-safe-api'

Authorizers.iam()
```

An IAM authorizer which uses AWS signature version 4 to authorize requests.

##### `none` <a name="none" id="@aws-prototyping-sdk/type-safe-api.Authorizers.none"></a>

```typescript
import { Authorizers } from '@aws-prototyping-sdk/type-safe-api'

Authorizers.none()
```

No authorizer.



### CognitoAuthorizer <a name="CognitoAuthorizer" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer"></a>

An authorizer that uses Cognito identity or access tokens.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer"></a>

```typescript
import { CognitoAuthorizer } from '@aws-prototyping-sdk/type-safe-api'

new CognitoAuthorizer(props: CognitoAuthorizerProps)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.props">props</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps">CognitoAuthorizerProps</a></code> | *No description.* |

---

##### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.Initializer.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizerProps">CognitoAuthorizerProps</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes">withScopes</a></code> | Returns this authorizer with scopes applied, intended for usage in individual operations where scopes may differ on a per-operation basis. |

---

##### `withScopes` <a name="withScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes"></a>

```typescript
public withScopes(authorizationScopes: string): CognitoAuthorizer
```

Returns this authorizer with scopes applied, intended for usage in individual operations where scopes may differ on a per-operation basis.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationscopes)

###### `authorizationScopes`<sup>Required</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.withScopes.parameter.authorizationScopes"></a>

- *Type:* string

the scopes to apply.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.userPools">userPools</a></code> | <code>aws-cdk-lib.aws_cognito.IUserPool[]</code> | The Cognito user pools associated with this authorizer. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---

##### `userPools`<sup>Required</sup> <a name="userPools" id="@aws-prototyping-sdk/type-safe-api.CognitoAuthorizer.property.userPools"></a>

```typescript
public readonly userPools: IUserPool[];
```

- *Type:* aws-cdk-lib.aws_cognito.IUserPool[]

The Cognito user pools associated with this authorizer.

---


### CustomAuthorizer <a name="CustomAuthorizer" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer"></a>

An authorizer that uses a lambda function to authorize requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer"></a>

```typescript
import { CustomAuthorizer } from '@aws-prototyping-sdk/type-safe-api'

new CustomAuthorizer(props: CustomAuthorizerProps)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.props">props</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps">CustomAuthorizerProps</a></code> | *No description.* |

---

##### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.Initializer.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerProps">CustomAuthorizerProps</a>

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerResultTtlInSeconds">authorizerResultTtlInSeconds</a></code> | <code>number</code> | The number of seconds during which the authorizer result is cached. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.function">function</a></code> | <code>aws-cdk-lib.aws_lambda.IFunction</code> | The lambda function used to authorize requests. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.identitySource">identitySource</a></code> | <code>string</code> | The source of the identity in an incoming request. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.type">type</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a></code> | The type of custom authorizer. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---

##### `authorizerResultTtlInSeconds`<sup>Required</sup> <a name="authorizerResultTtlInSeconds" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.authorizerResultTtlInSeconds"></a>

```typescript
public readonly authorizerResultTtlInSeconds: number;
```

- *Type:* number

The number of seconds during which the authorizer result is cached.

---

##### `function`<sup>Required</sup> <a name="function" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.function"></a>

```typescript
public readonly function: IFunction;
```

- *Type:* aws-cdk-lib.aws_lambda.IFunction

The lambda function used to authorize requests.

---

##### `identitySource`<sup>Required</sup> <a name="identitySource" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.identitySource"></a>

```typescript
public readonly identitySource: string;
```

- *Type:* string

The source of the identity in an incoming request.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource)

---

##### `type`<sup>Required</sup> <a name="type" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizer.property.type"></a>

```typescript
public readonly type: CustomAuthorizerType;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType">CustomAuthorizerType</a>

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

---


### IamAuthorizer <a name="IamAuthorizer" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer"></a>

An IAM authorizer.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.Initializer"></a>

```typescript
import { IamAuthorizer } from '@aws-prototyping-sdk/type-safe-api'

new IamAuthorizer()
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.IamAuthorizer.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---


### Integration <a name="Integration" id="@aws-prototyping-sdk/type-safe-api.Integration"></a>

An integration for an API operation.

You can extend this to implement your own integration if you like.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Integration.Initializer"></a>

```typescript
import { Integration } from '@aws-prototyping-sdk/type-safe-api'

new Integration()
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration.grant">grant</a></code> | Grant permissions for the API to invoke the integration. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integration.render">render</a></code> | Render the integration into an API Gateway OpenAPI extension. |

---

##### `grant` <a name="grant" id="@aws-prototyping-sdk/type-safe-api.Integration.grant"></a>

```typescript
public grant(_props: IntegrationGrantProps): void
```

Grant permissions for the API to invoke the integration.

###### `_props`<sup>Required</sup> <a name="_props" id="@aws-prototyping-sdk/type-safe-api.Integration.grant.parameter._props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps">IntegrationGrantProps</a>

---

##### `render` <a name="render" id="@aws-prototyping-sdk/type-safe-api.Integration.render"></a>

```typescript
public render(props: IntegrationRenderProps): ApiGatewayIntegration
```

Render the integration into an API Gateway OpenAPI extension.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.Integration.render.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps">IntegrationRenderProps</a>

---




### Integrations <a name="Integrations" id="@aws-prototyping-sdk/type-safe-api.Integrations"></a>

A collection of integrations to connect API operations with a backend to service requests.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.Integrations.Initializer"></a>

```typescript
import { Integrations } from '@aws-prototyping-sdk/type-safe-api'

new Integrations()
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---


#### Static Functions <a name="Static Functions" id="Static Functions"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Integrations.lambda">lambda</a></code> | An integration that invokes a lambda function to service the request. |

---

##### `lambda` <a name="lambda" id="@aws-prototyping-sdk/type-safe-api.Integrations.lambda"></a>

```typescript
import { Integrations } from '@aws-prototyping-sdk/type-safe-api'

Integrations.lambda(lambdaFunction: IFunction)
```

An integration that invokes a lambda function to service the request.

###### `lambdaFunction`<sup>Required</sup> <a name="lambdaFunction" id="@aws-prototyping-sdk/type-safe-api.Integrations.lambda.parameter.lambdaFunction"></a>

- *Type:* aws-cdk-lib.aws_lambda.IFunction

the function to invoke.

---



### LambdaIntegration <a name="LambdaIntegration" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration"></a>

A lambda integration.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer"></a>

```typescript
import { LambdaIntegration } from '@aws-prototyping-sdk/type-safe-api'

new LambdaIntegration(lambdaFunction: IFunction)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer.parameter.lambdaFunction">lambdaFunction</a></code> | <code>aws-cdk-lib.aws_lambda.IFunction</code> | *No description.* |

---

##### `lambdaFunction`<sup>Required</sup> <a name="lambdaFunction" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.Initializer.parameter.lambdaFunction"></a>

- *Type:* aws-cdk-lib.aws_lambda.IFunction

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant">grant</a></code> | Grant API Gateway permissions to invoke the lambda. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render">render</a></code> | Render the lambda integration as a snippet of OpenAPI. |

---

##### `grant` <a name="grant" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant"></a>

```typescript
public grant(__0: IntegrationGrantProps): void
```

Grant API Gateway permissions to invoke the lambda.

###### `__0`<sup>Required</sup> <a name="__0" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.grant.parameter.__0"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationGrantProps">IntegrationGrantProps</a>

---

##### `render` <a name="render" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render"></a>

```typescript
public render(props: IntegrationRenderProps): ApiGatewayIntegration
```

Render the lambda integration as a snippet of OpenAPI.

###### `props`<sup>Required</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.LambdaIntegration.render.parameter.props"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.IntegrationRenderProps">IntegrationRenderProps</a>

---




### NoneAuthorizer <a name="NoneAuthorizer" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer"></a>

No authorizer.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.Initializer"></a>

```typescript
import { NoneAuthorizer } from '@aws-prototyping-sdk/type-safe-api'

new NoneAuthorizer()
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |

---



#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationType">authorizationType</a></code> | <code>aws-cdk-lib.aws_apigateway.AuthorizationType</code> | The type of the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizerId">authorizerId</a></code> | <code>string</code> | The unique identifier for the authorizer. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationScopes">authorizationScopes</a></code> | <code>string[]</code> | Scopes for the authorizer, if any. |

---

##### `authorizationType`<sup>Required</sup> <a name="authorizationType" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationType"></a>

```typescript
public readonly authorizationType: AuthorizationType;
```

- *Type:* aws-cdk-lib.aws_apigateway.AuthorizationType

The type of the authorizer.

---

##### `authorizerId`<sup>Required</sup> <a name="authorizerId" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizerId"></a>

```typescript
public readonly authorizerId: string;
```

- *Type:* string

The unique identifier for the authorizer.

---

##### `authorizationScopes`<sup>Optional</sup> <a name="authorizationScopes" id="@aws-prototyping-sdk/type-safe-api.NoneAuthorizer.property.authorizationScopes"></a>

```typescript
public readonly authorizationScopes: string[];
```

- *Type:* string[]

Scopes for the authorizer, if any.

---


### OpenApiDefinition <a name="OpenApiDefinition" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition"></a>

The OpenAPI Spec.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer"></a>

```typescript
import { OpenApiDefinition } from '@aws-prototyping-sdk/type-safe-api'

new OpenApiDefinition(project: TypeSafeApiModelProject, options: OpenApiDefinitionOptions)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.project">project</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions">OpenApiDefinitionOptions</a></code> | *No description.* |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.project"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.Initializer.parameter.options"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinitionOptions">OpenApiDefinitionOptions</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.postSynthesize">postSynthesize</a></code> | Called after synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.preSynthesize">preSynthesize</a></code> | Called before synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.synthesize">synthesize</a></code> | Synthesizes files to the project output directory. |

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.postSynthesize"></a>

```typescript
public postSynthesize(): void
```

Called after synthesis.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.preSynthesize"></a>

```typescript
public preSynthesize(): void
```

Called before synthesis.

##### `synthesize` <a name="synthesize" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.synthesize"></a>

```typescript
public synthesize(): void
```

Synthesizes files to the project output directory.


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.project">project</a></code> | <code>projen.Project</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.openApiSpecificationPath">openApiSpecificationPath</a></code> | <code>string</code> | Path to the root OpenAPI specification file. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.project"></a>

```typescript
public readonly project: Project;
```

- *Type:* projen.Project

---

##### `openApiSpecificationPath`<sup>Required</sup> <a name="openApiSpecificationPath" id="@aws-prototyping-sdk/type-safe-api.OpenApiDefinition.property.openApiSpecificationPath"></a>

```typescript
public readonly openApiSpecificationPath: string;
```

- *Type:* string

Path to the root OpenAPI specification file.

---


### SmithyDefinition <a name="SmithyDefinition" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition"></a>

Creates a project which transforms a Smithy model to OpenAPI.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer"></a>

```typescript
import { SmithyDefinition } from '@aws-prototyping-sdk/type-safe-api'

new SmithyDefinition(project: TypeSafeApiModelProject, options: SmithyDefinitionOptions)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.project">project</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions">SmithyDefinitionOptions</a></code> | *No description.* |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.project"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.Initializer.parameter.options"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinitionOptions">SmithyDefinitionOptions</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.postSynthesize">postSynthesize</a></code> | Called after synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.preSynthesize">preSynthesize</a></code> | Called before synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.synthesize">synthesize</a></code> | Synthesizes files to the project output directory. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps">addDeps</a></code> | Add maven-style or local file dependencies to the smithy model project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps">addSmithyDeps</a></code> | Add dependencies on other smithy models, such that their shapes can be imported in this project. |

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.postSynthesize"></a>

```typescript
public postSynthesize(): void
```

Called after synthesis.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.preSynthesize"></a>

```typescript
public preSynthesize(): void
```

Called before synthesis.

##### `synthesize` <a name="synthesize" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.synthesize"></a>

```typescript
public synthesize(): void
```

Synthesizes files to the project output directory.

##### `addDeps` <a name="addDeps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps"></a>

```typescript
public addDeps(deps: string): void
```

Add maven-style or local file dependencies to the smithy model project.

###### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addDeps.parameter.deps"></a>

- *Type:* string

dependencies to add, eg "software.amazon.smithy:smithy-validation-model:1.27.2" or "file://../some/path/build/lib/my-shapes.jar.

---

##### `addSmithyDeps` <a name="addSmithyDeps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps"></a>

```typescript
public addSmithyDeps(deps: SmithyDefinition): void
```

Add dependencies on other smithy models, such that their shapes can be imported in this project.

###### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.addSmithyDeps.parameter.deps"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a>

smithy definitions to depend on.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.project">project</a></code> | <code>projen.Project</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.gradleProjectName">gradleProjectName</a></code> | <code>string</code> | Name of the gradle project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.openApiSpecificationPath">openApiSpecificationPath</a></code> | <code>string</code> | Path to the generated OpenAPI specification, relative to the project outdir. |

---

##### `project`<sup>Required</sup> <a name="project" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.project"></a>

```typescript
public readonly project: Project;
```

- *Type:* projen.Project

---

##### `gradleProjectName`<sup>Required</sup> <a name="gradleProjectName" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.gradleProjectName"></a>

```typescript
public readonly gradleProjectName: string;
```

- *Type:* string

Name of the gradle project.

---

##### `openApiSpecificationPath`<sup>Required</sup> <a name="openApiSpecificationPath" id="@aws-prototyping-sdk/type-safe-api.SmithyDefinition.property.openApiSpecificationPath"></a>

```typescript
public readonly openApiSpecificationPath: string;
```

- *Type:* string

Path to the generated OpenAPI specification, relative to the project outdir.

---


### TypeSafeApiModelProject <a name="TypeSafeApiModelProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject"></a>

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer"></a>

```typescript
import { TypeSafeApiModelProject } from '@aws-prototyping-sdk/type-safe-api'

new TypeSafeApiModelProject(options: TypeSafeApiModelProjectOptions)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions">TypeSafeApiModelProjectOptions</a></code> | *No description.* |

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.Initializer.parameter.options"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProjectOptions">TypeSafeApiModelProjectOptions</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup">addExcludeFromCleanup</a></code> | Exclude the matching files from pre-synth cleanup. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore">addGitIgnore</a></code> | Adds a .gitignore pattern. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore">addPackageIgnore</a></code> | Exclude these files from the bundled package. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask">addTask</a></code> | Adds a new task to this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip">addTip</a></code> | Prints a "tip" message during synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated">annotateGenerated</a></code> | Consider a set of files as "generated". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.postSynthesize">postSynthesize</a></code> | Called after all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.preSynthesize">preSynthesize</a></code> | Called before all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask">removeTask</a></code> | Removes a task from a project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand">runTaskCommand</a></code> | Returns the shell command to execute in order to run a task. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.synth">synth</a></code> | Synthesize all project files into `outdir`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile">tryFindFile</a></code> | Finds a file at the specified relative path within this project and all its subprojects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile">tryFindJsonFile</a></code> | Finds a json file by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile">tryFindObjectFile</a></code> | Finds an object file (like JsonFile, YamlFile, etc.) by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile">tryRemoveFile</a></code> | Finds a file at the specified relative path within this project and removes it. |

---

##### `addExcludeFromCleanup` <a name="addExcludeFromCleanup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup"></a>

```typescript
public addExcludeFromCleanup(globs: string): void
```

Exclude the matching files from pre-synth cleanup.

Can be used when, for example, some
source files include the projen marker and we don't want them to be erased during synth.

###### `globs`<sup>Required</sup> <a name="globs" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addExcludeFromCleanup.parameter.globs"></a>

- *Type:* string

The glob patterns to match.

---

##### `addGitIgnore` <a name="addGitIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore"></a>

```typescript
public addGitIgnore(pattern: string): void
```

Adds a .gitignore pattern.

###### `pattern`<sup>Required</sup> <a name="pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addGitIgnore.parameter.pattern"></a>

- *Type:* string

The glob pattern to ignore.

---

##### `addPackageIgnore` <a name="addPackageIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore"></a>

```typescript
public addPackageIgnore(_pattern: string): void
```

Exclude these files from the bundled package.

Implemented by project types based on the
packaging mechanism. For example, `NodeProject` delegates this to `.npmignore`.

###### `_pattern`<sup>Required</sup> <a name="_pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addPackageIgnore.parameter._pattern"></a>

- *Type:* string

The glob pattern to exclude.

---

##### `addTask` <a name="addTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask"></a>

```typescript
public addTask(name: string, props?: TaskOptions): Task
```

Adds a new task to this project.

This will fail if the project already has
a task with this name.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask.parameter.name"></a>

- *Type:* string

The task name to add.

---

###### `props`<sup>Optional</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTask.parameter.props"></a>

- *Type:* projen.TaskOptions

Task properties.

---

##### ~~`addTip`~~ <a name="addTip" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip"></a>

```typescript
public addTip(message: string): void
```

Prints a "tip" message during synthesis.

###### `message`<sup>Required</sup> <a name="message" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.addTip.parameter.message"></a>

- *Type:* string

The message.

---

##### `annotateGenerated` <a name="annotateGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated"></a>

```typescript
public annotateGenerated(_glob: string): void
```

Consider a set of files as "generated".

This method is implemented by
derived classes and used for example, to add git attributes to tell GitHub
that certain files are generated.

###### `_glob`<sup>Required</sup> <a name="_glob" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.annotateGenerated.parameter._glob"></a>

- *Type:* string

the glob pattern to match (could be a file path).

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.postSynthesize"></a>

```typescript
public postSynthesize(): void
```

Called after all components are synthesized.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.preSynthesize"></a>

```typescript
public preSynthesize(): void
```

Called before all components are synthesized.

##### `removeTask` <a name="removeTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask"></a>

```typescript
public removeTask(name: string): Task
```

Removes a task from a project.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.removeTask.parameter.name"></a>

- *Type:* string

The name of the task to remove.

---

##### `runTaskCommand` <a name="runTaskCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand"></a>

```typescript
public runTaskCommand(task: Task): string
```

Returns the shell command to execute in order to run a task.

By default, this is `npx projen@<version> <task>`

###### `task`<sup>Required</sup> <a name="task" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.runTaskCommand.parameter.task"></a>

- *Type:* projen.Task

The task for which the command is required.

---

##### `synth` <a name="synth" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.synth"></a>

```typescript
public synth(): void
```

Synthesize all project files into `outdir`.

1. Call "this.preSynthesize()"
2. Delete all generated files
3. Synthesize all sub-projects
4. Synthesize all components of this project
5. Call "postSynthesize()" for all components of this project
6. Call "this.postSynthesize()"

##### `tryFindFile` <a name="tryFindFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile"></a>

```typescript
public tryFindFile(filePath: string): FileBase
```

Finds a file at the specified relative path within this project and all its subprojects.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindFile.parameter.filePath"></a>

- *Type:* string

The file path.

If this path is relative, it will be resolved
from the root of _this_ project.

---

##### ~~`tryFindJsonFile`~~ <a name="tryFindJsonFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile"></a>

```typescript
public tryFindJsonFile(filePath: string): JsonFile
```

Finds a json file by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindJsonFile.parameter.filePath"></a>

- *Type:* string

The file path.

---

##### `tryFindObjectFile` <a name="tryFindObjectFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile"></a>

```typescript
public tryFindObjectFile(filePath: string): ObjectFile
```

Finds an object file (like JsonFile, YamlFile, etc.) by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryFindObjectFile.parameter.filePath"></a>

- *Type:* string

The file path.

---

##### `tryRemoveFile` <a name="tryRemoveFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile"></a>

```typescript
public tryRemoveFile(filePath: string): FileBase
```

Finds a file at the specified relative path within this project and removes it.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.tryRemoveFile.parameter.filePath"></a>

- *Type:* string

The file path.

If this path is relative, it will be
resolved from the root of _this_ project.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.buildTask">buildTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.commitGenerated">commitGenerated</a></code> | <code>boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.compileTask">compileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.components">components</a></code> | <code>projen.Component[]</code> | Returns all the components within this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.deps">deps</a></code> | <code>projen.Dependencies</code> | Project dependencies. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.ejected">ejected</a></code> | <code>boolean</code> | Whether or not the project is being ejected. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.files">files</a></code> | <code>projen.FileBase[]</code> | All files in this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitattributes">gitattributes</a></code> | <code>projen.GitAttributesFile</code> | The .gitattributes file for this repository. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitignore">gitignore</a></code> | <code>projen.IgnoreFile</code> | .gitignore. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.logger">logger</a></code> | <code>projen.Logger</code> | Logging utilities. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.name">name</a></code> | <code>string</code> | Project name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.outdir">outdir</a></code> | <code>string</code> | Absolute output directory of this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.packageTask">packageTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.postCompileTask">postCompileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.preCompileTask">preCompileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projectBuild">projectBuild</a></code> | <code>projen.ProjectBuild</code> | Manages the build process of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projenCommand">projenCommand</a></code> | <code>string</code> | The command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.root">root</a></code> | <code>projen.Project</code> | The root project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.tasks">tasks</a></code> | <code>projen.Tasks</code> | Project tasks. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.testTask">testTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.defaultTask">defaultTask</a></code> | <code>projen.Task</code> | This is the "default" task, the one that executes "projen". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.initProject">initProject</a></code> | <code>projen.InitProject</code> | The options used when this project is bootstrapped via `projen new`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parent">parent</a></code> | <code>projen.Project</code> | A parent project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.generateTask">generateTask</a></code> | <code>projen.Task</code> | Reference to the task used for generating the final bundled OpenAPI specification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parsedSpecFile">parsedSpecFile</a></code> | <code>string</code> | Name of the final bundled OpenAPI specification. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.openapi">openapi</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition">OpenApiDefinition</a></code> | Reference to the OpenAPI definition component. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.smithy">smithy</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a></code> | Reference to the Smithy definition component. |

---

##### `buildTask`<sup>Required</sup> <a name="buildTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.buildTask"></a>

```typescript
public readonly buildTask: Task;
```

- *Type:* projen.Task

---

##### `commitGenerated`<sup>Required</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.commitGenerated"></a>

```typescript
public readonly commitGenerated: boolean;
```

- *Type:* boolean

Whether to commit the managed files by default.

---

##### `compileTask`<sup>Required</sup> <a name="compileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.compileTask"></a>

```typescript
public readonly compileTask: Task;
```

- *Type:* projen.Task

---

##### `components`<sup>Required</sup> <a name="components" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.components"></a>

```typescript
public readonly components: Component[];
```

- *Type:* projen.Component[]

Returns all the components within this project.

---

##### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.deps"></a>

```typescript
public readonly deps: Dependencies;
```

- *Type:* projen.Dependencies

Project dependencies.

---

##### `ejected`<sup>Required</sup> <a name="ejected" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.ejected"></a>

```typescript
public readonly ejected: boolean;
```

- *Type:* boolean

Whether or not the project is being ejected.

---

##### `files`<sup>Required</sup> <a name="files" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.files"></a>

```typescript
public readonly files: FileBase[];
```

- *Type:* projen.FileBase[]

All files in this project.

---

##### `gitattributes`<sup>Required</sup> <a name="gitattributes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitattributes"></a>

```typescript
public readonly gitattributes: GitAttributesFile;
```

- *Type:* projen.GitAttributesFile

The .gitattributes file for this repository.

---

##### `gitignore`<sup>Required</sup> <a name="gitignore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.gitignore"></a>

```typescript
public readonly gitignore: IgnoreFile;
```

- *Type:* projen.IgnoreFile

.gitignore.

---

##### `logger`<sup>Required</sup> <a name="logger" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.logger"></a>

```typescript
public readonly logger: Logger;
```

- *Type:* projen.Logger

Logging utilities.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string

Project name.

---

##### `outdir`<sup>Required</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.outdir"></a>

```typescript
public readonly outdir: string;
```

- *Type:* string

Absolute output directory of this project.

---

##### `packageTask`<sup>Required</sup> <a name="packageTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.packageTask"></a>

```typescript
public readonly packageTask: Task;
```

- *Type:* projen.Task

---

##### `postCompileTask`<sup>Required</sup> <a name="postCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.postCompileTask"></a>

```typescript
public readonly postCompileTask: Task;
```

- *Type:* projen.Task

---

##### `preCompileTask`<sup>Required</sup> <a name="preCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.preCompileTask"></a>

```typescript
public readonly preCompileTask: Task;
```

- *Type:* projen.Task

---

##### `projectBuild`<sup>Required</sup> <a name="projectBuild" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projectBuild"></a>

```typescript
public readonly projectBuild: ProjectBuild;
```

- *Type:* projen.ProjectBuild

Manages the build process of the project.

---

##### `projenCommand`<sup>Required</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.projenCommand"></a>

```typescript
public readonly projenCommand: string;
```

- *Type:* string

The command to use in order to run the projen CLI.

---

##### `root`<sup>Required</sup> <a name="root" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.root"></a>

```typescript
public readonly root: Project;
```

- *Type:* projen.Project

The root project.

---

##### `tasks`<sup>Required</sup> <a name="tasks" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.tasks"></a>

```typescript
public readonly tasks: Tasks;
```

- *Type:* projen.Tasks

Project tasks.

---

##### `testTask`<sup>Required</sup> <a name="testTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.testTask"></a>

```typescript
public readonly testTask: Task;
```

- *Type:* projen.Task

---

##### `defaultTask`<sup>Optional</sup> <a name="defaultTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.defaultTask"></a>

```typescript
public readonly defaultTask: Task;
```

- *Type:* projen.Task

This is the "default" task, the one that executes "projen".

Undefined if
the project is being ejected.

---

##### `initProject`<sup>Optional</sup> <a name="initProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.initProject"></a>

```typescript
public readonly initProject: InitProject;
```

- *Type:* projen.InitProject

The options used when this project is bootstrapped via `projen new`.

It
includes the original set of options passed to the CLI and also the JSII
FQN of the project type.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parent"></a>

```typescript
public readonly parent: Project;
```

- *Type:* projen.Project

A parent project.

If undefined, this is the root project.

---

##### `generateTask`<sup>Required</sup> <a name="generateTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.generateTask"></a>

```typescript
public readonly generateTask: Task;
```

- *Type:* projen.Task

Reference to the task used for generating the final bundled OpenAPI specification.

---

##### `parsedSpecFile`<sup>Required</sup> <a name="parsedSpecFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.parsedSpecFile"></a>

```typescript
public readonly parsedSpecFile: string;
```

- *Type:* string

Name of the final bundled OpenAPI specification.

---

##### `openapi`<sup>Optional</sup> <a name="openapi" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.openapi"></a>

```typescript
public readonly openapi: OpenApiDefinition;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.OpenApiDefinition">OpenApiDefinition</a>

Reference to the OpenAPI definition component.

Will be defined if the model language is OpenAPI

---

##### `smithy`<sup>Optional</sup> <a name="smithy" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.smithy"></a>

```typescript
public readonly smithy: SmithyDefinition;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.SmithyDefinition">SmithyDefinition</a>

Reference to the Smithy definition component.

Will be defined if the model language is Smithy

---

#### Constants <a name="Constants" id="Constants"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.DEFAULT_TASK">DEFAULT_TASK</a></code> | <code>string</code> | The name of the default task (the task executed when `projen` is run without arguments). |

---

##### `DEFAULT_TASK`<sup>Required</sup> <a name="DEFAULT_TASK" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject.property.DEFAULT_TASK"></a>

```typescript
public readonly DEFAULT_TASK: string;
```

- *Type:* string

The name of the default task (the task executed when `projen` is run without arguments).

Normally
this task should synthesize the project files.

---

### TypeSafeApiProject <a name="TypeSafeApiProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject"></a>

Project for a type-safe API, defined using Smithy or OpenAPI.

Generates a CDK construct to deploy your API, as well as client and server code to help build your API quickly.

#### Initializers <a name="Initializers" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer"></a>

```typescript
import { TypeSafeApiProject } from '@aws-prototyping-sdk/type-safe-api'

new TypeSafeApiProject(options: TypeSafeApiProjectOptions)
```

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.options">options</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions">TypeSafeApiProjectOptions</a></code> | *No description.* |

---

##### `options`<sup>Required</sup> <a name="options" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.Initializer.parameter.options"></a>

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProjectOptions">TypeSafeApiProjectOptions</a>

---

#### Methods <a name="Methods" id="Methods"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup">addExcludeFromCleanup</a></code> | Exclude the matching files from pre-synth cleanup. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore">addGitIgnore</a></code> | Adds a .gitignore pattern. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore">addPackageIgnore</a></code> | Exclude these files from the bundled package. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask">addTask</a></code> | Adds a new task to this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip">addTip</a></code> | Prints a "tip" message during synthesis. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated">annotateGenerated</a></code> | Consider a set of files as "generated". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.postSynthesize">postSynthesize</a></code> | Called after all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.preSynthesize">preSynthesize</a></code> | Called before all components are synthesized. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask">removeTask</a></code> | Removes a task from a project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand">runTaskCommand</a></code> | Returns the shell command to execute in order to run a task. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.synth">synth</a></code> | Synthesize all project files into `outdir`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile">tryFindFile</a></code> | Finds a file at the specified relative path within this project and all its subprojects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile">tryFindJsonFile</a></code> | Finds a json file by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile">tryFindObjectFile</a></code> | Finds an object file (like JsonFile, YamlFile, etc.) by name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile">tryRemoveFile</a></code> | Finds a file at the specified relative path within this project and removes it. |

---

##### `addExcludeFromCleanup` <a name="addExcludeFromCleanup" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup"></a>

```typescript
public addExcludeFromCleanup(globs: string): void
```

Exclude the matching files from pre-synth cleanup.

Can be used when, for example, some
source files include the projen marker and we don't want them to be erased during synth.

###### `globs`<sup>Required</sup> <a name="globs" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addExcludeFromCleanup.parameter.globs"></a>

- *Type:* string

The glob patterns to match.

---

##### `addGitIgnore` <a name="addGitIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore"></a>

```typescript
public addGitIgnore(pattern: string): void
```

Adds a .gitignore pattern.

###### `pattern`<sup>Required</sup> <a name="pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addGitIgnore.parameter.pattern"></a>

- *Type:* string

The glob pattern to ignore.

---

##### `addPackageIgnore` <a name="addPackageIgnore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore"></a>

```typescript
public addPackageIgnore(_pattern: string): void
```

Exclude these files from the bundled package.

Implemented by project types based on the
packaging mechanism. For example, `NodeProject` delegates this to `.npmignore`.

###### `_pattern`<sup>Required</sup> <a name="_pattern" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addPackageIgnore.parameter._pattern"></a>

- *Type:* string

The glob pattern to exclude.

---

##### `addTask` <a name="addTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask"></a>

```typescript
public addTask(name: string, props?: TaskOptions): Task
```

Adds a new task to this project.

This will fail if the project already has
a task with this name.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask.parameter.name"></a>

- *Type:* string

The task name to add.

---

###### `props`<sup>Optional</sup> <a name="props" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTask.parameter.props"></a>

- *Type:* projen.TaskOptions

Task properties.

---

##### ~~`addTip`~~ <a name="addTip" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip"></a>

```typescript
public addTip(message: string): void
```

Prints a "tip" message during synthesis.

###### `message`<sup>Required</sup> <a name="message" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.addTip.parameter.message"></a>

- *Type:* string

The message.

---

##### `annotateGenerated` <a name="annotateGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated"></a>

```typescript
public annotateGenerated(_glob: string): void
```

Consider a set of files as "generated".

This method is implemented by
derived classes and used for example, to add git attributes to tell GitHub
that certain files are generated.

###### `_glob`<sup>Required</sup> <a name="_glob" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.annotateGenerated.parameter._glob"></a>

- *Type:* string

the glob pattern to match (could be a file path).

---

##### `postSynthesize` <a name="postSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.postSynthesize"></a>

```typescript
public postSynthesize(): void
```

Called after all components are synthesized.

Order is *not* guaranteed.

##### `preSynthesize` <a name="preSynthesize" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.preSynthesize"></a>

```typescript
public preSynthesize(): void
```

Called before all components are synthesized.

##### `removeTask` <a name="removeTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask"></a>

```typescript
public removeTask(name: string): Task
```

Removes a task from a project.

###### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.removeTask.parameter.name"></a>

- *Type:* string

The name of the task to remove.

---

##### `runTaskCommand` <a name="runTaskCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand"></a>

```typescript
public runTaskCommand(task: Task): string
```

Returns the shell command to execute in order to run a task.

By default, this is `npx projen@<version> <task>`

###### `task`<sup>Required</sup> <a name="task" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.runTaskCommand.parameter.task"></a>

- *Type:* projen.Task

The task for which the command is required.

---

##### `synth` <a name="synth" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.synth"></a>

```typescript
public synth(): void
```

Synthesize all project files into `outdir`.

1. Call "this.preSynthesize()"
2. Delete all generated files
3. Synthesize all sub-projects
4. Synthesize all components of this project
5. Call "postSynthesize()" for all components of this project
6. Call "this.postSynthesize()"

##### `tryFindFile` <a name="tryFindFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile"></a>

```typescript
public tryFindFile(filePath: string): FileBase
```

Finds a file at the specified relative path within this project and all its subprojects.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindFile.parameter.filePath"></a>

- *Type:* string

The file path.

If this path is relative, it will be resolved
from the root of _this_ project.

---

##### ~~`tryFindJsonFile`~~ <a name="tryFindJsonFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile"></a>

```typescript
public tryFindJsonFile(filePath: string): JsonFile
```

Finds a json file by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindJsonFile.parameter.filePath"></a>

- *Type:* string

The file path.

---

##### `tryFindObjectFile` <a name="tryFindObjectFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile"></a>

```typescript
public tryFindObjectFile(filePath: string): ObjectFile
```

Finds an object file (like JsonFile, YamlFile, etc.) by name.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryFindObjectFile.parameter.filePath"></a>

- *Type:* string

The file path.

---

##### `tryRemoveFile` <a name="tryRemoveFile" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile"></a>

```typescript
public tryRemoveFile(filePath: string): FileBase
```

Finds a file at the specified relative path within this project and removes it.

###### `filePath`<sup>Required</sup> <a name="filePath" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.tryRemoveFile.parameter.filePath"></a>

- *Type:* string

The file path.

If this path is relative, it will be
resolved from the root of _this_ project.

---


#### Properties <a name="Properties" id="Properties"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.buildTask">buildTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.commitGenerated">commitGenerated</a></code> | <code>boolean</code> | Whether to commit the managed files by default. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.compileTask">compileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.components">components</a></code> | <code>projen.Component[]</code> | Returns all the components within this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.deps">deps</a></code> | <code>projen.Dependencies</code> | Project dependencies. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.ejected">ejected</a></code> | <code>boolean</code> | Whether or not the project is being ejected. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.files">files</a></code> | <code>projen.FileBase[]</code> | All files in this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitattributes">gitattributes</a></code> | <code>projen.GitAttributesFile</code> | The .gitattributes file for this repository. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitignore">gitignore</a></code> | <code>projen.IgnoreFile</code> | .gitignore. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.logger">logger</a></code> | <code>projen.Logger</code> | Logging utilities. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.name">name</a></code> | <code>string</code> | Project name. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.outdir">outdir</a></code> | <code>string</code> | Absolute output directory of this project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.packageTask">packageTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.postCompileTask">postCompileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.preCompileTask">preCompileTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projectBuild">projectBuild</a></code> | <code>projen.ProjectBuild</code> | Manages the build process of the project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projenCommand">projenCommand</a></code> | <code>string</code> | The command to use in order to run the projen CLI. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.root">root</a></code> | <code>projen.Project</code> | The root project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.tasks">tasks</a></code> | <code>projen.Tasks</code> | Project tasks. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.testTask">testTask</a></code> | <code>projen.Task</code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.defaultTask">defaultTask</a></code> | <code>projen.Task</code> | This is the "default" task, the one that executes "projen". |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.initProject">initProject</a></code> | <code>projen.InitProject</code> | The options used when this project is bootstrapped via `projen new`. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.parent">parent</a></code> | <code>projen.Project</code> | A parent project. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.infrastructure">infrastructure</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a></code> | Generated infrastructure projects. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.model">model</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a></code> | Project for the api model. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.runtime">runtime</a></code> | <code><a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a></code> | Generated runtime projects. |

---

##### `buildTask`<sup>Required</sup> <a name="buildTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.buildTask"></a>

```typescript
public readonly buildTask: Task;
```

- *Type:* projen.Task

---

##### `commitGenerated`<sup>Required</sup> <a name="commitGenerated" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.commitGenerated"></a>

```typescript
public readonly commitGenerated: boolean;
```

- *Type:* boolean

Whether to commit the managed files by default.

---

##### `compileTask`<sup>Required</sup> <a name="compileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.compileTask"></a>

```typescript
public readonly compileTask: Task;
```

- *Type:* projen.Task

---

##### `components`<sup>Required</sup> <a name="components" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.components"></a>

```typescript
public readonly components: Component[];
```

- *Type:* projen.Component[]

Returns all the components within this project.

---

##### `deps`<sup>Required</sup> <a name="deps" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.deps"></a>

```typescript
public readonly deps: Dependencies;
```

- *Type:* projen.Dependencies

Project dependencies.

---

##### `ejected`<sup>Required</sup> <a name="ejected" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.ejected"></a>

```typescript
public readonly ejected: boolean;
```

- *Type:* boolean

Whether or not the project is being ejected.

---

##### `files`<sup>Required</sup> <a name="files" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.files"></a>

```typescript
public readonly files: FileBase[];
```

- *Type:* projen.FileBase[]

All files in this project.

---

##### `gitattributes`<sup>Required</sup> <a name="gitattributes" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitattributes"></a>

```typescript
public readonly gitattributes: GitAttributesFile;
```

- *Type:* projen.GitAttributesFile

The .gitattributes file for this repository.

---

##### `gitignore`<sup>Required</sup> <a name="gitignore" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.gitignore"></a>

```typescript
public readonly gitignore: IgnoreFile;
```

- *Type:* projen.IgnoreFile

.gitignore.

---

##### `logger`<sup>Required</sup> <a name="logger" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.logger"></a>

```typescript
public readonly logger: Logger;
```

- *Type:* projen.Logger

Logging utilities.

---

##### `name`<sup>Required</sup> <a name="name" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.name"></a>

```typescript
public readonly name: string;
```

- *Type:* string

Project name.

---

##### `outdir`<sup>Required</sup> <a name="outdir" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.outdir"></a>

```typescript
public readonly outdir: string;
```

- *Type:* string

Absolute output directory of this project.

---

##### `packageTask`<sup>Required</sup> <a name="packageTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.packageTask"></a>

```typescript
public readonly packageTask: Task;
```

- *Type:* projen.Task

---

##### `postCompileTask`<sup>Required</sup> <a name="postCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.postCompileTask"></a>

```typescript
public readonly postCompileTask: Task;
```

- *Type:* projen.Task

---

##### `preCompileTask`<sup>Required</sup> <a name="preCompileTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.preCompileTask"></a>

```typescript
public readonly preCompileTask: Task;
```

- *Type:* projen.Task

---

##### `projectBuild`<sup>Required</sup> <a name="projectBuild" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projectBuild"></a>

```typescript
public readonly projectBuild: ProjectBuild;
```

- *Type:* projen.ProjectBuild

Manages the build process of the project.

---

##### `projenCommand`<sup>Required</sup> <a name="projenCommand" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.projenCommand"></a>

```typescript
public readonly projenCommand: string;
```

- *Type:* string

The command to use in order to run the projen CLI.

---

##### `root`<sup>Required</sup> <a name="root" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.root"></a>

```typescript
public readonly root: Project;
```

- *Type:* projen.Project

The root project.

---

##### `tasks`<sup>Required</sup> <a name="tasks" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.tasks"></a>

```typescript
public readonly tasks: Tasks;
```

- *Type:* projen.Tasks

Project tasks.

---

##### `testTask`<sup>Required</sup> <a name="testTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.testTask"></a>

```typescript
public readonly testTask: Task;
```

- *Type:* projen.Task

---

##### `defaultTask`<sup>Optional</sup> <a name="defaultTask" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.defaultTask"></a>

```typescript
public readonly defaultTask: Task;
```

- *Type:* projen.Task

This is the "default" task, the one that executes "projen".

Undefined if
the project is being ejected.

---

##### `initProject`<sup>Optional</sup> <a name="initProject" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.initProject"></a>

```typescript
public readonly initProject: InitProject;
```

- *Type:* projen.InitProject

The options used when this project is bootstrapped via `projen new`.

It
includes the original set of options passed to the CLI and also the JSII
FQN of the project type.

---

##### `parent`<sup>Optional</sup> <a name="parent" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.parent"></a>

```typescript
public readonly parent: Project;
```

- *Type:* projen.Project

A parent project.

If undefined, this is the root project.

---

##### `infrastructure`<sup>Required</sup> <a name="infrastructure" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.infrastructure"></a>

```typescript
public readonly infrastructure: GeneratedCodeProjects;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a>

Generated infrastructure projects.

Only the property corresponding to `infrastructure.language` will be defined.

---

##### `model`<sup>Required</sup> <a name="model" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.model"></a>

```typescript
public readonly model: TypeSafeApiModelProject;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiModelProject">TypeSafeApiModelProject</a>

Project for the api model.

---

##### `runtime`<sup>Required</sup> <a name="runtime" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.runtime"></a>

```typescript
public readonly runtime: GeneratedCodeProjects;
```

- *Type:* <a href="#@aws-prototyping-sdk/type-safe-api.GeneratedCodeProjects">GeneratedCodeProjects</a>

Generated runtime projects.

When `runtime.languages` includes the corresponding language, the project can be
assumed to be defined.

---

#### Constants <a name="Constants" id="Constants"></a>

| **Name** | **Type** | **Description** |
| --- | --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.DEFAULT_TASK">DEFAULT_TASK</a></code> | <code>string</code> | The name of the default task (the task executed when `projen` is run without arguments). |

---

##### `DEFAULT_TASK`<sup>Required</sup> <a name="DEFAULT_TASK" id="@aws-prototyping-sdk/type-safe-api.TypeSafeApiProject.property.DEFAULT_TASK"></a>

```typescript
public readonly DEFAULT_TASK: string;
```

- *Type:* string

The name of the default task (the task executed when `projen` is run without arguments).

Normally
this task should synthesize the project files.

---


## Enums <a name="Enums" id="Enums"></a>

### CustomAuthorizerType <a name="CustomAuthorizerType" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType"></a>

The type of custom authorizer.

> [https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type)

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.TOKEN">TOKEN</a></code> | A custom authorizer that uses a Lambda function. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.REQUEST">REQUEST</a></code> | An authorizer that uses a Lambda function using incoming request parameters. |

---

##### `TOKEN` <a name="TOKEN" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.TOKEN"></a>

A custom authorizer that uses a Lambda function.

---


##### `REQUEST` <a name="REQUEST" id="@aws-prototyping-sdk/type-safe-api.CustomAuthorizerType.REQUEST"></a>

An authorizer that uses a Lambda function using incoming request parameters.

---


### DocumentationFormat <a name="DocumentationFormat" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat"></a>

Formats for documentation generation.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML_REDOC">HTML_REDOC</a></code> | HTML Documentation generated by redoc. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML2">HTML2</a></code> | OpenAPI Generator 'html2' documentation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.MARKDOWN">MARKDOWN</a></code> | OpenAPI Generator 'markdown' documentation. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.DocumentationFormat.PLANTUML">PLANTUML</a></code> | OpenAPI Generator 'plantuml' documentation. |

---

##### `HTML_REDOC` <a name="HTML_REDOC" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML_REDOC"></a>

HTML Documentation generated by redoc.

> [https://github.com/Redocly/redoc](https://github.com/Redocly/redoc)

---


##### `HTML2` <a name="HTML2" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.HTML2"></a>

OpenAPI Generator 'html2' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/html2.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/html2.md)

---


##### `MARKDOWN` <a name="MARKDOWN" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.MARKDOWN"></a>

OpenAPI Generator 'markdown' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/markdown.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/markdown.md)

---


##### `PLANTUML` <a name="PLANTUML" id="@aws-prototyping-sdk/type-safe-api.DocumentationFormat.PLANTUML"></a>

OpenAPI Generator 'plantuml' documentation.

> [https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/plantuml.md](https://github.com/OpenAPITools/openapi-generator/blob/master/docs/generators/plantuml.md)

---


### Language <a name="Language" id="@aws-prototyping-sdk/type-safe-api.Language"></a>

Supported languages for code generation.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.TYPESCRIPT">TYPESCRIPT</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.PYTHON">PYTHON</a></code> | *No description.* |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.Language.JAVA">JAVA</a></code> | *No description.* |

---

##### `TYPESCRIPT` <a name="TYPESCRIPT" id="@aws-prototyping-sdk/type-safe-api.Language.TYPESCRIPT"></a>

---


##### `PYTHON` <a name="PYTHON" id="@aws-prototyping-sdk/type-safe-api.Language.PYTHON"></a>

---


##### `JAVA` <a name="JAVA" id="@aws-prototyping-sdk/type-safe-api.Language.JAVA"></a>

---


### ModelLanguage <a name="ModelLanguage" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage"></a>

The model definition language.

#### Members <a name="Members" id="Members"></a>

| **Name** | **Description** |
| --- | --- |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage.SMITHY">SMITHY</a></code> | Smithy. |
| <code><a href="#@aws-prototyping-sdk/type-safe-api.ModelLanguage.OPENAPI">OPENAPI</a></code> | OpenAPI. |

---

##### `SMITHY` <a name="SMITHY" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage.SMITHY"></a>

Smithy.

> [https://smithy.io/2.0/](https://smithy.io/2.0/)

---


##### `OPENAPI` <a name="OPENAPI" id="@aws-prototyping-sdk/type-safe-api.ModelLanguage.OPENAPI"></a>

OpenAPI.

> [https://www.openapis.org/](https://www.openapis.org/)

---

